@@ -1,5 +1,6 @@
-/* elgamal.c  -  ElGamal Public Key encryption
- *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
+/* elgamal.c  -  elgamal Public Key encryption
+ * Copyright (C) 1998, 2000, 2001, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * For a description of the algorithm, see:
  *   Bruce Schneier: Applied Cryptography. John Wiley & Sons, 1996.
@@ -26,7 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "g10lib.h"
+#include "util.h"
 #include "mpi.h"
 #include "cipher.h"
 #include "elgamal.h"
@@ -47,13 +48,11 @@ typedef struct {
 
 
 static void test_keys( ELG_secret_key *sk, unsigned nbits );
-static MPI gen_k( MPI p );
+static MPI gen_k( MPI p, int small_k );
 static void generate( ELG_secret_key *sk, unsigned nbits, MPI **factors );
 static int  check_secret_key( ELG_secret_key *sk );
-static void encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey );
+static void do_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey );
 static void decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey );
-static void sign(MPI a, MPI b, MPI input, ELG_secret_key *skey);
-static int  verify(MPI a, MPI b, MPI input, ELG_public_key *pkey);
 
 
 static void (*progress_cb) ( void *, int );
@@ -78,7 +77,7 @@ progress( int c )
 
 
 /****************
- * Michael Wiener's table on subgroup sizes to match field sizes
+ * Michael Wiener's table about subgroup sizes to match field sizes
  * (floating around somewhere - Fixme: need a reference)
  */
 static unsigned int
@@ -130,16 +129,16 @@ test_keys( ELG_secret_key *sk, unsigned nbits )
     pk.g = sk->g;
     pk.y = sk->y;
 
-    gcry_mpi_randomize( test, nbits, GCRY_WEAK_RANDOM );
+    /*mpi_set_bytes( test, nbits, get_random_byte, 0 );*/
+    {	char *p = get_random_bits( nbits, 0, 0 );
+	mpi_set_buffer( test, p, (nbits+7)/8, 0 );
+	m_free(p);
+    }
 
-    encrypt( out1_a, out1_b, test, &pk );
+    do_encrypt( out1_a, out1_b, test, &pk );
     decrypt( out2, out1_a, out1_b, sk );
     if( mpi_cmp( test, out2 ) )
-	log_fatal("ElGamal operation: encrypt, decrypt failed\n");
-
-    sign( out1_a, out1_b, test, sk );
-    if( !verify( out1_a, out1_b, test, &pk ) )
-	log_fatal("ElGamal operation: sign, verify failed\n");
+	log_fatal("Elgamal operation: encrypt, decrypt failed\n");
 
     mpi_free( test );
     mpi_free( out1_a );
@@ -149,53 +148,56 @@ test_keys( ELG_secret_key *sk, unsigned nbits )
 
 
 /****************
- * generate a random secret exponent k from prime p, so
- * that k is relatively prime to p-1
+ * Generate a random secret exponent k from prime p, so that k is
+ * relatively prime to p-1.  With SMALL_K set, k will be selected for
+ * better encryption performance - this must never bee used signing!
  */
 static MPI
-gen_k( MPI p )
+gen_k( MPI p, int small_k )
 {
     MPI k = mpi_alloc_secure( 0 );
     MPI temp = mpi_alloc( mpi_get_nlimbs(p) );
     MPI p_1 = mpi_copy(p);
     unsigned int orig_nbits = mpi_get_nbits(p);
-    unsigned int nbits, nbytes;
+    unsigned int nbits;
+    unsigned int nbytes;
     char *rndbuf = NULL;
 
-    /* IMO using a k much lesser than p is sufficient and it greatly
-     * improves the encryption performance.  We use Wiener's table
-     * and add a large safety margin.
-     */
-    nbits = wiener_map( orig_nbits ) * 3 / 2;
-    if( nbits >= orig_nbits )
-	BUG();
+    if (small_k)
+      {
+        /* Using a k much lesser than p is sufficient for encryption and
+         * it greatly improves the encryption performance.  We use
+         * Wiener's table and add a large safety margin.
+         */
+        nbits = wiener_map( orig_nbits ) * 3 / 2;
+        if( nbits >= orig_nbits )
+          BUG();
+      }
+    else
+      nbits = orig_nbits;
 
     nbytes = (nbits+7)/8;
     if( DBG_CIPHER )
-	log_debug("choosing a random k ");
+	log_debug("choosing a random k of %u bits", nbits);
     mpi_sub_ui( p_1, p, 1);
     for(;;) {
 	if( !rndbuf || nbits < 32 ) {
-	    g10_free(rndbuf);
-	    rndbuf = gcry_random_bytes_secure( nbytes, GCRY_STRONG_RANDOM );
+	    m_free(rndbuf);
+	    rndbuf = get_random_bits( nbits, 1, 1 );
 	}
-	else { /* change only some of the higher bits */
-	    /* we could improve this by directly requesting more memory
-	     * at the first call to get_random_bytes() and use this the here
+	else { /* Change only some of the higher bits. */
+	    /* We could impprove this by directly requesting more memory
+	     * at the first call to get_random_bits() and use this the here
 	     * maybe it is easier to do this directly in random.c
 	     * Anyway, it is highly inlikely that we will ever reach this code
 	     */
-	    char *pp = gcry_random_bytes_secure( 4, GCRY_STRONG_RANDOM );
-	    memcpy( rndbuf, pp, 4 );
-	    g10_free(pp);
-	    log_debug("gen_k: tsss, never expected to reach this\n");
+	    char *pp = get_random_bits( 32, 1, 1 );
+	    memcpy( rndbuf,pp, 4 );
+	    m_free(pp);
 	}
 	mpi_set_buffer( k, rndbuf, nbytes, 0 );
 
 	for(;;) {
-	    /* Hmm, actually we don't need this step here
-	     * because we use k much smaller than p - we do it anyway
-	     * just in case the keep on adding a one to k ;) */
 	    if( !(mpi_cmp( k, p_1 ) < 0) ) {  /* check: k < (p-1) */
 		if( DBG_CIPHER )
 		    progress('+');
@@ -214,7 +216,7 @@ gen_k( MPI p )
 	}
     }
   found:
-    g10_free(rndbuf);
+    m_free(rndbuf);
     if( DBG_CIPHER )
 	progress('\n');
     mpi_free(p_1);
@@ -257,10 +259,13 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
      * secret part.  The prime is public and may be shared anyway,
      * so a random generator level of 1 is used for the prime.
      *
-     * I don't see a reason to have a x of about the same size
-     * as the p.  It should be sufficient to have one about the size
-     * of q or the later used k plus a large safety margin. Decryption
-     * will be much faster with such an x.
+     * I don't see a reason to have a x of about the same size as the
+     * p.  It should be sufficient to have one about the size of q or
+     * the later used k plus a large safety margin. Decryption will be
+     * much faster with such an x.  Note that this is not optimal for
+     * signing keys becuase it makes an attack using accidential small
+     * K values even easier.  Well, one should not use ElGamal signing
+     * anyway.
      */
     xbits = qbits * 3 / 2;
     if( xbits >= nbits )
@@ -274,28 +279,24 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
 	    progress('.');
 	if( rndbuf ) { /* change only some of the higher bits */
 	    if( xbits < 16 ) {/* should never happen ... */
-		g10_free(rndbuf);
-		rndbuf = gcry_random_bytes_secure( (xbits+7)/8,
-						   GCRY_VERY_STRONG_RANDOM );
+		m_free(rndbuf);
+		rndbuf = get_random_bits( xbits, 2, 1 );
 	    }
 	    else {
-		char *r = gcry_random_bytes_secure( 2,
-						   GCRY_VERY_STRONG_RANDOM );
-		memcpy(rndbuf, r, 2 );
-		g10_free(r);
+		char *r = get_random_bits( 16, 2, 1 );
+		memcpy(rndbuf, r, 16/8 );
+		m_free(r);
 	    }
 	}
-	else {
-	    rndbuf = gcry_random_bytes_secure( (xbits+7)/8,
-					       GCRY_VERY_STRONG_RANDOM );
-	}
+	else
+	    rndbuf = get_random_bits( xbits, 2, 1 );
 	mpi_set_buffer( x, rndbuf, (xbits+7)/8, 0 );
 	mpi_clear_highbit( x, xbits+1 );
     } while( !( mpi_cmp_ui( x, 0 )>0 && mpi_cmp( x, p_min1 )<0 ) );
-    g10_free(rndbuf);
+    m_free(rndbuf);
 
     y = mpi_alloc(nbits/BITS_PER_MPI_LIMB);
-    gcry_mpi_powm( y, g, x, p );
+    mpi_powm( y, g, x, p );
 
     if( DBG_CIPHER ) {
 	progress('\n');
@@ -329,7 +330,7 @@ check_secret_key( ELG_secret_key *sk )
     int rc;
     MPI y = mpi_alloc( mpi_get_nlimbs(sk->y) );
 
-    gcry_mpi_powm( y, sk->g, sk->x, sk->p );
+    mpi_powm( y, sk->g, sk->x, sk->p );
     rc = !mpi_cmp( y, sk->y );
     mpi_free( y );
     return rc;
@@ -337,7 +338,7 @@ check_secret_key( ELG_secret_key *sk )
 
 
 static void
-encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
+do_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
 {
     MPI k;
 
@@ -346,16 +347,16 @@ encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
      * error code.
      */
 
-    k = gen_k( pkey->p );
-    gcry_mpi_powm( a, pkey->g, k, pkey->p );
+    k = gen_k( pkey->p, 1 );
+    mpi_powm( a, pkey->g, k, pkey->p );
     /* b = (y^k * input) mod p
      *	 = ((y^k mod p) * (input mod p)) mod p
      * and because input is < p
      *	 = ((y^k mod p) * input) mod p
      */
-    gcry_mpi_powm( b, pkey->y, k, pkey->p );
+    mpi_powm( b, pkey->y, k, pkey->p );
     mpi_mulm( b, b, input, pkey->p );
-  #if 0
+#if 0
     if( DBG_CIPHER ) {
 	log_mpidump("elg encrypted y= ", pkey->y);
 	log_mpidump("elg encrypted p= ", pkey->p);
@@ -364,23 +365,21 @@ encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
 	log_mpidump("elg encrypted a= ", a);
 	log_mpidump("elg encrypted b= ", b);
     }
-  #endif
+#endif
     mpi_free(k);
 }
 
 
-
-
 static void
 decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey )
 {
     MPI t1 = mpi_alloc_secure( mpi_get_nlimbs( skey->p ) );
 
     /* output = b/(a^x) mod p */
-    gcry_mpi_powm( t1, a, skey->x, skey->p );
+    mpi_powm( t1, a, skey->x, skey->p );
     mpi_invm( t1, t1, skey->p );
     mpi_mulm( output, b, t1, skey->p );
-  #if 0
+#if 0
     if( DBG_CIPHER ) {
 	log_mpidump("elg decrypted x= ", skey->x);
 	log_mpidump("elg decrypted p= ", skey->p);
@@ -388,112 +387,11 @@ decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey )
 	log_mpidump("elg decrypted b= ", b);
 	log_mpidump("elg decrypted M= ", output);
     }
-  #endif
+#endif
     mpi_free(t1);
 }
 
 
-/****************
- * Make an Elgamal signature out of INPUT
- */
-
-static void
-sign(MPI a, MPI b, MPI input, ELG_secret_key *skey )
-{
-    MPI k;
-    MPI t   = mpi_alloc( mpi_get_nlimbs(a) );
-    MPI inv = mpi_alloc( mpi_get_nlimbs(a) );
-    MPI p_1 = mpi_copy(skey->p);
-
-   /*
-    * b = (t * inv) mod (p-1)
-    * b = (t * inv(k,(p-1),(p-1)) mod (p-1)
-    * b = (((M-x*a) mod (p-1)) * inv(k,(p-1),(p-1))) mod (p-1)
-    *
-    */
-    mpi_sub_ui(p_1, p_1, 1);
-    k = gen_k( skey->p );
-    gcry_mpi_powm( a, skey->g, k, skey->p );
-    mpi_mul(t, skey->x, a );
-    mpi_subm(t, input, t, p_1 );
-    mpi_invm(inv, k, p_1 );
-    mpi_mulm(b, t, inv, p_1 );
-
-  #if 0
-    if( DBG_CIPHER ) {
-	log_mpidump("elg sign p= ", skey->p);
-	log_mpidump("elg sign g= ", skey->g);
-	log_mpidump("elg sign y= ", skey->y);
-	log_mpidump("elg sign x= ", skey->x);
-	log_mpidump("elg sign k= ", k);
-	log_mpidump("elg sign M= ", input);
-	log_mpidump("elg sign a= ", a);
-	log_mpidump("elg sign b= ", b);
-    }
-  #endif
-    mpi_free(k);
-    mpi_free(t);
-    mpi_free(inv);
-    mpi_free(p_1);
-}
-
-
-/****************
- * Returns true if the signature composed of A and B is valid.
- */
-static int
-verify(MPI a, MPI b, MPI input, ELG_public_key *pkey )
-{
-    int rc;
-    MPI t1;
-    MPI t2;
-    MPI base[4];
-    MPI exp[4];
-
-    if( !(mpi_cmp_ui( a, 0 ) > 0 && mpi_cmp( a, pkey->p ) < 0) )
-	return 0; /* assertion	0 < a < p  failed */
-
-    t1 = mpi_alloc( mpi_get_nlimbs(a) );
-    t2 = mpi_alloc( mpi_get_nlimbs(a) );
-
-  #if 0
-    /* t1 = (y^a mod p) * (a^b mod p) mod p */
-    gcry_mpi_powm( t1, pkey->y, a, pkey->p );
-    gcry_mpi_powm( t2, a, b, pkey->p );
-    mpi_mulm( t1, t1, t2, pkey->p );
-
-    /* t2 = g ^ input mod p */
-    gcry_mpi_powm( t2, pkey->g, input, pkey->p );
-
-    rc = !mpi_cmp( t1, t2 );
-  #elif 0
-    /* t1 = (y^a mod p) * (a^b mod p) mod p */
-    base[0] = pkey->y; exp[0] = a;
-    base[1] = a;       exp[1] = b;
-    base[2] = NULL;    exp[2] = NULL;
-    mpi_mulpowm( t1, base, exp, pkey->p );
-
-    /* t2 = g ^ input mod p */
-    gcry_mpi_powm( t2, pkey->g, input, pkey->p );
-
-    rc = !mpi_cmp( t1, t2 );
-  #else
-    /* t1 = g ^ - input * y ^ a * a ^ b  mod p */
-    mpi_invm(t2, pkey->g, pkey->p );
-    base[0] = t2     ; exp[0] = input;
-    base[1] = pkey->y; exp[1] = a;
-    base[2] = a;       exp[2] = b;
-    base[3] = NULL;    exp[3] = NULL;
-    mpi_mulpowm( t1, base, exp, pkey->p );
-    rc = !mpi_cmp_ui( t1, 1 );
-
-  #endif
-
-    mpi_free(t1);
-    mpi_free(t2);
-    return rc;
-}
-
 /*********************************************
  **************  interface  ******************
  *********************************************/
@@ -504,7 +402,7 @@ elg_generate( int algo, unsigned nbits, MPI *skey, MPI **retfactors )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
+	return G10ERR_PUBKEY_ALGO;
 
     generate( &sk, nbits, retfactors );
     skey[0] = sk.p;
@@ -521,38 +419,37 @@ elg_check_secret_key( int algo, MPI *skey )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
+	return G10ERR_PUBKEY_ALGO;
     if( !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return GCRYERR_BAD_MPI;
+	return G10ERR_BAD_MPI;
 
     sk.p = skey[0];
     sk.g = skey[1];
     sk.y = skey[2];
     sk.x = skey[3];
     if( !check_secret_key( &sk ) )
-	return GCRYERR_BAD_SECRET_KEY;
+	return G10ERR_BAD_SECKEY;
 
     return 0;
 }
 
 
-
 int
 elg_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey )
 {
     ELG_public_key pk;
 
     if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
+	return G10ERR_PUBKEY_ALGO;
     if( !data || !pkey[0] || !pkey[1] || !pkey[2] )
-	return GCRYERR_BAD_MPI;
+	return G10ERR_BAD_MPI;
 
     pk.p = pkey[0];
     pk.g = pkey[1];
     pk.y = pkey[2];
     resarr[0] = mpi_alloc( mpi_get_nlimbs( pk.p ) );
     resarr[1] = mpi_alloc( mpi_get_nlimbs( pk.p ) );
-    encrypt( resarr[0], resarr[1], data, &pk );
+    do_encrypt( resarr[0], resarr[1], data, &pk );
     return 0;
 }
 
@@ -562,10 +459,10 @@ elg_decrypt( int algo, MPI *result, MPI *data, MPI *skey )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
+	return G10ERR_PUBKEY_ALGO;
     if( !data[0] || !data[1]
 	|| !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return GCRYERR_BAD_MPI;
+	return G10ERR_BAD_MPI;
 
     sk.p = skey[0];
     sk.g = skey[1];
@@ -576,47 +473,6 @@ elg_decrypt( int algo, MPI *result, MPI *data, MPI *skey )
     return 0;
 }
 
-int
-elg_sign( int algo, MPI *resarr, MPI data, MPI *skey )
-{
-    ELG_secret_key sk;
-
-    if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
-    if( !data || !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return GCRYERR_BAD_MPI;
-
-    sk.p = skey[0];
-    sk.g = skey[1];
-    sk.y = skey[2];
-    sk.x = skey[3];
-    resarr[0] = mpi_alloc( mpi_get_nlimbs( sk.p ) );
-    resarr[1] = mpi_alloc( mpi_get_nlimbs( sk.p ) );
-    sign( resarr[0], resarr[1], data, &sk );
-    return 0;
-}
-
-int
-elg_verify( int algo, MPI hash, MPI *data, MPI *pkey,
-		    int (*cmp)(void *, MPI), void *opaquev )
-{
-    ELG_public_key pk;
-
-    if( !is_ELGAMAL(algo) )
-	return GCRYERR_INV_PK_ALGO;
-    if( !data[0] || !data[1] || !hash
-	|| !pkey[0] || !pkey[1] || !pkey[2] )
-	return GCRYERR_BAD_MPI;
-
-    pk.p = pkey[0];
-    pk.g = pkey[1];
-    pk.y = pkey[2];
-    if( !verify( data[0], data[1], hash, &pk ) )
-	return GCRYERR_BAD_SIGNATURE;
-    return 0;
-}
-
-
 
 unsigned int
 elg_get_nbits( int algo, MPI *pkey )
@@ -634,9 +490,6 @@ elg_get_nbits( int algo, MPI *pkey )
  *	    the ALGO is invalid.
  * Usage: Bit 0 set : allows signing
  *	      1 set : allows encryption
- * NOTE: This function allows signing also for ELG-E, which is not
- * okay but a bad hack to allow to work with old gpg keys. The real check
- * is done in the gnupg ocde depending on the packet version.
  */
 const char *
 elg_get_info( int algo, int *npkey, int *nskey, int *nenc, int *nsig,
@@ -648,14 +501,9 @@ elg_get_info( int algo, int *npkey, int *nskey, int *nenc, int *nsig,
     *nsig = 2;
 
     switch( algo ) {
-      case GCRY_PK_ELG:
-	*use = GCRY_PK_USAGE_SIGN|GCRY_PK_USAGE_ENCR;
-	return "ELG";
-      case GCRY_PK_ELG_E:
-	*use = GCRY_PK_USAGE_SIGN|GCRY_PK_USAGE_ENCR;
+      case PUBKEY_ALGO_ELGAMAL_E:
+	*use = PUBKEY_USAGE_ENC;
 	return "ELG-E";
       default: *use = 0; return NULL;
     }
 }
-
-
