@@ -117,7 +117,8 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	;
     n--;
     if( !n || (mode==1 && n < 2) )
-	log_fatal("can't gen prime with pbits=%u qbits=%u\n", pbits, qbits );
+	log_fatal(_("can't gen prime with pbits=%u qbits=%u\n"),
+                  pbits, qbits );
     if( mode == 1 ) {
 	n--;
 	fbits = (pbits - 2*req_qbits -1) / n;
@@ -293,7 +294,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 
 
 static MPI
-gen_prime( unsigned  nbits, int secret, int randomlevel )
+gen_prime( unsigned int nbits, int secret, int randomlevel )
 {
     unsigned  nlimbs;
     MPI prime, ptest, pminus1, val_2, val_3, result;
@@ -461,15 +462,16 @@ is_prime( MPI n, int steps, int *count )
 	    mpi_set_ui( x, 2 );
 	}
 	else {
-	    /*mpi_set_bytes( x, nbits-1, get_random_byte, 0 );*/
-	    {	char *p = get_random_bits( nbits, 0, 0 );
-		mpi_set_buffer( x, p, (nbits+7)/8, 0 );
-		m_free(p);
-	    }
-	    /* make sure that the number is smaller than the prime
-	     * and keep the randomness of the high bit */
+            char *p;
+            
+            p = get_random_bits( nbits, 0, 0 );
+            mpi_set_buffer( x, p, (nbits+7)/8, 0 );
+            m_free(p);
+
+	    /* Make sure that the number is smaller than the prime
+	     * and keep the randomness of the high bit.  */
 	    if( mpi_test_bit( x, nbits-2 ) ) {
-		mpi_set_highbit( x, nbits-2 ); /* clear all higher bits */
+		mpi_set_highbit( x, nbits-2 ); /* Clear all higher bits */
 	    }
 	    else {
 		mpi_set_highbit( x, nbits-2 );
