@@ -1,5 +1,5 @@
 /* rndlinux.c  -  raw random number for OSes with /dev/random
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -20,6 +20,9 @@
 
 
 #include <config.h>
+
+#ifdef USE_RNDLINUX
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -28,25 +31,25 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_GETTIMEOFDAY
-  #include <sys/times.h>
+#include <sys/times.h>
 #endif
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 #if 0
-  #ifdef HAVE_LINUX_RANDOM_H
-    #include <sys/ioctl.h>
-    #include <asm/types.h>
-    #include <linux/random.h>
-  #endif
+#include <sys/ioctl.h>
+#include <asm/types.h>
+#include <linux/random.h>
 #endif
 #include "types.h"
-#include "g10lib.h"
-#include "dynload.h"
+#include "util.h"
+#include "ttyio.h"
+#include "algorithms.h"
+
+#include "i18n.h"
 
 static int open_device( const char *name, int minor );
-static int gather_random( void (*add)(const void*, size_t, int), int requester,
-					  size_t length, int level );
+
 
 #if 0
 #ifdef HAVE_DEV_RANDOM_IOCTL
@@ -63,7 +66,7 @@ get_entropy_count( int fd )
 #endif
 
 /****************
- * Used to open the Linux and xBSD /dev/random devices
+ * Used to open the /dev/random devices (Linux, xBSD, Solaris (if it exists), ...)
  */
 static int
 open_device( const char *name, int minor )
@@ -76,14 +79,19 @@ open_device( const char *name, int minor )
 	g10_log_fatal("can't open %s: %s\n", name, strerror(errno) );
     if( fstat( fd, &sb ) )
 	g10_log_fatal("stat() off %s failed: %s\n", name, strerror(errno) );
-    if( !S_ISCHR(sb.st_mode) )
-	g10_log_fatal("invalid random device!\n" );
+    /* Don't check device type for better portability */
+    /*  if( (!S_ISCHR(sb.st_mode)) && (!S_ISFIFO(sb.st_mode)) )
+	  g10_log_fatal("invalid random device!\n" ); */
     return fd;
 }
 
 
-static int
-gather_random( void (*add)(const void*, size_t, int), int requester,
+/****************
+ * Note:  Using a level of 0 should never block and better add nothing
+ * to the pool.  This is easy to accomplish with /dev/urandom.
+ */
+int
+rndlinux_gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level )
 {
     static int fd_urandom = -1;
@@ -99,16 +107,18 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	fd = fd_random;
     }
     else {
+	/* this will also be used for elve 0 but by using /dev/urandom
+	 * we can be sure that oit will never block. */
 	if( fd_urandom == -1 )
 	    fd_urandom = open_device( NAME_OF_DEV_URANDOM, 9 );
 	fd = fd_urandom;
     }
 
-  #if 0
-  #ifdef HAVE_DEV_RANDOM_IOCTL
+#if 0
+#ifdef HAVE_DEV_RANDOM_IOCTL
     g10_log_info("entropy count of %d is %lu\n", fd, get_entropy_count(fd) );
-  #endif
-  #endif
+#endif
+#endif
     while( length ) {
 	fd_set rfds;
 	struct timeval tv;
@@ -119,17 +129,17 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	tv.tv_sec = 3;
 	tv.tv_usec = 0;
 	if( !(rc=select(fd+1, &rfds, NULL, NULL, &tv)) ) {
-	    #warning FIXME: Replace fprintf by a callback
 	    if( !warn )
-		fprintf(stderr,
+		tty_printf(
 _("\n"
 "Not enough random bytes available.  Please do some other work to give\n"
-"the OS a chance to collect more entropy! (Need %d more bytes)\n"), length );
+"the OS a chance to collect more entropy! (Need %d more bytes)\n"), (int)length );
 	    warn = 1;
 	    continue;
 	}
 	else if( rc == -1 ) {
-	    fprintf(stderr, "select() error: %s\n", strerror(errno));
+	    tty_printf(
+		       "select() error: %s\n", strerror(errno));
 	    continue;
 	}
 
@@ -146,75 +156,9 @@ _("\n"
 	(*add)( buffer, n, requester );
 	length -= n;
     }
-    memset(buffer, 0, sizeof(buffer) );
+    wipememory(buffer, sizeof(buffer) );
 
     return 0; /* success */
 }
 
-
-
-#ifndef IS_MODULE
-static
-#endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.12 $)";
-
-static struct {
-    int class;
-    int version;
-    void *func;
-} func_table[] = {
-    { 40, 1, gather_random },
-};
-
-
-
-/****************
- * Enumerate the names of the functions together with informations about
- * this function. Set sequence to an integer with a initial value of 0 and
- * do not change it.
- * If what is 0 all kind of functions are returned.
- * Return values: class := class of function:
- *			   10 = message digest algorithm info function
- *			   11 = integer with available md algorithms
- *			   20 = cipher algorithm info function
- *			   21 = integer with available cipher algorithms
- *			   30 = public key algorithm info function
- *			   31 = integer with available pubkey algorithms
- *			   40 = get gather_random function
- *			   41 = get fast_random_poll function
- *		  version = interface version of the function/pointer
- *			    (currently this is 1 for all functions)
- */
-
-#ifndef IS_MODULE
-static
-#endif
-void *
-gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
-{
-    void *ret;
-    int i = *sequence;
-
-    do {
-	if ( i >= DIM(func_table) || i < 0 ) {
-	    return NULL;
-	}
-	*class = func_table[i].class;
-	*vers  = func_table[i].version;
-	ret = func_table[i].func;
-	i++;
-    } while ( what && what != *class );
-
-    *sequence = i;
-    return ret;
-}
-
-#ifndef IS_MODULE
-void
-rndlinux_constructor(void)
-{
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
-}
-#endif
-
+#endif /*USE_RNDLINUX*/
