@@ -44,12 +44,13 @@
   code base to be maintained */
 
 
-/* Fixme: We use plain mallocs here beucase it may be used as a module
- * should be changed. */
 
 /* General includes */
 
 #include <config.h>
+
+#ifdef USE_RNDUNIX
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -79,9 +80,10 @@
 #endif				/* __QNX__ */
 #if defined( _AIX ) || defined( __QNX__ )
 #include <sys/select.h>
-#endif				/* _AIX */
+#endif				/* _AIX || __QNX__ */
 #ifndef __QNX__
 #include <sys/shm.h>
+#include <signal.h>
 #include <sys/signal.h>
 #endif				/* __QNX__ */
 #include <sys/stat.h>
@@ -98,10 +100,8 @@
 #include <errno.h>
 
 #include "types.h"  /* for byte and u32 typedefs */
-#ifndef IS_MODULE
-#include "dynload.h"
-#endif
-#include "g10lib.h"
+#include "algorithms.h"
+#include "util.h"
 
 #ifndef EAGAIN
 #define EAGAIN	EWOULDBLOCK
@@ -244,6 +244,7 @@ static struct RI {
     {	"/usr/ucb/ps", "aux", SC(0.3), NULL, 0, 0, 0, 1       },
     {	"/usr/bin/ps", "aux", SC(0.3), NULL, 0, 0, 0, 1       },
     {	"/bin/ps", "aux", SC(0.3), NULL, 0, 0, 0, 0          },
+    {   "/bin/ps", "-A", SC(0.3), NULL, 0, 0, 0, 0           }, /*QNX*/
     {	"/usr/bin/ipcs", "-a", SC(0.5), NULL, 0, 0, 0, 1      },
     {	"/bin/ipcs", "-a", SC(0.5), NULL, 0, 0, 0, 0         },
     /* Unreliable source, depends on system usage */
@@ -292,6 +293,10 @@ static struct RI {
     /* This is a complex and screwball program.  Some systems have things
      * like rX_dmn, x = integer, for RAID systems, but the statistics are
      * pretty dodgy */
+#ifdef __QNXNTO__                                                             
+    { "/bin/pidin", "-F%A%B%c%d%E%I%J%K%m%M%n%N%p%P%S%s%T", SC(0.3),
+             NULL, 0, 0, 0, 0       },
+#endif     
 #if 0
     /* The following aren't enabled since they're somewhat slow and not very
      * unpredictable, however they give an indication of the sort of sources
@@ -316,13 +321,14 @@ typedef struct {
     char data[500];	/* gathered data */
 } GATHER_MSG;
 
+
 #ifndef HAVE_WAITPID
 pid_t
 waitpid(pid_t pid, int *statptr, int options)
 {
-     #ifdef HAVE_WAIT4
+#ifdef HAVE_WAIT4
 	return wait4(pid, statptr, options, NULL);
-     #else
+#else
 	/* If wait4 is also not available, try wait3 for SVR3 variants */
 	/* Less ideal because can't actually request a specific pid */
 	/* For that reason, first check to see if pid is for an */
@@ -338,10 +344,11 @@ waitpid(pid_t pid, int *statptr, int options)
 		    (tmp_pid != -1) && (tmp_pid != 0) && (pid != -1))
 	    ;
 	return tmp_pid;
-     #endif
+#endif
 }
 #endif
 
+
 /* Under SunOS popen() doesn't record the pid of the child process.  When
  * pclose() is called, instead of calling waitpid() for the correct child, it
  * calls wait() repeatedly until the right child is reaped.  The problem is
@@ -382,10 +389,17 @@ my_popen(struct RI *entry)
 
     if (entry->pid == (pid_t) 0) {
 	struct passwd *passwd;
+	int fd;
 
 	/* We are the child.  Make the read side of the pipe be stdout */
 	if (dup2(pipedes[STDOUT_FILENO], STDOUT_FILENO) < 0)
 	    exit(127);
+	/* Connect the other standard handles to the bit bucket. */
+	if ((fd = open ("/dev/null", O_RDWR)) != -1) {
+           dup2 (fd, STDIN_FILENO);
+           dup2 (fd, STDERR_FILENO);
+           close (fd);
+	}
 
 	/* Now that everything is set up, give up our permissions to make
 	 * sure we don't read anything sensitive.  If the getpwnam() fails,
@@ -396,7 +410,7 @@ my_popen(struct RI *entry)
 
 	setuid(gatherer_uid);
 
-	/* Close the pipe descriptors */
+	/* Close the pipe descriptors. */
 	close(pipedes[STDIN_FILENO]);
 	close(pipedes[STDOUT_FILENO]);
 
@@ -478,11 +492,11 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
     int moreSources;
     struct timeval tv;
     fd_set fds;
-  #if defined( __hpux )
+#if defined( __hpux )
     size_t maxFD = 0;
-  #else
+#else
     int maxFD = 0;
-  #endif /* OS-specific brokenness */
+#endif /* OS-specific brokenness */
     int bufPos, i, usefulness = 0;
 
 
@@ -505,11 +519,9 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 	    dataSources[i].pipeFD = fileno(dataSources[i].pipe);
 	    if (dataSources[i].pipeFD > maxFD)
 		maxFD = dataSources[i].pipeFD;
-	  #ifdef O_NONBLOCK /* Ohhh what a hack (used for Atari) */
+#ifdef O_NONBLOCK /* Ohhh what a hack (used for Atari) */
 	    fcntl(dataSources[i].pipeFD, F_SETFL, O_NONBLOCK);
-	  #else
-	    #warning O_NONBLOCK is missing
-	  #endif
+#endif
 	    FD_SET(dataSources[i].pipeFD, &fds);
 	    dataSources[i].length = 0;
 
@@ -536,11 +548,11 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 	tv.tv_sec = 10;
 	tv.tv_usec = 0;
 
-      #if defined( __hpux ) && ( OS_VERSION == 9 )
+#if defined( __hpux ) && ( OS_VERSION == 9 )
 	if (select(maxFD + 1, (int *)&fds, NULL, NULL, &tv) == -1)
-      #else  /*  */
+#else  /*  */
 	if (select(maxFD + 1, &fds, NULL, NULL, &tv) == -1)
-      #endif /* __hpux */
+#endif /* __hpux */
 	    break;
 
 	/* One of the sources has data available, read it into the buffer */
@@ -640,6 +652,13 @@ start_gatherer( int pipefd )
     FILE *dbgfp = NULL;
     int dbgall;
 
+#ifdef ENABLE_SELINUX_HACKS
+    /* We don't allow writing to the log file because this might be
+       sued to corrupt a secured file.  Given that this is used as a
+       library by the ../g10/ code, we can't access the check function
+       from ../g10/misc.c.  */
+    dbgall = 0;
+#else
     {
 	const char *s = getenv("GNUPG_RNDUNIX_DBG");
 	if( s ) {
@@ -652,29 +671,7 @@ start_gatherer( int pipefd )
 	}
 	dbgall = !!getenv("GNUPG_RNDUNIX_DBGALL");
     }
-    /* close all files but the ones we need */
-    {	int nmax, n1, n2, i;
-      #ifdef _SC_OPEN_MAX
-	if( (nmax=sysconf( _SC_OPEN_MAX )) < 0 ) {
-	  #ifdef _POSIX_OPEN_MAX
-	    nmax = _POSIX_OPEN_MAX;
-	  #else
-	    nmax = 20; /* assume a reasonable value */
-	  #endif
-	}
-      #else
-	nmax = 20; /* assume a reasonable value */
-      #endif
-	n1 = fileno( stderr );
-	n2 = dbgfp? fileno( dbgfp ) : -1;
-	for(i=0; i < nmax; i++ ) {
-	    if( i != n1 && i != n2 && i != pipefd )
-		close(i);
-	}
-	errno = 0;
-    }
-
-
+#endif
 
     /* Set up the buffer */
     gather_buffer_size = GATHER_BUFSIZE;
@@ -692,13 +689,45 @@ start_gatherer( int pipefd )
      * return an error, so the read data won't be added to the randomness
      * pool.  There are two types of SIGC(H)LD naming, the SysV SIGCLD and
      * the BSD/Posix SIGCHLD, so we need to handle either possibility */
-  #ifdef SIGCLD
+#ifdef SIGCLD
     signal(SIGCLD, SIG_DFL);
-  #else
+#else
     signal(SIGCHLD, SIG_DFL);
-  #endif
+#endif
+
+    fflush (stderr);
+    /* Arrghh!!  It's Stuart code!! */
+    /* (close all files but the ones we need) */
+    {	int nmax, n1, i;
+#ifdef _SC_OPEN_MAX
+	if( (nmax=sysconf( _SC_OPEN_MAX )) < 0 ) {
+#ifdef _POSIX_OPEN_MAX
+	    nmax = _POSIX_OPEN_MAX;
+#else
+	    nmax = 20; /* assume a reasonable value */
+#endif
+	}
+#else
+	nmax = 20; /* assume a reasonable value */
+#endif
+	{  
+	  int fd;
+	  if ((fd = open ("/dev/null", O_RDWR)) != -1) {
+	    dup2 (fd, STDIN_FILENO);
+	    dup2 (fd, STDOUT_FILENO);
+	    dup2 (fd, STDERR_FILENO);
+	    close (fd);
+	  }
+	}
+	n1 = dbgfp? fileno (dbgfp) : -1;
+	for(i=0; i < nmax; i++ ) {
+	    if (i != STDIN_FILENO && i != STDOUT_FILENO && i != STDERR_FILENO
+		&& i != n1 && i != pipefd )
+	      close(i);
+	}
+	errno = 0;
+    }
 
-    fclose(stderr);		/* Arrghh!!  It's Stuart code!! */
 
     for(;;) {
 	GATHER_MSG msg;
@@ -764,9 +793,9 @@ read_a_msg( int fd, GATHER_MSG *msg )
  * Using a level of 0 should never block and better add nothing
  * to the pool.  So this is just a dummy for this gatherer.
  */
-static int
-gather_random( void (*add)(const void*, size_t, int), int requester,
-	       size_t length, int level )
+int
+rndunix_gather_random( void (*add)(const void*, size_t, int), int requester,
+                       size_t length, int level )
 {
     static pid_t gatherer_pid = 0;
     static int pipedes[2];
@@ -842,70 +871,4 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
     return 0;
 }
 
-
-
-#ifndef IS_MODULE
-static
-#endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.18 $)";
-
-
-static struct {
-    int class;
-    int version;
-    void *func;
-} func_table[] = {
-    { 40, 1, gather_random },
-};
-
-/****************
- * Enumerate the names of the functions together with informations about
- * this function. Set sequence to an integer with a initial value of 0 and
- * do not change it.
- * If what is 0 all kind of functions are returned.
- * Return values: class := class of function:
- *			   10 = message digest algorithm info function
- *			   11 = integer with available md algorithms
- *			   20 = cipher algorithm info function
- *			   21 = integer with available cipher algorithms
- *			   30 = public key algorithm info function
- *			   31 = integer with available pubkey algorithms
- *			   40 = get read_random_source() function
- *			   41 = get fast_random_poll function
- *		  version = interface version of the function/pointer
- *			    (currently this is 1 for all functions)
- */
-
-#ifndef IS_MODULE
-static
-#endif
-void *
-gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
-{
-    void *ret;
-    int i = *sequence;
-
-    do {
-	if ( i >= DIM(func_table) || i < 0 ) {
-	    return NULL;
-	}
-	*class = func_table[i].class;
-	*vers  = func_table[i].version;
-	ret = func_table[i].func;
-	i++;
-    } while ( what && what != *class );
-
-    *sequence = i;
-    return ret;
-}
-
-#ifndef IS_MODULE
-void
-rndunix_constructor(void)
-{
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
-}
-#endif
-
-
+#endif /*USE_RNDUNIX*/
