@@ -1,5 +1,6 @@
 /* armor.c - Armor flter
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+ *               2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -28,7 +29,7 @@
 
 #include "errors.h"
 #include "iobuf.h"
-#include <gcrypt.h>
+#include "memory.h"
 #include "util.h"
 #include "filter.h"
 #include "packet.h"
@@ -37,12 +38,6 @@
 #include "status.h"
 #include "i18n.h"
 
-#ifdef HAVE_DOSISH_SYSTEM
-  #define LF "\r\n"
-#else
-  #define LF "\n"
-#endif
-
 #define MAX_LINELEN 20000
 
 #define CRCINIT 0xB704CE
@@ -118,7 +113,6 @@ static char *tail_strings[] = {
 };
 
 
-
 static void
 initialize(void)
 {
@@ -151,8 +145,9 @@ initialize(void)
 }
 
 /****************
- * Check whether this is an armored file or not
- * See also parse-packet.c for details on this code
+ * Check whether this is an armored file or not See also
+ * parse-packet.c for details on this code For unknown historic
+ * reasons we use a string here but only the first byte will be used.
  * Returns: True if it seems to be armored
  */
 static int
@@ -195,6 +190,7 @@ use_armor_filter( IOBUF a )
     byte buf[1];
     int n;
 
+    /* fixme: there might be a problem with iobuf_peek */
     n = iobuf_peek(a, buf, 1 );
     if( n == -1 )
 	return 0; /* EOF, doesn't matter whether armored or not */
@@ -210,7 +206,7 @@ static void
 invalid_armor(void)
 {
     write_status(STATUS_BADARMOR);
-    gpg_exit(1); /* stop here */
+    g10_exit(1); /* stop here */
 }
 
 
@@ -245,8 +241,12 @@ parse_hash_header( const char *line )
 	    found |= 2;
 	else if( !strncmp( s, "MD5", s2-s ) )
 	    found |= 4;
-	else if( !strncmp( s, "TIGER", s2-s ) )
+	else if( !strncmp( s, "SHA256", s2-s ) )
 	    found |= 8;
+	else if( !strncmp( s, "SHA384", s2-s ) )
+	    found |= 16;
+	else if( !strncmp( s, "SHA512", s2-s ) )
+	    found |= 32;
 	else
 	    return 0;
 	for(; *s2 && (*s2==' ' || *s2 == '\t'); s2++ )
@@ -283,10 +283,25 @@ is_armor_header( byte *line, unsigned len )
 	return -1;
     save_p = p;
     p += 5;
-    if( *p == '\r' )
-	p++;
-    if( *p == '\n' )
+
+    /* Some Windows environments seem to add whitespace to the end of
+       the line, so we strip it here.  This becomes strict if
+       --rfc2440 is set since 2440 reads "The header lines, therefore,
+       MUST start at the beginning of a line, and MUST NOT have text
+       following them on the same line."  It is unclear whether "text"
+       refers to all text or just non-whitespace text. */
+
+    if(RFC2440)
+      {
+	if( *p == '\r' )
+	  p++;
+	if( *p == '\n' )
+	  p++;
+      }
+    else
+      while(*p==' ' || *p=='\r' || *p=='\n' || *p=='\t')
 	p++;
+
     if( *p )
 	return -1; /* garbage after dashes */
     save_c = *save_p; *save_p = 0;
@@ -312,19 +327,19 @@ is_armor_header( byte *line, unsigned len )
  *	 >0: Good header line
  */
 static int
-parse_header_line( armor_filter_context_t *afx, byte *line, unsigned len )
+parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 {
     byte *p;
     int hashes=0;
+    unsigned int len2;
 
-    /* fixme: why this double check?  I think the original code w/o the
-     * second check for an empty line was done from an early draft of
-     * of OpenPGP - or simply very stupid code */
-    if( *line == '\n' || ( len && (*line == '\r' && line[1]=='\n') ) )
-	return 0; /* empty line */
-    len = trim_trailing_ws( line, len );
-    if( !len )
-	return 0; /* WS only same as empty line */
+    len2 = check_trailing_ws( line, len );
+    if( !len2 ) {
+        afx->buffer_pos = len2;  /* (it is not the fine way to do it here) */
+	return 0; /* WS only: same as empty line */
+    }
+    len = len2;
+    line[len2] = 0;
 
     p = strchr( line, ':');
     if( !p || !p[1] ) {
@@ -399,7 +414,7 @@ check_input( armor_filter_context_t *afx, IOBUF a )
 	    if( hdr_line == BEGIN_SIGNED_MSG_IDX ) {
 		if( afx->in_cleartext ) {
 		    log_error(_("nested clear text signatures\n"));
-		    rc = GPGERR_INVALID_ARMOR;
+		    rc = G10ERR_INVALID_ARMOR;
 		}
 		afx->in_cleartext = 1;
 	    }
@@ -429,7 +444,7 @@ check_input( armor_filter_context_t *afx, IOBUF a )
 	i = parse_header_line( afx, line, len );
 	if( i <= 0 ) {
 	    if( i )
-		rc = GPGERR_INVALID_ARMOR;
+		rc = G10ERR_INVALID_ARMOR;
 	    break;
 	}
     }
@@ -449,7 +464,8 @@ check_input( armor_filter_context_t *afx, IOBUF a )
     return rc;
 }
 
-
+#define PARTIAL_CHUNK 512
+#define PARTIAL_POW   9
 
 /****************
  * Fake a literal data packet and wait for the next armor line
@@ -465,19 +481,31 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
     int lastline = 0;
     unsigned maxlen, n;
     byte *p;
+    byte tempbuf[PARTIAL_CHUNK];
+    size_t tempbuf_len=0;
 
-    len = 2;	/* reserve 2 bytes for the length header */
-    size -= 2;	/* and 2 for the terminating header */
-    while( !rc && len < size ) {
+    while( !rc && size-len>=(PARTIAL_CHUNK+1)) {
 	/* copy what we have in the line buffer */
 	if( afx->faked == 1 )
 	    afx->faked++; /* skip the first (empty) line */
-	else {
-	    while( len < size && afx->buffer_pos < afx->buffer_len )
-		buf[len++] = afx->buffer[afx->buffer_pos++];
-	    if( len >= size )
+	else
+	  {
+	    /* It's full, so write this partial chunk */
+	    if(tempbuf_len==PARTIAL_CHUNK)
+	      {
+		buf[len++]=0xE0+PARTIAL_POW;
+		memcpy(&buf[len],tempbuf,PARTIAL_CHUNK);
+		len+=PARTIAL_CHUNK;
+		tempbuf_len=0;
 		continue;
-	}
+	      }
+
+	    while( tempbuf_len < PARTIAL_CHUNK
+		   && afx->buffer_pos < afx->buffer_len )
+	      tempbuf[tempbuf_len++] = afx->buffer[afx->buffer_pos++];
+	    if( tempbuf_len==PARTIAL_CHUNK )
+	      continue;
+	  }
 
 	/* read the next line */
 	maxlen = MAX_LINELEN;
@@ -490,19 +518,68 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	}
 	if( !maxlen )
 	    afx->truncated++;
-	if( !afx->not_dash_escaped ) {
-	    int crlf;
-	    p = afx->buffer;
-	    n = afx->buffer_len;
-	    crlf = n > 1 && p[n-2] == '\r' && p[n-1]=='\n';
+
+	p = afx->buffer;
+	n = afx->buffer_len;
+
+	/* Armor header or dash-escaped line? */
+	if(p[0]=='-')
+	  {
+	    /* 2440bis-10: When reversing dash-escaping, an
+	       implementation MUST strip the string "- " if it occurs
+	       at the beginning of a line, and SHOULD warn on "-" and
+	       any character other than a space at the beginning of a
+	       line.  */
+
+	    if(p[1]==' ' && !afx->not_dash_escaped)
+	      {
+		/* It's a dash-escaped line, so skip over the
+		   escape. */
+		afx->buffer_pos = 2;
+	      }
+	    else if(p[1]=='-' && p[2]=='-' && p[3]=='-' && p[4]=='-')
+	      {
+		/* Five dashes in a row mean it's probably armor
+		   header. */
+		int type = is_armor_header( p, n );
+		if( afx->not_dash_escaped && type != BEGIN_SIGNATURE )
+		  ; /* this is okay */
+		else
+		  {
+		    if( type != BEGIN_SIGNATURE )
+		      {
+			log_info(_("unexpected armor: "));
+			print_string( stderr, p, n, 0 );
+			putc('\n', stderr);
+		      }
+
+		    lastline = 1;
+		    rc = -1;
+		  }
+	      }
+	    else if(!afx->not_dash_escaped)
+	      {
+		/* Bad dash-escaping. */
+		log_info(_("invalid dash escaped line: "));
+		print_string( stderr, p, n, 0 );
+		putc('\n', stderr);
+	      }
+	  }
+
+	/* Now handle the end-of-line canonicalization */
+	if( !afx->not_dash_escaped )
+	  {
+	    int crlf = n > 1 && p[n-2] == '\r' && p[n-1]=='\n';
 
 	    /* PGP2 does not treat a tab as white space character */
-	    afx->buffer_len = trim_trailing_chars( p, n,
-					 afx->pgp2mode ? " \r\n" : " \t\r\n");
+	    afx->buffer_len=
+	      trim_trailing_chars( &p[afx->buffer_pos], n-afx->buffer_pos,
+				   afx->pgp2mode ? " \r\n" : " \t\r\n");
+	    afx->buffer_len+=afx->buffer_pos;
 	    /* the buffer is always allocated with enough space to append
 	     * the removed [CR], LF and a Nul
 	     * The reason for this complicated procedure is to keep at least
-	     * the original tupe of lineending - handling of the removed
+	     * the original type of lineending - handling of the removed
 	     * trailing spaces seems to be impossible in our method
 	     * of faking a packet; either we have to use a temporary file
 	     * or calculate the hash here in this module and somehow find
@@ -510,48 +587,23 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	     * faked packet could do the job).
 	     */
 	    if( crlf )
-		afx->buffer[afx->buffer_len++] = '\r';
+	      afx->buffer[afx->buffer_len++] = '\r';
 	    afx->buffer[afx->buffer_len++] = '\n';
-	    afx->buffer[afx->buffer_len] = 0;
-	}
-	p = afx->buffer;
-	n = afx->buffer_len;
-
-	if( n > 2 && *p == '-' ) {
-	    /* check for dash escaped or armor header */
-	    if( p[1] == ' ' && !afx->not_dash_escaped ) {
-		/* issue a warning if it is not regular encoded */
-		if( p[2] != '-' && !( n > 6 && !memcmp(p+2, "From ", 5))) {
-		    log_info(_("invalid dash escaped line: "));
-		    print_string( stderr, p, n, 0 );
-		    putc('\n', stderr);
-		}
-		afx->buffer_pos = 2; /* skip */
-	    }
-	    else if( n >= 15 &&  p[1] == '-' && p[2] == '-' && p[3] == '-' ) {
-		int type = is_armor_header( p, n );
-		if( afx->not_dash_escaped && type != BEGIN_SIGNATURE )
-		    ; /* this is okay */
-		else {
-		    if( type != BEGIN_SIGNATURE ) {
-			log_info(_("unexpected armor:"));
-			print_string( stderr, p, n, 0 );
-			putc('\n', stderr);
-		    }
-		    lastline = 1;
-		    rc = -1;
-		}
-	    }
-	}
+	    afx->buffer[afx->buffer_len] = '\0';
+	  }
     }
 
-    buf[0] = (len-2) >> 8;
-    buf[1] = (len-2);
     if( lastline ) { /* write last (ending) length header */
-	if( buf[0] || buf[1] ) { /* only if we have some text */
-	    buf[len++] = 0;
-	    buf[len++] = 0;
-	}
+        if(tempbuf_len<192)
+	  buf[len++]=tempbuf_len;
+	else
+	  {
+	    buf[len++]=((tempbuf_len-192)/256) + 192;
+	    buf[len++]=(tempbuf_len-192) % 256;
+	  }
+	memcpy(&buf[len],tempbuf,tempbuf_len);
+	len+=tempbuf_len;
+
 	rc = 0;
 	afx->faked = 0;
 	afx->in_cleartext = 0;
@@ -590,6 +642,15 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 }
 
 
+static int
+invalid_crc(void)
+{
+    if ( opt.ignore_crc_error )
+        return 0;
+    log_inc_errorcount();
+    return G10ERR_INVALID_ARMOR;
+}
+
 
 static int
 radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
@@ -636,9 +697,9 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		if( isxdigit(cc1) && isxdigit(cc2)
 				  && strchr( "=\n\r\t ", cc3 )) {
 		    /* well it seems to be the case - adjust */
-		    c = isdigit(cc1)? (cc1 - '0'): (toupper(cc1)-'A'+10);
+		    c = isdigit(cc1)? (cc1 - '0'): (ascii_toupper(cc1)-'A'+10);
 		    c <<= 4;
-		    c |= isdigit(cc2)? (cc2 - '0'): (toupper(cc2)-'A'+10);
+		    c |= isdigit(cc2)? (cc2 - '0'): (ascii_toupper(cc2)-'A'+10);
 		    afx->buffer_pos += 2;
 		    afx->qp_detected = 1;
 		    goto again;
@@ -651,7 +712,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	    break;
 	}
 	else if( (c = asctobin[(c2=c)]) == 255 ) {
-	    log_error(_("invalid radix64 character %02x skipped\n"), c2);
+	    log_error(_("invalid radix64 character %02X skipped\n"), c2);
 	    continue;
 	}
 	switch(idx) {
@@ -728,21 +789,28 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		    break; /* eof */
 	    } while( ++idx < 4 );
 	    if( c == -1 ) {
-		log_error(_("premature eof (in CRC)\n"));
-		rc = GPGERR_INVALID_ARMOR;
+		log_info(_("premature eof (in CRC)\n"));
+		rc = invalid_crc();
+	    }
+	    else if( idx == 0 ) {
+	        /* No CRC at all is legal ("MAY") */
+	        rc=0;
 	    }
 	    else if( idx != 4 ) {
-		log_error(_("malformed CRC\n"));
-		rc = GPGERR_INVALID_ARMOR;
+		log_info(_("malformed CRC\n"));
+		rc = invalid_crc();
 	    }
 	    else if( mycrc != afx->crc ) {
-		log_error(_("CRC error; %06lx - %06lx\n"),
+                log_info (_("CRC error; %06lX - %06lX\n"),
 				    (ulong)afx->crc, (ulong)mycrc);
-		rc = GPGERR_INVALID_ARMOR;
+                rc = invalid_crc();
 	    }
 	    else {
 		rc = 0;
-	      #if 0
+                /* FIXME: Here we should emit another control packet,
+                 * so that we know in mainproc that we are processing
+                 * a clearsign message */
+#if 0
 		for(rc=0;!rc;) {
 		    rc = 0 /*check_trailer( &fhdr, c )*/;
 		    if( !rc ) {
@@ -753,14 +821,14 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		if( rc == -1 )
 		    rc = 0;
 		else if( rc == 2 ) {
-		    log_error(_("premature eof (in Trailer)\n"));
-		    rc = GPGERR_INVALID_ARMOR;
+		    log_error(_("premature eof (in trailer)\n"));
+		    rc = G10ERR_INVALID_ARMOR;
 		}
 		else {
 		    log_error(_("error in trailer line\n"));
-		    rc = GPGERR_INVALID_ARMOR;
+		    rc = G10ERR_INVALID_ARMOR;
 		}
-	      #endif
+#endif
 	    }
 	}
     }
@@ -786,14 +854,14 @@ armor_filter( void *opaque, int control,
     int  idx, idx2;
     size_t n=0;
     u32 crc;
-  #if 0
+#if 0
     static FILE *fp ;
 
     if( !fp ) {
 	fp = fopen("armor.out", "w");
 	assert(fp);
     }
-  #endif
+#endif
 
     if( DBG_FILTER )
 	log_debug("armor-filter: control: %d\n", control );
@@ -815,7 +883,10 @@ armor_filter( void *opaque, int control,
 	*ret_len = n;
     }
     else if( control == IOBUFCTRL_UNDERFLOW ) {
-	if( size < 15+(4*15) )	/* need space for up to 4 onepass_sigs */
+        /* We need some space for the faked packet.  The minmum
+         * required size is the PARTIAL_CHUNK size plus a byte for the
+         * length itself */
+	if( size < PARTIAL_CHUNK+1 ) 
 	    BUG(); /* supplied buffer too short */
 
 	if( afx->faked )
@@ -831,54 +902,57 @@ armor_filter( void *opaque, int control,
 		    rc = -1;
 	    }
 	    else if( afx->faked ) {
-		unsigned hashes = afx->hashes;
+	        unsigned int hashes = afx->hashes;
+                const byte *sesmark;
+                size_t sesmarklen;
+                
+                sesmark = get_session_marker( &sesmarklen );
+                if ( sesmarklen > 20 )
+                    BUG();
+
 		/* the buffer is at least 15+n*15 bytes long, so it
 		 * is easy to construct the packets */
 
-		hashes &= 1|2|4|8;
+		hashes &= 1|2|4|8|16|32|64;
 		if( !hashes ) {
 		    hashes |= 4;  /* default to MD 5 */
+		    /* This is non-ideal since PGP 5-8 have the same
+		       end-of-line bugs as PGP 2. However, we only
+		       enable pgp2mode if there is no Hash: header. */
 		    if( opt.pgp2_workarounds )
 			afx->pgp2mode = 1;
 		}
 		n=0;
-		do {
-		    /* first some onepass signature packets */
-		    buf[n++] = 0x90; /* old format, type 4, 1 length byte */
-		    buf[n++] = 13;   /* length */
-		    buf[n++] = 3;    /* version */
-		    buf[n++] = afx->not_dash_escaped? 0:1; /* sigclass */
-		    if( hashes & 1 ) {
-			hashes &= ~1;
-			buf[n++] = GCRY_MD_RMD160;
-		    }
-		    else if( hashes & 2 ) {
-			hashes &= ~2;
-			buf[n++] = GCRY_MD_SHA1;
-		    }
-		    else if( hashes & 4 ) {
-			hashes &= ~4;
-			buf[n++] = GCRY_MD_MD5;
-		    }
-		    else if( hashes & 8 ) {
-			hashes &= ~8;
-			buf[n++] = GCRY_MD_TIGER;
-		    }
-		    else
-			buf[n++] = 0;	 /* (don't know) */
+                /* First a gpg control packet... */
+                buf[n++] = 0xff; /* new format, type 63, 1 length byte */
+                n++;   /* see below */
+                memcpy(buf+n, sesmark, sesmarklen ); n+= sesmarklen;
+                buf[n++] = CTRLPKT_CLEARSIGN_START; 
+                buf[n++] = afx->not_dash_escaped? 0:1; /* sigclass */
+                if( hashes & 1 )
+                    buf[n++] = DIGEST_ALGO_RMD160;
+                if( hashes & 2 )
+                    buf[n++] = DIGEST_ALGO_SHA1;
+                if( hashes & 4 )
+                    buf[n++] = DIGEST_ALGO_MD5;
+                if( hashes & 8 )
+                    buf[n++] = DIGEST_ALGO_SHA256;
+                if( hashes & 16 )
+                    buf[n++] = DIGEST_ALGO_SHA384;
+                if( hashes & 32 )
+                    buf[n++] = DIGEST_ALGO_SHA512;
+                buf[1] = n - 2;
 
-		    buf[n++] = 0;    /* public key algo (don't know) */
-		    memset(buf+n, 0, 8); /* don't know the keyid */
-		    n += 8;
-		    buf[n++] = !hashes;   /* last one */
-		} while( hashes );
+		/* ...followed by an invented plaintext packet.
+		   Amusingly enough, this packet is not compliant with
+		   2440 as the initial partial length is less than 512
+		   bytes.  Of course, we'll accept it anyway ;) */
 
-		/* followed by a plaintext packet */
-		buf[n++] = 0xaf; /* old packet format, type 11, var length */
-		buf[n++] = 0;	 /* set the length header */
-		buf[n++] = 6;
+		buf[n++] = 0xCB; /* new packet format, type 11 */
+		buf[n++] = 0xE1; /* 2^1 == 2 bytes */
 		buf[n++] = 't';  /* canonical text mode */
 		buf[n++] = 0;	 /* namelength */
+		buf[n++] = 0xE2; /* 2^2 == 4 more bytes */
 		memset(buf+n, 0, 4); /* timestamp */
 		n += 4;
 	    }
@@ -887,47 +961,61 @@ armor_filter( void *opaque, int control,
 	}
 	else
 	    rc = radix64_read( afx, a, &n, buf, size );
-      #if 0
+#if 0
 	if( n )
 	    if( fwrite(buf, n, 1, fp ) != 1 )
 		BUG();
-      #endif
+#endif
 	*ret_len = n;
     }
     else if( control == IOBUFCTRL_FLUSH && !afx->cancel ) {
 	if( !afx->status ) { /* write the header line */
 	    const char *s;
+	    STRLIST comment=opt.comments;
 
 	    if( afx->what >= DIM(head_strings) )
 		log_bug("afx->what=%d", afx->what);
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, head_strings[afx->what] );
-	    iobuf_writestr(a, "-----" LF );
+	    iobuf_writestr(a, "-----" );
+	    iobuf_writestr(a,afx->eol);
 	    if( !opt.no_version )
+	      {
 		iobuf_writestr(a, "Version: GnuPG v"  VERSION " ("
-					      PRINTABLE_OS_NAME ")" LF );
+			       PRINTABLE_OS_NAME ")" );
+		iobuf_writestr(a,afx->eol);
+	      }
 
-	    /* write the comment string or a default one */
-	    s = opt.comment_string ? opt.comment_string
-				   : _("For info see http://www.gnupg.org");
-	    if( *s ) {
+	    /* write the comment strings */
+	    for(s=comment->d;comment;comment=comment->next,s=comment->d)
+	      {
 		iobuf_writestr(a, "Comment: " );
-		for( ; *s; s++ ) {
+		for( ; *s; s++ )
+		  {
 		    if( *s == '\n' )
-			iobuf_writestr(a, "\\n" );
+		      iobuf_writestr(a, "\\n" );
 		    else if( *s == '\r' )
-			iobuf_writestr(a, "\\r" );
+		      iobuf_writestr(a, "\\r" );
 		    else if( *s == '\v' )
-			iobuf_writestr(a, "\\v" );
+		      iobuf_writestr(a, "\\v" );
 		    else
-			iobuf_put(a, *s );
-		}
-		iobuf_writestr(a, LF );
-	    }
+		      iobuf_put(a, *s );
+		  }
 
-	    if( afx->hdrlines )
-		iobuf_writestr(a, afx->hdrlines);
-	    iobuf_writestr(a, LF );
+		iobuf_writestr(a,afx->eol);
+	      }
+
+	    if ( afx->hdrlines ) {
+                for ( s = afx->hdrlines; *s; s++ ) {
+#ifdef HAVE_DOSISH_SYSTEM
+                    if ( *s == '\n' )
+                        iobuf_put( a, '\r');
+#endif
+                    iobuf_put(a, *s );
+                }
+            }
+
+	    iobuf_writestr(a,afx->eol);
 	    afx->status++;
 	    afx->idx = 0;
 	    afx->idx2 = 0;
@@ -956,10 +1044,11 @@ armor_filter( void *opaque, int control,
 		iobuf_put(a, c);
 		c = bintoasc[radbuf[2]&077];
 		iobuf_put(a, c);
-		if( ++idx2 >= (64/4) ) { /* pgp doesn't like 72 here */
-		    iobuf_writestr(a, LF );
+		if( ++idx2 >= (64/4) )
+		  { /* pgp doesn't like 72 here */
+		    iobuf_writestr(a,afx->eol);
 		    idx2=0;
-		}
+		  }
 	    }
 	}
 	for(i=0; i < idx; i++ )
@@ -968,10 +1057,23 @@ armor_filter( void *opaque, int control,
 	afx->idx2 = idx2;
 	afx->crc  = crc;
     }
-    else if( control == IOBUFCTRL_INIT ) {
+    else if( control == IOBUFCTRL_INIT )
+      {
 	if( !is_initialized )
-	    initialize();
-    }
+	  initialize();
+
+	/* Figure out what we're using for line endings if the caller
+	   didn't specify. */
+	if(afx->eol[0]==0)
+	  {
+#ifdef HAVE_DOSISH_SYSTEM
+	    afx->eol[0]='\r';
+	    afx->eol[1]='\n';
+#else
+	    afx->eol[0]='\n';
+#endif
+	  }
+      }
     else if( control == IOBUFCTRL_CANCEL ) {
 	afx->cancel = 1;
     }
@@ -1000,14 +1102,15 @@ armor_filter( void *opaque, int control,
 		    iobuf_put(a, c);
 		    iobuf_put(a, '=');
 		}
-		if( ++idx2 >= (64/4) ) { /* pgp doesn't like 72 here */
-		    iobuf_writestr(a, LF );
+		if( ++idx2 >= (64/4) )
+		  { /* pgp doesn't like 72 here */
+		    iobuf_writestr(a,afx->eol);
 		    idx2=0;
-		}
+		  }
 	    }
 	    /* may need a linefeed */
 	    if( idx2 )
-		iobuf_writestr(a, LF );
+	      iobuf_writestr(a,afx->eol);
 	    /* write the CRC */
 	    iobuf_put(a, '=');
 	    radbuf[0] = crc >>16;
@@ -1021,13 +1124,14 @@ armor_filter( void *opaque, int control,
 	    iobuf_put(a, c);
 	    c = bintoasc[radbuf[2]&077];
 	    iobuf_put(a, c);
-	    iobuf_writestr(a, LF );
+	    iobuf_writestr(a,afx->eol);
 	    /* and the the trailer */
 	    if( afx->what >= DIM(tail_strings) )
 		log_bug("afx->what=%d", afx->what);
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, tail_strings[afx->what] );
-	    iobuf_writestr(a, "-----" LF );
+	    iobuf_writestr(a, "-----" );
+	    iobuf_writestr(a,afx->eol);
 	}
 	else if( !afx->any_data && !afx->inp_bypass ) {
 	    log_error(_("no valid OpenPGP data found.\n"));
@@ -1041,7 +1145,7 @@ armor_filter( void *opaque, int control,
 	if( afx->qp_detected )
 	    log_error(_("quoted printable character in armor - "
 			"probably a buggy MTA has been used\n") );
-	gcry_free( afx->buffer );
+	m_free( afx->buffer );
 	afx->buffer = NULL;
     }
     else if( control == IOBUFCTRL_DESC )
@@ -1058,7 +1162,7 @@ make_radix64_string( const byte *data, size_t len )
 {
     char *buffer, *p;
 
-    buffer = p = gcry_xmalloc( (len+2)/3*4 + 1 );
+    buffer = p = m_alloc( (len+2)/3*4 + 1 );
     for( ; len >= 3 ; len -= 3, data += 3 ) {
 	*p++ = bintoasc[(data[0] >> 2) & 077];
 	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
@@ -1078,3 +1182,219 @@ make_radix64_string( const byte *data, size_t len )
     return buffer;
 }
 
+
+/***********************************************
+ *  For the pipemode command we can't use the armor filter for various
+ *  reasons, so we use this new unarmor_pump stuff to remove the armor 
+ */
+
+enum unarmor_state_e {
+    STA_init = 0,
+    STA_bypass,
+    STA_wait_newline,
+    STA_wait_dash,
+    STA_first_dash, 
+    STA_compare_header,
+    STA_found_header_wait_newline,
+    STA_skip_header_lines,
+    STA_skip_header_lines_non_ws,
+    STA_read_data,
+    STA_wait_crc,
+    STA_read_crc,
+    STA_ready
+};
+
+struct unarmor_pump_s {
+    enum unarmor_state_e state;
+    byte val;
+    int checkcrc;
+    int pos;   /* counts from 0..3 */
+    u32 crc;
+    u32 mycrc; /* the one store in the data */
+};
+
+
+
+UnarmorPump
+unarmor_pump_new (void)
+{
+    UnarmorPump x;
+
+    if( !is_initialized )
+        initialize();
+    x = m_alloc_clear (sizeof *x);
+    return x;
+}
+
+void
+unarmor_pump_release (UnarmorPump x)
+{
+    m_free (x);
+}
+
+/* 
+ * Get the next character from the ascii armor taken from the IOBUF
+ * created earlier by unarmor_pump_new().
+ * Return:  c = Character
+ *        256 = ignore this value
+ *         -1 = End of current armor 
+ *         -2 = Premature EOF (not used)
+ *         -3 = Invalid armor
+ */
+int
+unarmor_pump (UnarmorPump x, int c)
+{
+    int rval = 256; /* default is to ignore the return value */
+
+    switch (x->state) {
+      case STA_init:
+        { 
+            byte tmp[1];
+            tmp[0] = c; 
+            if ( is_armored (tmp) )
+                x->state = c == '-'? STA_first_dash : STA_wait_newline;
+            else {
+                x->state = STA_bypass;
+                return c;
+            }
+        }
+        break;
+      case STA_bypass:
+        return c; /* return here to avoid crc calculation */
+      case STA_wait_newline:
+        if (c == '\n')
+            x->state = STA_wait_dash;
+        break;
+      case STA_wait_dash:
+        x->state = c == '-'? STA_first_dash : STA_wait_newline;
+        break;
+      case STA_first_dash: /* just need for initalization */
+        x->pos = 0;
+        x->state = STA_compare_header;
+      case STA_compare_header:
+        if ( "-----BEGIN PGP SIGNATURE-----"[++x->pos] == c ) {
+            if ( x->pos == 28 ) 
+                x->state = STA_found_header_wait_newline;
+        }
+        else 
+            x->state = c == '\n'? STA_wait_dash : STA_wait_newline;
+        break;
+      case STA_found_header_wait_newline:
+        /* to make CR,LF issues easier we simply allow for white space
+           behind the 5 dashes */
+        if ( c == '\n' )
+            x->state = STA_skip_header_lines;
+        else if ( c != '\r' && c != ' ' && c != '\t' )
+            x->state = STA_wait_dash; /* garbage after the header line */
+        break;
+      case STA_skip_header_lines:
+        /* i.e. wait for one empty line */
+        if ( c == '\n' ) {
+            x->state = STA_read_data;
+            x->crc = CRCINIT;
+            x->val = 0;
+            x->pos = 0;
+        }
+        else if ( c != '\r' && c != ' ' && c != '\t' )
+            x->state = STA_skip_header_lines_non_ws;
+        break;
+      case STA_skip_header_lines_non_ws:
+        /* like above but we already encountered non white space */
+        if ( c == '\n' )
+            x->state = STA_skip_header_lines;
+        break;
+      case STA_read_data:
+        /* fixme: we don't check for the trailing dash lines but rely
+         * on the armor stop characters */
+        if( c == '\n' || c == ' ' || c == '\r' || c == '\t' )
+            break; /* skip all kind of white space */
+
+        if( c == '=' ) { /* pad character: stop */
+            if( x->pos == 1 ) /* in this case val has some value */
+                rval = x->val;
+            x->state = STA_wait_crc;
+            break;
+        }
+
+        {
+            int c2;
+            if( (c = asctobin[(c2=c)]) == 255 ) {
+                log_error(_("invalid radix64 character %02X skipped\n"), c2);
+                break;
+            }
+        }
+        
+        switch(x->pos) {
+          case 0:
+            x->val = c << 2;
+            break;
+          case 1:
+            x->val |= (c>>4)&3;
+            rval = x->val;
+            x->val = (c<<4)&0xf0;
+            break;
+          case 2:
+            x->val |= (c>>2)&15;
+            rval = x->val;
+            x->val = (c<<6)&0xc0;
+            break;
+          case 3:
+            x->val |= c&0x3f;
+            rval = x->val;
+            break;
+        }
+        x->pos = (x->pos+1) % 4;
+        break;
+      case STA_wait_crc:
+        if( c == '\n' || c == ' ' || c == '\r' || c == '\t' || c == '=' )
+            break; /* skip ws and pad characters */
+        /* assume that we are at the next line */
+        x->state = STA_read_crc;
+        x->pos = 0;
+        x->mycrc = 0;
+      case STA_read_crc:
+        if( (c = asctobin[c]) == 255 ) {
+            rval = -1; /* ready */
+            if( x->crc != x->mycrc ) {
+                log_info (_("CRC error; %06lX - %06lX\n"),
+                          (ulong)x->crc, (ulong)x->mycrc);
+                if ( invalid_crc() )
+                    rval = -3;
+            }
+            x->state = STA_ready; /* not sure whether this is correct */
+            break;
+        }
+        
+        switch(x->pos) {
+          case 0:
+            x->val = c << 2;
+            break;
+          case 1:
+            x->val |= (c>>4)&3;
+            x->mycrc |= x->val << 16;
+            x->val = (c<<4)&0xf0;
+            break;
+          case 2:
+            x->val |= (c>>2)&15;
+            x->mycrc |= x->val << 8;
+            x->val = (c<<6)&0xc0;
+            break;
+          case 3:
+            x->val |= c&0x3f;
+            x->mycrc |= x->val;
+            break;
+        }
+        x->pos = (x->pos+1) % 4;
+        break;
+      case STA_ready:
+        rval = -1;
+        break;
+    }
+
+    if ( !(rval & ~255) ) { /* compute the CRC */
+        x->crc = (x->crc << 8) ^ crc_table[((x->crc >> 16)&0xff) ^ rval];
+        x->crc &= 0x00ffffff;
+    }
+
+    return rval;
+}
