@@ -1,6 +1,6 @@
 /* getkey.c -  Get a key from the database
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ *               2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -183,7 +183,7 @@ get_primary_uid ( KBNODE keyblock, size_t *uidlen )
     } 
     /* fixme: returning translatable constants instead of a user ID is 
      * not good because they are probably not utf-8 encoded. */
-    s = _("[User id not found]");
+    s = _("[User ID not found]");
     *uidlen = strlen (s);
     return s;
 }
@@ -376,7 +376,6 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
 }
 
 
-
 /* Get a public key and store it into the allocated pk.  This function
    differs from get_pubkey() in that it does not do a check of the key
    to avoid recursion.  It should be used only in very certain cases.
@@ -565,20 +564,22 @@ seckey_available( u32 *keyid )
  *   is not case sensitive.
  */
 
-static int
-classify_user_id2( const char *name, 
-                   KEYDB_SEARCH_DESC *desc,
-                   int *force_exact )
+int
+classify_user_id( const char *name, KEYDB_SEARCH_DESC *desc )
 {
     const char *s;
     int hexprefix = 0;
     int hexlength;
     int mode = 0;   
-    
+    KEYDB_SEARCH_DESC dummy_desc;
+
+    if (!desc)
+        desc = &dummy_desc;
+
     /* clear the structure so that the mode field is set to zero unless
      * we set it to the correct value right at the end of this function */
     memset (desc, 0, sizeof *desc);
-    *force_exact = 0;
+
     /* skip leading spaces.  Fixme: what is with trailing spaces? */
     for(s = name; *s && spacep (s); s++ )
 	;
@@ -656,7 +657,7 @@ classify_user_id2( const char *name,
 
 	    hexlength = strspn(s, "0123456789abcdefABCDEF");
             if (hexlength >= 8 && s[hexlength] =='!') {
-                *force_exact = 1;
+		desc->exact = 1;
                 hexlength++; /* just for the following check */
             }
 
@@ -668,7 +669,7 @@ classify_user_id2( const char *name,
 		    hexlength = 0;  /* a hex number, but really were not. */
 	    }
 
-            if (*force_exact)
+            if (desc->exact)
                 hexlength--;
 
 	    if (hexlength == 8
@@ -724,7 +725,7 @@ classify_user_id2( const char *name,
 		if (hexprefix)	/* This was a hex number with a prefix */
 		    return 0;	/* and a wrong length */
 
-                *force_exact = 0;
+		desc->exact = 0;
                 desc->u.name = s;
 		mode = KEYDB_SEARCH_MODE_SUBSTR;   /* default mode */
 	    }
@@ -734,51 +735,62 @@ classify_user_id2( const char *name,
     return mode;
 }
 
-int
-classify_user_id (const char *name, KEYDB_SEARCH_DESC *desc)
-{
-    int dummy;
-    KEYDB_SEARCH_DESC dummy_desc;
-
-    if (!desc)
-        desc = &dummy_desc;
-    return classify_user_id2 (name, desc, &dummy);
-}
 
 static int
-skip_disabled(void *dummy,u32 *keyid)
+skip_unusable(void *dummy,u32 *keyid,PKT_user_id *uid)
 {
-  int rc,disabled=0;
-  PKT_public_key *pk=m_alloc_clear(sizeof(PKT_public_key));
+  int unusable=0;
+  KBNODE keyblock;
 
-  rc = get_pubkey(pk, keyid);
-  if(rc)
+  keyblock=get_pubkeyblock(keyid);
+  if(!keyblock)
     {
-      log_error("error checking disabled status of %08lX: %s\n",
- 		(ulong)keyid[1],g10_errstr(rc));
+      log_error("error checking usability status of %s\n",keystr(keyid));
       goto leave;
     }
- 
-  disabled=pk_is_disabled(pk);
+
+  /* Is the user ID in question revoked/expired? */
+  if(uid)
+    {
+      KBNODE node;
+
+      for(node=keyblock;node;node=node->next)
+	{
+	  if(node->pkt->pkttype==PKT_USER_ID)
+	    {
+	      if(cmp_user_ids(uid,node->pkt->pkt.user_id)==0
+		 && (node->pkt->pkt.user_id->is_revoked
+		     || node->pkt->pkt.user_id->is_expired))
+		{
+		  unusable=1;
+		  break;
+		}
+	    }
+	}
+    }
+
+  if(!unusable)
+    unusable=pk_is_disabled(keyblock->pkt->pkt.public_key);
 
  leave:
-  free_public_key(pk);
-  return disabled;
+  release_kbnode(keyblock);
+  return unusable;
 }
 
 /****************
  * Try to get the pubkey by the userid. This function looks for the
- * first pubkey certificate which has the given name in a user_id.
- * if pk/sk has the pubkey algo set, the function will only return
- * a pubkey with that algo.
- * The caller should provide storage for either the pk or the sk.
- * If ret_kb is not NULL the function will return the keyblock there.
+ * first pubkey certificate which has the given name in a user_id.  if
+ * pk/sk has the pubkey algo set, the function will only return a
+ * pubkey with that algo.  If namelist is NULL, the first key is
+ * returned.  The caller should provide storage for either the pk or
+ * the sk.  If ret_kb is not NULL the function will return the
+ * keyblock there.
  */
 
 static int
 key_byname( GETKEY_CTX *retctx, STRLIST namelist,
 	    PKT_public_key *pk, PKT_secret_key *sk,
-	    int secmode, int include_disabled,
+	    int secmode, int include_unusable,
             KBNODE *ret_kb, KEYDB_HANDLE *ret_kdbhd )
 {
     int rc = 0;
@@ -786,7 +798,6 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
     STRLIST r;
     GETKEY_CTX ctx;
     KBNODE help_kb = NULL;
-    int exact;
     
     if( retctx ) {/* reset the returned context in case of error */
         assert (!ret_kdbhd);  /* not allowed because the handle is
@@ -796,29 +807,43 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
     if (ret_kdbhd)
         *ret_kdbhd = NULL;
 
-    /* build the search context */
-    for(n=0, r=namelist; r; r = r->next )
-	n++;
-    ctx = m_alloc_clear (sizeof *ctx + (n-1)*sizeof ctx->items );
-    ctx->nitems = n;
+    if(!namelist)
+      {
+	ctx = m_alloc_clear (sizeof *ctx);
+	ctx->nitems = 1;
+	ctx->items[0].mode=KEYDB_SEARCH_MODE_FIRST;
+	if(!include_unusable)
+	  ctx->items[0].skipfnc=skip_unusable;
+      }
+    else
+      {
+	/* build the search context */
+	for(n=0, r=namelist; r; r = r->next )
+	  n++;
 
-    for(n=0, r=namelist; r; r = r->next, n++ ) {
-	classify_user_id2 (r->d, &ctx->items[n], &exact);
+	ctx = m_alloc_clear (sizeof *ctx + (n-1)*sizeof ctx->items );
+	ctx->nitems = n;
+
+	for(n=0, r=namelist; r; r = r->next, n++ )
+	  {
+	    classify_user_id (r->d, &ctx->items[n]);
         
-        if (exact)
-            ctx->exact = 1;
-        if (!ctx->items[n].mode) {
-	    m_free (ctx);
-	    return G10ERR_INV_USER_ID;
-	}
-	if(!include_disabled
- 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_SHORT_KID
- 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_LONG_KID
- 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR16
-	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR20
-	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR)
- 	  ctx->items[n].skipfnc=skip_disabled;
-    }
+	    if (ctx->items[n].exact)
+	      ctx->exact = 1;
+	    if (!ctx->items[n].mode)
+	      {
+		m_free (ctx);
+		return G10ERR_INV_USER_ID;
+	      }
+	    if(!include_unusable
+	       && ctx->items[n].mode!=KEYDB_SEARCH_MODE_SHORT_KID
+	       && ctx->items[n].mode!=KEYDB_SEARCH_MODE_LONG_KID
+	       && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR16
+	       && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR20
+	       && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR)
+	      ctx->items[n].skipfnc=skip_unusable;
+	  }
+      }
 
     ctx->kr_handle = keydb_new (secmode);
     if ( !ret_kb ) 
@@ -868,14 +893,14 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
 int
 get_pubkey_byname (PKT_public_key *pk,
 		   const char *name, KBNODE *ret_keyblock,
-                   KEYDB_HANDLE *ret_kdbhd, int include_disabled ) 
+                   KEYDB_HANDLE *ret_kdbhd, int include_unusable )
 {
     int rc;
     STRLIST namelist = NULL;
 
     add_to_strlist( &namelist, name );
     rc = key_byname( NULL, namelist, pk, NULL, 0,
-		     include_disabled, ret_keyblock, ret_kdbhd);
+		     include_unusable, ret_keyblock, ret_kdbhd);
     free_strlist( namelist );
     return rc;
 }
@@ -899,7 +924,6 @@ get_pubkey_next( GETKEY_CTX ctx, PKT_public_key *pk, KBNODE *ret_keyblock )
     return rc;
 }
 
-
 void
 get_pubkey_end( GETKEY_CTX ctx )
 {
@@ -912,8 +936,6 @@ get_pubkey_end( GETKEY_CTX ctx )
 }
 
 
-
-
 /****************
  * Search for a key with the given fingerprint.
  * FIXME:
@@ -997,10 +1019,6 @@ get_pubkey_byfprint_fast (PKT_public_key *pk,
   return 0;
 }
 
-
-
-
-
 /****************
  * Search for a key with the given fingerprint and return the
  * complete keyblock which may have more than only this key.
@@ -1037,49 +1055,25 @@ get_keyblock_byfprint( KBNODE *ret_keyblock, const byte *fprint,
  */
 static int
 get_seckey_byname2( GETKEY_CTX *retctx,
-                   PKT_secret_key *sk, const char *name, int unprotect,
-                   KBNODE *retblock )
+		    PKT_secret_key *sk, const char *name, int unprotect,
+		    KBNODE *retblock )
 {
-    STRLIST namelist = NULL;
-    int rc;
+  STRLIST namelist = NULL;
+  int rc;
 
-    if( !name && opt.def_secret_key && *opt.def_secret_key ) {
-	add_to_strlist( &namelist, opt.def_secret_key );
-	rc = key_byname( retctx, namelist, NULL, sk, 1, 1, retblock, NULL );
-    }
-    else if( !name ) { /* use the first one as default key */
-	struct getkey_ctx_s ctx;
-        KBNODE kb = NULL;
+  if( !name && opt.def_secret_key && *opt.def_secret_key )
+    add_to_strlist( &namelist, opt.def_secret_key );
+  else if(name)
+    add_to_strlist( &namelist, name );
 
-        assert (!retctx ); /* do we need this at all */
-        assert (!retblock);
-	memset( &ctx, 0, sizeof ctx );
-	ctx.not_allocated = 1;
-        ctx.kr_handle = keydb_new (1);
-	ctx.nitems = 1;
-	ctx.items[0].mode = KEYDB_SEARCH_MODE_FIRST;
-	rc = lookup( &ctx, &kb, 1 );
-        if (!rc && sk )
-	  {
-	    sk_from_block ( &ctx, sk, kb );
-	    if(sk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
-	       || (sk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && sk->version<4))
-	      rc=G10ERR_UNU_SECKEY;
-	  }
-        release_kbnode ( kb );
-	get_seckey_end( &ctx );
-    }
-    else {
-	add_to_strlist( &namelist, name );
-	rc = key_byname( retctx, namelist, NULL, sk, 1, 1, retblock, NULL );
-    }
+  rc = key_byname( retctx, namelist, NULL, sk, 1, 1, retblock, NULL );
 
-    free_strlist( namelist );
+  free_strlist( namelist );
 
-    if( !rc && unprotect )
-	rc = check_secret_key( sk, 0 );
+  if( !rc && unprotect )
+    rc = check_secret_key( sk, 0 );
 
-    return rc;
+  return rc;
 }
 
 int 
@@ -1145,13 +1139,41 @@ get_seckey_byfprint( PKT_secret_key *sk,
         if (!rc && sk )
             sk_from_block ( &ctx, sk, kb );
         release_kbnode ( kb );
-	get_pubkey_end( &ctx );
+	get_seckey_end( &ctx );
     }
     else
 	rc = G10ERR_GENERAL; /* Oops */
     return rc;
 }
 
+
+/* Search for a secret key with the given fingerprint and return the
+   complete keyblock which may have more than only this key. */
+int
+get_seckeyblock_byfprint (KBNODE *ret_keyblock, const byte *fprint,
+                          size_t fprint_len )
+{
+  int rc;
+  struct getkey_ctx_s ctx;
+  
+  if (fprint_len != 20 && fprint_len == 16)
+    return G10ERR_GENERAL; /* Oops */
+    
+  memset (&ctx, 0, sizeof ctx);
+  ctx.not_allocated = 1;
+  ctx.kr_handle = keydb_new (1);
+  ctx.nitems = 1;
+  ctx.items[0].mode = (fprint_len==16
+                       ? KEYDB_SEARCH_MODE_FPR16
+                       : KEYDB_SEARCH_MODE_FPR20);
+  memcpy (ctx.items[0].u.fpr, fprint, fprint_len);
+  rc = lookup (&ctx, ret_keyblock, 1);
+  get_seckey_end (&ctx);
+  
+  return rc;
+}
+
+
 
 /************************************************
  ************* Merging stuff ********************
@@ -1248,6 +1270,51 @@ merge_keys_and_selfsig( KBNODE keyblock )
     }
 }
 
+static int
+parse_key_usage(PKT_signature *sig)
+{
+  int key_usage=0;
+  const byte *p;
+  size_t n;
+  byte flags;
+
+  p=parse_sig_subpkt(sig->hashed,SIGSUBPKT_KEY_FLAGS,&n);
+  if(p && n)
+    {
+      /* first octet of the keyflags */
+      flags=*p;
+
+      if(flags & 3)
+	{
+	  key_usage |= PUBKEY_USAGE_SIG;
+	  flags&=~3;
+	}
+
+      if(flags & 12)
+	{
+	  key_usage |= PUBKEY_USAGE_ENC;
+	  flags&=~12;
+	}
+
+      if(flags & 0x20)
+	{
+	  key_usage |= PUBKEY_USAGE_AUTH;
+	  flags&=~0x20;
+	}
+
+      if(flags)
+	key_usage |= PUBKEY_USAGE_UNKNOWN;
+    }
+
+  /* We set PUBKEY_USAGE_UNKNOWN to indicate that this key has a
+     capability that we do not handle.  This serves to distinguish
+     between a zero key usage which we handle as the default
+     capabilities for that algorithm, and a usage that we do not
+     handle. */
+
+  return key_usage;
+}
+
 /*
  * Apply information from SIGNODE (which is the valid self-signature
  * associated with that UID) to the UIDNODE:
@@ -1280,17 +1347,7 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     uid->expiredate = sig->expiredate;
 
     /* store the key flags in the helper variable for later processing */
-    uid->help_key_usage = 0;
-    p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_FLAGS, &n );
-    if ( p && n ) {
-        /* first octet of the keyflags */   
-        if ( (*p & 3) )
-            uid->help_key_usage |= PUBKEY_USAGE_SIG;
-        if ( (*p & 12) )    
-            uid->help_key_usage |= PUBKEY_USAGE_ENC;
-        /* Note: we do not set the CERT flag here because it can be assumed
-         * that thre is no real policy to set it. */
-    }
+    uid->help_key_usage=parse_key_usage(sig);
 
     /* ditto or the key expiration */
     uid->help_key_expire = 0;
@@ -1349,10 +1406,25 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_FEATURES, &n);
     if (p && n && (p[0] & 0x01))
         uid->mdc_feature = 1;
+
+    /* and the keyserver modify flag */
+    uid->ks_modify = 1;
+    p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KS_FLAGS, &n);
+    if (p && n && (p[0] & 0x80))
+        uid->ks_modify = 0;
 }
 
 static void
-merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
+sig_to_revoke_info(PKT_signature *sig,struct revoke_info *rinfo)
+{
+  rinfo->date = sig->timestamp;
+  rinfo->algo = sig->pubkey_algo;
+  rinfo->keyid[0] = sig->keyid[0];
+  rinfo->keyid[1] = sig->keyid[1];
+}
+
+static void
+merge_selfsigs_main(KBNODE keyblock, int *r_revoked, struct revoke_info *rinfo)
 {
     PKT_public_key *pk = NULL;
     KBNODE k;
@@ -1367,6 +1439,8 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
     byte sigversion = 0;
 
     *r_revoked = 0;
+    memset(rinfo,0,sizeof(*rinfo));
+
     if ( keyblock->pkt->pkttype != PKT_PUBLIC_KEY )
         BUG ();
     pk = keyblock->pkt->pkt.public_key;
@@ -1412,6 +1486,7 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
                      * that key.
                      */ 
                     *r_revoked = 1;
+		    sig_to_revoke_info(sig,rinfo);
                 }
                 else if ( IS_KEY_SIG (sig) ) {
 		  /* Add any revocation keys onto the pk.  This is
@@ -1480,33 +1555,27 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 			       pk->numrevkeys*sizeof(struct revocation_key));
       }
 
-    if ( signode ) {
+    if ( signode )
+      {
         /* some information from a direct key signature take precedence
          * over the same information given in UID sigs.
          */
         PKT_signature *sig = signode->pkt->pkt.signature;
         const byte *p;
-        size_t n;
-        
-        p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_FLAGS, &n );
-        if ( p && n ) {
-            /* first octet of the keyflags */   
-            if ( (*p & 3) )
-                key_usage |= PUBKEY_USAGE_SIG;
-            if ( (*p & 12) )    
-                key_usage |= PUBKEY_USAGE_ENC;
-        }
+
+	key_usage=parse_key_usage(sig);
 
 	p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE, NULL);
-	if ( p ) {
-	  key_expire = keytimestamp + buffer_to_u32(p);
-	  key_expire_seen = 1;
-        }
+	if ( p )
+	  {
+	    key_expire = keytimestamp + buffer_to_u32(p);
+	    key_expire_seen = 1;
+	  }
 
         /* mark that key as valid: one direct key signature should 
          * render a key as valid */
         pk->is_valid = 1;
-    }
+      }
 
     /* pass 1.5: look for key revocation signatures that were not made
        by the key (i.e. did a revocation key issue a revocation for
@@ -1527,6 +1596,7 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 		  if(rc==0)
 		    {
 		      *r_revoked=2;
+		      sig_to_revoke_info(sig,rinfo);
 		      /* don't continue checking since we can't be any
 			 more revoked than this */
 		      break;
@@ -1604,10 +1674,8 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
     if(!pk->is_valid && opt.allow_non_selfsigned_uid)
       {
 	if(opt.verbose)
-	  log_info(_("Invalid key %08lX made valid by "
-		     "--allow-non-selfsigned-uid\n"),
-		   (ulong)keyid_from_pk(pk,NULL));
-
+	  log_info(_("Invalid key %s made valid by"
+		     " --allow-non-selfsigned-uid\n"),keystr_from_pk(pk));
 	pk->is_valid = 1;
       }
 
@@ -1631,9 +1699,15 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 
 		    ultimate_pk=m_alloc_clear(sizeof(*ultimate_pk));
 
+                    /* We don't want to use the full get_pubkey to
+                       avoid infinite recursion in certain cases.
+                       There is no reason to check that an ultimately
+                       trusted key is still valid - if it has been
+                       revoked or the user should also renmove the
+                       ultimate trust flag.  */
 		    if(get_pubkey_fast(ultimate_pk,sig->keyid)==0
 		       && check_key_signature2(keyblock,k,ultimate_pk,
-					       NULL,NULL,NULL)==0
+					       NULL,NULL,NULL,NULL)==0
 		       && get_ownertrust(ultimate_pk)==TRUST_ULTIMATE)
 		      {
 			free_public_key(ultimate_pk);
@@ -1684,13 +1758,6 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         if ( x ) /* mask it down to the actual allowed usage */
             key_usage &= x; 
     }
-
-    /* Type 20 Elgamal keys and the old v3 Elgamal keys are not
-       usable. */
-    if(pk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
-       || (pk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && pk->version<4))
-      key_usage=0;
-
     pk->pubkey_usage = key_usage;
 
     if ( !key_expire_seen ) {
@@ -1831,7 +1898,6 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
     u32 keytimestamp = 0;
     u32 key_expire = 0;
     const byte *p;
-    size_t n;
 
     if ( subnode->pkt->pkttype != PKT_PUBLIC_SUBKEY )
         BUG ();
@@ -1866,6 +1932,7 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
                      problem is in the distribution.  Plus, PGP (7)
                      does this the same way.  */
                     subpk->is_revoked = 1;
+		    sig_to_revoke_info(sig,&subpk->revoked);
                     /* although we could stop now, we continue to 
                      * figure out other information like the old expiration
                      * time */
@@ -1882,37 +1949,26 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
         }
     }
 
-    if ( !signode ) {
-        return;  /* no valid key binding */
-    }
+    /* no valid key binding */
+    if ( !signode )
+      return;
 
-    subpk->is_valid = 1;
     sig = signode->pkt->pkt.signature;
     sig->flags.chosen_selfsig=1; /* so we know which selfsig we chose later */
-        
-    p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_FLAGS, &n );
-    if ( p && n ) {
-        /* first octet of the keyflags */   
-        if ( (*p & 3) )
-            key_usage |= PUBKEY_USAGE_SIG;
-        if ( (*p & 12) )    
-            key_usage |= PUBKEY_USAGE_ENC;
-    }
-    if ( !key_usage ) { /* no key flags at all: get it from the algo */
+
+    key_usage=parse_key_usage(sig);
+    if ( !key_usage )
+      {
+	/* no key flags at all: get it from the algo */
         key_usage = openpgp_pk_algo_usage ( subpk->pubkey_algo );
-    }
-    else { /* check that the usage matches the usage as given by the algo */
+      }
+    else
+      {
+	/* check that the usage matches the usage as given by the algo */
         int x = openpgp_pk_algo_usage ( subpk->pubkey_algo );
         if ( x ) /* mask it down to the actual allowed usage */
-            key_usage &= x; 
-    }
-
-    /* Type 20 Elgamal subkeys, any subkey on a type 20 primary, or
-       any subkey on an old v3 Elgamal(e) primary are not usable. */
-    if(mainpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
-       || subpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
-       || (mainpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && mainpk->version<4))
-      key_usage=0;
+	  key_usage &= x; 
+      }
 
     subpk->pubkey_usage = key_usage;
     
@@ -1924,6 +1980,12 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
     subpk->has_expired = key_expire >= curtime? 0 : key_expire;
     subpk->expiredate = key_expire;
 
+    /* algo doesn't exist */
+    if(check_pubkey_algo(subpk->pubkey_algo))
+      return;
+
+    subpk->is_valid = 1;
+
 #ifndef DO_BACKSIGS
     /* Pretend the backsig is present and accounted for. */
     subpk->backsig=2;
@@ -1932,6 +1994,7 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
     if(subpk->backsig==0)
       {
 	int seq=0;
+	size_t n;
 
 	while((p=enum_sig_subpkt(sig->hashed,
 				 SIGSUBPKT_SIGNATURE,&n,&seq,NULL)))
@@ -1987,6 +2050,7 @@ merge_selfsigs( KBNODE keyblock )
 {
     KBNODE k;
     int revoked;
+    struct revoke_info rinfo;
     PKT_public_key *main_pk;
     prefitem_t *prefs;
     int mdc_feature;
@@ -2003,7 +2067,7 @@ merge_selfsigs( KBNODE keyblock )
         BUG ();
     }
 
-    merge_selfsigs_main ( keyblock, &revoked );
+    merge_selfsigs_main ( keyblock, &revoked, &rinfo );
 
     /* now merge in the data from each of the subkeys */
     for(k=keyblock; k; k = k->next ) {
@@ -2024,7 +2088,10 @@ merge_selfsigs( KBNODE keyblock )
 		if(!main_pk->is_valid)
 		  pk->is_valid = 0;
 		if(revoked && !pk->is_revoked)
-		  pk->is_revoked = revoked;
+		  {
+		    pk->is_revoked = revoked;
+		    memcpy(&pk->revoked,&rinfo,sizeof(rinfo));
+		  }
                 if(main_pk->has_expired)
 		  pk->has_expired = main_pk->has_expired;
             }
@@ -2145,7 +2212,8 @@ premerge_public_with_secret ( KBNODE pubblock, KBNODE secblock )
                             /* The secret parts are not available so
                                we can't use that key for signing etc.
                                Fix the pubkey usage */
-                            pk->pubkey_usage &= ~PUBKEY_USAGE_SIG;
+                            pk->pubkey_usage &= ~(PUBKEY_USAGE_SIG
+                                                  |PUBKEY_USAGE_AUTH);
                         }
                         /* transfer flag bits 0 and 1 to the pubblock */
                         pub->flag |= (sec->flag &3);
@@ -2157,14 +2225,14 @@ premerge_public_with_secret ( KBNODE pubblock, KBNODE secblock )
                 KBNODE next, ll;
 
                 if (opt.verbose)
-                  log_info ( _("no secret subkey "
-                               "for public subkey %08lX - ignoring\n"),  
-                           (ulong)keyid_from_pk (pk,NULL) );
+                  log_info (_("no secret subkey"
+			      " for public subkey %s - ignoring\n"),  
+			    keystr_from_pk (pk));
                 /* we have to remove the subkey in this case */
                 assert ( last );
                 /* find the next subkey */
                 for (next=pub->next,ll=pub;
-                     next && pub->pkt->pkttype != PKT_PUBLIC_SUBKEY;
+                     next && next->pkt->pkttype != PKT_PUBLIC_SUBKEY;
                      ll = next, next = next->next ) 
                     ;
                 /* make new link */
@@ -2384,12 +2452,14 @@ finish_lookup (GETKEY_CTX ctx)
         
     ctx->found_key = latest_key;
 
-    if (latest_key != keyblock && opt.verbose) {
-        log_info(_("using secondary key %08lX "
-                   "instead of primary key %08lX\n"),
-                 (ulong)keyid_from_pk( latest_key->pkt->pkt.public_key, NULL),
-                 (ulong)keyid_from_pk( keyblock->pkt->pkt.public_key, NULL) );
-    }
+    if (latest_key != keyblock && opt.verbose)
+      {
+	char *tempkeystr=
+	  m_strdup(keystr_from_pk(latest_key->pkt->pkt.public_key));
+        log_info(_("using subkey %s instead of primary key %s\n"),
+                 tempkeystr, keystr_from_pk(keyblock->pkt->pkt.public_key));
+	m_free(tempkeystr);
+      }
 
     cache_user_id( keyblock );
     
@@ -2430,12 +2500,13 @@ lookup( GETKEY_CTX ctx, KBNODE *ret_keyblock, int secmode )
 
             keyid_from_sk (k->pkt->pkt.secret_key, aki);
             k = get_pubkeyblock (aki);
-            if( !k ) {
+            if( !k )
+	      {
                 if (!opt.quiet)
-                    log_info(_("key %08lX: secret key without public key "
-                               "- skipped\n"),  (ulong)aki[1] );
+		  log_info(_("key %s: secret key without public key"
+			     " - skipped\n"), keystr(aki));
                 goto skip;
-            }
+	      }
             secblock = ctx->keyblock;
             ctx->keyblock = k;
 
@@ -2503,6 +2574,9 @@ lookup( GETKEY_CTX ctx, KBNODE *ret_keyblock, int secmode )
  *                   b) to get the ultimately trusted keys.
  *        The a) usage might have some problems.
  *
+ * set with_subkeys true to include subkeys
+ * set with_spm true to include secret-parts-missing keys
+ *
  * Enumerate all primary secret keys.  Caller must use these procedure:
  *  1) create a void pointer and initialize it to NULL
  *  2) pass this void pointer by reference to this function
@@ -2513,7 +2587,8 @@ lookup( GETKEY_CTX ctx, KBNODE *ret_keyblock, int secmode )
  *     so that can free it's context.
  */
 int
-enum_secret_keys( void **context, PKT_secret_key *sk, int with_subkeys )
+enum_secret_keys( void **context, PKT_secret_key *sk,
+		  int with_subkeys, int with_spm )
 {
     int rc=0;
     struct {
@@ -2548,9 +2623,11 @@ enum_secret_keys( void **context, PKT_secret_key *sk, int with_subkeys )
     do {
         /* get the next secret key from the current keyblock */
         for (; c->node; c->node = c->node->next) {
-            if (c->node->pkt->pkttype == PKT_SECRET_KEY
+            if ((c->node->pkt->pkttype == PKT_SECRET_KEY
                 || (with_subkeys
-                    && c->node->pkt->pkttype == PKT_SECRET_SUBKEY) ) {
+                    && c->node->pkt->pkttype == PKT_SECRET_SUBKEY) )
+		&& !(c->node->pkt->pkt.secret_key->protect.s2k.mode==1001
+		     && !with_spm)) {
                 copy_secret_key (sk, c->node->pkt->pkt.secret_key );
                 c->node = c->node->next;
                 return 0; /* found */
@@ -2587,26 +2664,29 @@ enum_secret_keys( void **context, PKT_secret_key *sk, int with_subkeys )
 char*
 get_user_id_string( u32 *keyid )
 {
-    user_id_db_t r;
-    char *p;
-    int pass=0;
-    /* try it two times; second pass reads from key resources */
-    do {
-	for(r=user_id_db; r; r = r->next ) {
-            keyid_list_t a;
-            for (a=r->keyids; a; a= a->next ) {
-                if( a->keyid[0] == keyid[0] && a->keyid[1] == keyid[1] ) {
-                    p = m_alloc( r->len + 10 );
-                    sprintf(p, "%08lX %.*s",
-                            (ulong)keyid[1], r->len, r->name );
-                    return p;
-                }
-            }
+  user_id_db_t r;
+  char *p;
+  int pass=0;
+  /* try it two times; second pass reads from key resources */
+  do
+    {
+      for(r=user_id_db; r; r = r->next )
+	{
+	  keyid_list_t a;
+	  for (a=r->keyids; a; a= a->next )
+	    {
+	      if( a->keyid[0] == keyid[0] && a->keyid[1] == keyid[1] )
+		{
+		  p = m_alloc( keystrlen() + 1 + r->len + 1 );
+		  sprintf(p, "%s %.*s", keystr(keyid), r->len, r->name );
+		  return p;
+		}
+	    }
         }
     } while( ++pass < 2 && !get_pubkey( NULL, keyid ) );
-    p = m_alloc( 15 );
-    sprintf(p, "%08lX [?]", (ulong)keyid[1] );
-    return p;
+  p = m_alloc( keystrlen() + 5 );
+  sprintf(p, "%s [?]", keystr(keyid));
+  return p;
 }
 
 
@@ -2667,7 +2747,7 @@ get_user_id( u32 *keyid, size_t *rn )
             }
         }
     } while( ++pass < 2 && !get_pubkey( NULL, keyid ) );
-    p = m_strdup( _("[User id not found]") );
+    p = m_strdup( _("[User ID not found]") );
     *rn = strlen(p);
     return p;
 }
