@@ -129,6 +129,7 @@ struct
   {
     char *uri;
     char *scheme;
+    char *auth;
     char *host;
     char *port;
     char *path;
@@ -187,11 +188,12 @@ struct
   int no_homedir_creation;
   struct groupitem *grouplist;
   int strict;
-  int mangle_dos_filenames; 
+  int mangle_dos_filenames;
   int enable_progress_filter;
   unsigned int screen_columns;
   unsigned int screen_lines;
   byte *show_subpackets;
+  int rfc2440_text;
 
 #ifdef ENABLE_CARD_SUPPORT
   const char *ctapi_driver; /* Library to access the ctAPI. */
@@ -250,11 +252,13 @@ struct {
 #define IMPORT_FAST                      (1<<2)
 #define IMPORT_SK2PK                     (1<<3)
 #define IMPORT_MERGE_ONLY                (1<<4)
+#define IMPORT_UNUSABLE_SIGS             (1<<5)
 
 #define EXPORT_LOCAL_SIGS                (1<<0)
 #define EXPORT_ATTRIBUTES                (1<<1)
 #define EXPORT_SENSITIVE_REVKEYS         (1<<2)
 #define EXPORT_MINIMAL                   (1<<3)
+#define EXPORT_UNUSABLE_SIGS             (1<<4)
 
 #define LIST_SHOW_PHOTOS                 (1<<0)
 #define LIST_SHOW_POLICY_URLS            (1<<1)
