@@ -22,7 +22,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#ifdef __MINGW32__ 
+#ifdef _WIN32 
 # include <windows.h>
 # ifndef VER_PLATFORM_WIN32_WINDOWS
 #  define VER_PLATFORM_WIN32_WINDOWS 1
@@ -39,6 +39,7 @@
 #include "options.h"
 #include "main.h"
 #include "photoid.h"
+#include "ttyio.h"
 
 /* Generate a new photo id packet, or return NULL if canceled */
 PKT_user_id *generate_photo_id(PKT_public_key *pk)
@@ -62,8 +63,8 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 #define EXTRA_UID_NAME_SPACE 71
   uid=m_alloc_clear(sizeof(*uid)+71);
 
-  printf(_("\nPick an image to use for your photo ID.  "
-	   "The image must be a JPEG file.\n"
+  tty_printf(_("\nPick an image to use for your photo ID.  "
+           "The image must be a JPEG file.\n"
 	   "Remember that the image is stored within your public key.  "
 	   "If you use a\n"
 	   "very large picture, your key will become very large as well!\n"
@@ -71,7 +72,7 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 
   while(photo==NULL)
     {
-      printf("\n");
+      tty_printf("\n");
 
       m_free(filename);
 
@@ -82,9 +83,15 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 	goto scram;
 
       file=iobuf_open(filename);
+      if (file && is_secured_file (iobuf_get_fd (file)))
+        {
+          iobuf_close (file);
+          file = NULL;
+          errno = EPERM;
+        }
       if(!file)
 	{
-	  log_error(_("Unable to open photo \"%s\": %s\n"),
+	  log_error(_("unable to open JPEG file `%s': %s\n"),
 		    filename,strerror(errno));
 	  continue;
 	}
@@ -92,9 +99,9 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
       len=iobuf_get_filelength(file);
       if(len>6144)
 	{
-	  printf("This JPEG is really large (%d bytes) !\n",len);
+	  tty_printf( _("This JPEG is really large (%d bytes) !\n"),len);
 	  if(!cpr_get_answer_is_yes("photoid.jpeg.size",
-			    _("Are you sure you want to use it (y/N)? ")))
+			    _("Are you sure you want to use it? (y/N) ")))
 	  {
 	    iobuf_close(file);
 	    continue;
@@ -109,7 +116,7 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
       if(photo[0]!=0xFF || photo[1]!=0xD8 ||
 	 photo[6]!='J' || photo[7]!='F' || photo[8]!='I' || photo[9]!='F')
 	{
-	  log_error(_("\"%s\" is not a JPEG file\n"),filename);
+	  log_error(_("`%s' is not a JPEG file\n"),filename);
 	  m_free(photo);
 	  photo=NULL;
 	  continue;
@@ -223,7 +230,7 @@ char *image_type_to_string(byte type,int style)
 #if !defined(FIXED_PHOTO_VIEWER) && !defined(DISABLE_PHOTO_VIEWER)
 static const char *get_default_photo_command(void)
 {
-#if defined(__MINGW32__)
+#if defined(_WIN32)
   OSVERSIONINFO osvi;
 
   memset(&osvi,0,sizeof(osvi));
@@ -307,8 +314,8 @@ void show_photos(const struct user_attribute *attrs,
 	  }
 
 #ifdef __riscos__
-        riscos_set_filetype(spawn->tempfile_in,
-                            image_type_to_string(args.imagetype,2));
+        riscos_set_filetype_by_mimetype(spawn->tempfile_in,
+                                        image_type_to_string(args.imagetype,2));
 #endif
 
 	m_free(name);
