@@ -1,5 +1,6 @@
 /* plaintext.c -  process plaintext packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,8 +25,9 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
+#include <sys/types.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
+#include <fcntl.h> /* for setmode() */
 #endif
 
 #include "util.h"
@@ -39,7 +41,6 @@
 #include "i18n.h"
 
 
-
 /****************
  * Handle a plaintext packet.  If MFX is not NULL, update the MDs
  * Note: we should use the filter stuff here, but we have to add some
@@ -52,9 +53,31 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 {
     char *fname = NULL;
     FILE *fp = NULL;
+    static off_t count=0;
     int rc = 0;
     int c;
-    int convert = pt->mode == 't';
+    int convert = (pt->mode == 't' || pt->mode == 'u');
+#ifdef __riscos__
+    int filetype = 0xfff;
+#endif
+
+    /* Let people know what the plaintext info is. This allows the
+       receiving program to try and do something different based on
+       the format code (say, recode UTF-8 to local). */
+    if(!nooutput && is_status_enabled())
+      {
+	char status[50];
+
+	sprintf(status,"%X %lu ",(byte)pt->mode,(ulong)pt->timestamp);
+	write_status_text_and_buffer(STATUS_PLAINTEXT,
+				     status,pt->name,pt->namelen,0);
+
+	if(!pt->is_partial)
+	  {
+	    sprintf(status,"%lu",(ulong)pt->len);
+	    write_status_text(STATUS_PLAINTEXT_LENGTH,status);
+	  }
+      }
 
     /* create the filename as C string */
     if( nooutput )
@@ -82,13 +105,14 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 
     if( nooutput )
 	;
-    else if( !*fname || (*fname=='-' && !fname[1])) {
-	/* no filename or "-" given; write to stdout */
+    else if ( iobuf_is_pipe_filename (fname) || !*fname)
+      {
+	/* No filename or "-" given; write to stdout. */
 	fp = stdout;
-      #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
 	setmode ( fileno(fp) , O_BINARY );
-      #endif
-    }
+#endif
+      }
     else {
 	while( !overwrite_filep (fname) ) {
             char *tmp = ask_outfile_name (NULL, 0);
@@ -102,17 +126,54 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
         }
     }
 
+#ifndef __riscos__
     if( fp || nooutput )
 	;
+    else if (is_secured_filename (fname))
+      {
+        errno = EPERM;
+	log_error(_("error creating `%s': %s\n"), fname, strerror(errno) );
+	rc = G10ERR_CREATE_FILE;
+	goto leave;
+      }
     else if( !(fp = fopen(fname,"wb")) ) {
 	log_error(_("error creating `%s': %s\n"), fname, strerror(errno) );
 	rc = G10ERR_CREATE_FILE;
-#ifdef __riscos__
-        if (errno == 106)
-            log_info("perhaps the output file has the same name as the input file?\n");
-#endif /* __riscos__ */
 	goto leave;
     }
+#else /* __riscos__ */
+    /* If no output filename was given, i.e. we constructed it,
+       convert all '.' in fname to '/' but not vice versa as
+       we don't create directories! */
+    if( !opt.outfile )
+        for( c=0; fname[c]; ++c )
+            if( fname[c] == '.' )
+                fname[c] = '/';
+
+    if( fp || nooutput )
+	;
+    else {
+        fp = fopen(fname,"wb");
+        if( !fp ) {
+            log_error(_("error creating `%s': %s\n"), fname, strerror(errno) );
+            rc = G10ERR_CREATE_FILE;
+            if (errno == 106)
+                log_info("Do output file and input file have the same name?\n");
+            goto leave;
+	}
+
+        /* If there's a ,xxx extension in the embedded filename,
+           use that, else check whether the user input (in fname)
+           has a ,xxx appended, then use that in preference */
+        if( (c = riscos_get_filetype_from_string( pt->name,
+                                                  pt->namelen )) != -1 )
+            filetype = c;
+        if( (c = riscos_get_filetype_from_string( fname,
+                                                  strlen(fname) )) != -1 )
+            filetype = c;
+        riscos_set_filetype_by_number(fname, filetype);
+    }
+#endif /* __riscos__ */
 
     if( !pt->is_partial ) {
         /* we have an actual length (which might be zero). */
@@ -127,18 +188,27 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		}
 		if( mfx->md )
 		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
+#ifndef HAVE_DOSISH_SYSTEM
 		if( c == '\r' )  /* convert to native line ending */
 		    continue;	 /* fixme: this hack might be too simple */
-	      #endif
-		if( fp ) {
-		    if( putc( c, fp ) == EOF ) {
+#endif
+		if( fp )
+		  {
+		    if(opt.max_output && (++count)>opt.max_output)
+		      {
+			log_error("Error writing to `%s': %s\n",
+				  fname,"exceeded --max-output limit\n");
+			rc = G10ERR_WRITE_FILE;
+			goto leave;
+		      }
+		    else if( putc( c, fp ) == EOF )
+		      {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
 			rc = G10ERR_WRITE_FILE;
 			goto leave;
-		    }
-		}
+		      }
+		  }
 	    }
 	}
 	else { /* binary mode */
@@ -155,15 +225,25 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		}
 		if( mfx->md )
 		    md_write( mfx->md, buffer, len );
-		if( fp ) {
-		    if( fwrite( buffer, 1, len, fp ) != len ) {
+		if( fp )
+		  {
+		    if(opt.max_output && (count+=len)>opt.max_output)
+		      {
+			log_error("Error writing to `%s': %s\n",
+				  fname,"exceeded --max-output limit\n");
+			rc = G10ERR_WRITE_FILE;
+			m_free( buffer );
+			goto leave;
+		      }
+		    else if( fwrite( buffer, 1, len, fp ) != len )
+		      {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
 			rc = G10ERR_WRITE_FILE;
 			m_free( buffer );
 			goto leave;
-		    }
-		}
+		      }
+		  }
 		pt->len -= len;
 	    }
 	    m_free( buffer );
@@ -174,18 +254,27 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    while( (c = iobuf_get(pt->buf)) != -1 ) {
 		if( mfx->md )
 		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
+#ifndef HAVE_DOSISH_SYSTEM
 		if( convert && c == '\r' )
 		    continue; /* fixme: this hack might be too simple */
-	      #endif
-		if( fp ) {
-		    if( putc( c, fp ) == EOF ) {
+#endif
+		if( fp )
+		  {
+		    if(opt.max_output && (++count)>opt.max_output)
+		      {
+			log_error("Error writing to `%s': %s\n",
+				  fname,"exceeded --max-output limit\n");
+			rc = G10ERR_WRITE_FILE;
+			goto leave;
+		      }
+		    else if( putc( c, fp ) == EOF )
+		      {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
 			rc = G10ERR_WRITE_FILE;
 			goto leave;
-		    }
-		}
+		      }
+		  }
 	    }
 	}
 	else { /* binary mode */
@@ -205,15 +294,24 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    eof = 1;
 		if( mfx->md )
 		    md_write( mfx->md, buffer, len );
-		if( fp ) {
-		    if( fwrite( buffer, 1, len, fp ) != len ) {
+		if( fp )
+		  {
+		    if(opt.max_output && (count+=len)>opt.max_output)
+		      {
 			log_error("Error writing to `%s': %s\n",
-				  fname, strerror(errno) );
+				  fname,"exceeded --max-output limit\n");
 			rc = G10ERR_WRITE_FILE;
 			m_free( buffer );
 			goto leave;
+		      }
+		    else if( fwrite( buffer, 1, len, fp ) != len ) {
+		      log_error("Error writing to `%s': %s\n",
+				fname, strerror(errno) );
+		      rc = G10ERR_WRITE_FILE;
+		      m_free( buffer );
+		      goto leave;
 		    }
-		}
+		  }
 	    }
 	    m_free( buffer );
 	}
@@ -223,14 +321,23 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	int state = 0;
 
 	while( (c = iobuf_get(pt->buf)) != -1 ) {
-	    if( fp ) {
-		if( putc( c, fp ) == EOF ) {
+	    if( fp )
+	      {
+		if(opt.max_output && (++count)>opt.max_output)
+		  {
 		    log_error("Error writing to `%s': %s\n",
-						fname, strerror(errno) );
+			      fname,"exceeded --max-output limit\n");
 		    rc = G10ERR_WRITE_FILE;
 		    goto leave;
-		}
-	    }
+		  }
+		else if( putc( c, fp ) == EOF )
+		  {
+		    log_error("Error writing to `%s': %s\n",
+			      fname, strerror(errno) );
+		    rc = G10ERR_WRITE_FILE;
+		    goto leave;
+		  }
+	      }
 	    if( !mfx->md )
 		continue;
 	    if( state == 2 ) {
@@ -347,15 +454,22 @@ ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
 		goto leave;
 	    }
 	    fp = iobuf_open(answer);
+            if (fp && is_secured_file (iobuf_get_fd (fp)))
+              {
+                iobuf_close (fp);
+                fp = NULL;
+                errno = EPERM;
+              }
 	    if( !fp && errno == ENOENT ) {
 		tty_printf("No such file, try again or hit enter to quit.\n");
 		any++;
 	    }
-	    else if( !fp ) {
-		log_error("can't open `%s': %s\n", answer, strerror(errno) );
+	    else if( !fp )
+	      {
+		log_error(_("can't open `%s': %s\n"), answer, strerror(errno));
 		rc = G10ERR_READ_FILE;
 		goto leave;
-	    }
+	      }
 	} while( !fp );
     }
 
@@ -402,6 +516,12 @@ hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 
     for (sl=files; sl; sl = sl->next ) {
 	fp = iobuf_open( sl->d );
+        if (fp && is_secured_file (iobuf_get_fd (fp)))
+          {
+            iobuf_close (fp);
+            fp = NULL;
+            errno = EPERM;
+          }
 	if( !fp ) {
 	    log_error(_("can't open signed data `%s'\n"),
 						print_fname_stdin(sl->d));
@@ -414,8 +534,3 @@ hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 
     return 0;
 }
-
-
-
-
-
