@@ -1,5 +1,6 @@
 /* revoke.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -89,7 +90,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
   node=find_kbnode(keyblock,PKT_PUBLIC_KEY);
   if(!node)
     {
-      log_error(_("key incomplete\n"));
+      log_error("key incomplete\n");
       return G10ERR_GENERAL;
     }
 
@@ -142,7 +143,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
 	    break;
 	  else
 	    {
-	      log_error(_("key %08lX incomplete\n"),(ulong)keyid[1]);
+	      log_error(_("key %s has no user IDs\n"),keystr(keyid));
 	      return G10ERR_GENERAL;
 	    }
 	}
@@ -210,10 +211,11 @@ gen_desig_revoke( const char *uname )
     u32 keyid[2];
     int i,any=0;
 
-    if( opt.batch ) {
-	log_error(_("sorry, can't do this in batch mode\n"));
+    if( opt.batch )
+      {
+	log_error(_("can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-    }
+      }
 
     memset( &afx, 0, sizeof afx);
 
@@ -221,7 +223,7 @@ gen_desig_revoke( const char *uname )
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
     if (rc) {
-	log_error (_("key `%s' not found: %s\n"),uname, g10_errstr (rc));
+	log_error (_("key \"%s\" not found: %s\n"),uname, g10_errstr (rc));
 	goto leave;
     }
 
@@ -260,41 +262,22 @@ gen_desig_revoke( const char *uname )
 	/* We have the revocation key */
 	if(!rc)
 	  {
-	    size_t n;
-	    char *p;
-	    u32 sk_keyid[2];
-	    PKT_signature *revkey=NULL;
+	    PKT_signature *revkey = NULL;
 
-	    any=1;
-	    keyid_from_sk(sk,sk_keyid);
+	    any = 1;
 
-	    tty_printf("\npub  %4u%c/%08lX %s   ",
-		       nbits_from_pk( pk ),
-		       pubkey_letter( pk->pubkey_algo ),
-		       (ulong)keyid[1], datestr_from_pk(pk) );
+            print_pubkey_info (NULL, pk);
+	    tty_printf ("\n");
 
-	    p = get_user_id( keyid, &n );
-	    tty_print_utf8_string( p, n );
-	    m_free(p);
-	    tty_printf("\n\n");
+	    tty_printf (_("To be revoked by:\n"));
+            print_seckey_info (sk);
 
-	    tty_printf(_("To be revoked by:\n"));
-
-	    tty_printf("\nsec  %4u%c/%08lX %s   ",
-		       nbits_from_sk( sk ),
-		       pubkey_letter( sk->pubkey_algo ),
-		       (ulong)sk_keyid[1], datestr_from_sk(sk) );
-
-	    p = get_user_id( sk_keyid, &n );
-	    tty_print_utf8_string( p, n );
-	    m_free(p);
-	    tty_printf("\n");
 	    if(pk->revkey[i].class&0x40)
 	      tty_printf(_("(This is a sensitive revocation key)\n"));
 	    tty_printf("\n");
 
 	    if( !cpr_get_answer_is_yes("gen_desig_revoke.okay",
-		       _("Create a revocation certificate for this key? ")) )
+         _("Create a designated revocation certificate for this key? (y/N) ")))
 	      continue;
 
 	    /* get the reason for the revocation (this is always v4) */
@@ -313,7 +296,7 @@ gen_desig_revoke( const char *uname )
 	      goto leave;
 
 	    afx.what = 1;
-	    afx.hdrlines = "Comment: A revocation certificate should follow\n";
+	    afx.hdrlines = "Comment: A designated revocation certificate should follow\n";
 	    iobuf_push_filter( out, armor_filter, &afx );
 
 	    /* create it */
@@ -390,7 +373,7 @@ gen_desig_revoke( const char *uname )
       }
 
     if(!any)
-      log_error(_("no revocation keys found for `%s'\n"),uname);
+      log_error(_("no revocation keys found for \"%s\"\n"),uname);
 
   leave:
     if( pk )
@@ -429,10 +412,11 @@ gen_revoke( const char *uname )
     struct revocation_reason_info *reason = NULL;
     KEYDB_SEARCH_DESC desc;
 
-    if( opt.batch ) {
-	log_error(_("sorry, can't do this in batch mode\n"));
+    if( opt.batch )
+      {
+	log_error(_("can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-    }
+      }
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
@@ -443,11 +427,12 @@ gen_revoke( const char *uname )
     kdbhd = keydb_new (1);
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
-    if (rc) {
-	log_error (_("secret key `%s' not found: %s\n"),
+    if (rc)
+      {
+	log_error (_("secret key \"%s\" not found: %s\n"),
                    uname, g10_errstr (rc));
 	goto leave;
-    }
+      }
 
     rc = keydb_get_keyblock (kdbhd, &keyblock );
     if( rc ) {
@@ -464,17 +449,8 @@ gen_revoke( const char *uname )
      * it's used all over the source */
     sk = node->pkt->pkt.secret_key;
     keyid_from_sk( sk, sk_keyid );
-    tty_printf("\nsec  %4u%c/%08lX %s   ",
-	      nbits_from_sk( sk ),
-	      pubkey_letter( sk->pubkey_algo ),
-	      (ulong)sk_keyid[1], datestr_from_sk(sk) );
-    {
-	size_t n;
-	char *p = get_user_id( sk_keyid, &n );
-	tty_print_utf8_string( p, n );
-	m_free(p);
-	tty_printf("\n");
-    }
+    print_seckey_info (sk);
+
     pk = m_alloc_clear( sizeof *pk );
 
     /* FIXME: We should get the public key direct from the secret one */
@@ -500,10 +476,11 @@ gen_revoke( const char *uname )
 
     tty_printf("\n");
     if( !cpr_get_answer_is_yes("gen_revoke.okay",
-			_("Create a revocation certificate for this key? ")) ){
+		  _("Create a revocation certificate for this key? (y/N) ")) )
+      {
 	rc = 0;
 	goto leave;
-    }
+      }
 
     if(sk->version>=4 || opt.force_v4_certs) {
       /* get the reason for the revocation */
@@ -549,7 +526,7 @@ gen_revoke( const char *uname )
 	goto leave;
     }
 
-    if(opt.pgp2 || opt.pgp6 || opt.pgp7 | opt.pgp8)
+    if(PGP2 || PGP6 || PGP7 || PGP8)
       {
 	/* Use a minimal pk for PGPx mode, since PGP can't import bare
 	   revocation certificates. */
@@ -608,6 +585,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
     const char *code_text = NULL;
 
     do {
+        code=-1;
 	m_free(description);
 	description = NULL;
 
@@ -635,7 +613,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	      return NULL; /* cancel */
 	    if( hint && !*answer )
 		n = hint;
-	    else if(!isdigit( *answer ) )
+	    else if(!digitp( answer ) )
  	        n = -1;
 	    else
 		n = atoi(answer);
@@ -699,7 +677,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    tty_printf("%s\n", description );
 
     } while( !cpr_get_answer_is_yes("ask_revocation_reason.okay",
-					    _("Is this okay? "))  );
+					    _("Is this okay? (y/N) "))  );
 
     reason = m_alloc( sizeof *reason );
     reason->code = code;
