@@ -1,6 +1,6 @@
 /* sign.c - sign data
- * Copyright (C) 1998, 1999, 2000, 2001, 2002,
- *               2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -40,6 +40,7 @@
 #include "trustdb.h"
 #include "status.h"
 #include "i18n.h"
+#include "cardglue.h"
 
 
 #ifdef HAVE_DOSISH_SYSTEM
@@ -58,7 +59,7 @@ static int recipient_digest_algo=0;
  * NAME=VALUE format.
  */
 static void
-mk_notation_and_policy( PKT_signature *sig,
+mk_notation_policy_etc( PKT_signature *sig,
 			PKT_public_key *pk, PKT_secret_key *sk )
 {
     const char *string;
@@ -155,7 +156,7 @@ mk_notation_and_policy( PKT_signature *sig,
 	s=pct_expando(string,&args);
 	if(!s)
 	  {
-	    log_error(_("WARNING: unable to %%-expand policy url "
+	    log_error(_("WARNING: unable to %%-expand policy URL "
 			"(too large).  Using unexpanded.\n"));
 	    s=m_strdup(string);
 	  }
@@ -166,6 +167,34 @@ mk_notation_and_policy( PKT_signature *sig,
 
 	m_free(s);
       }
+
+    /* preferred keyserver URL */
+    if( IS_SIG(sig) && opt.sig_keyserver_url )
+      {
+	if(sig->version<4)
+	  log_info("can't put a preferred keyserver URL into v3 signatures\n");
+	else
+	  pu=opt.sig_keyserver_url;
+      }
+
+    for(;pu;pu=pu->next)
+      {
+        string = pu->d;
+
+	s=pct_expando(string,&args);
+	if(!s)
+	  {
+	    log_error(_("WARNING: unable to %%-expand preferred keyserver URL"
+			" (too large).  Using unexpanded.\n"));
+	    s=m_strdup(string);
+	  }
+
+	build_sig_subpkt(sig,SIGSUBPKT_PREF_KS|
+			 ((pu->flags & 1)?SIGSUBPKT_FLAG_CRITICAL:0),
+			 s,strlen(s));
+
+	m_free(s);
+      }
 }
 
 
@@ -266,12 +295,6 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 	    return G10ERR_TIME_CONFLICT;
     }
 
-    /* For safety, only allow revocation sigs from Elgamal
-       sign+encrypt keys.  Note that this allows for Elgamal
-       designated revocations as well, but that's arguably a good
-       thing. */
-    if(is_ELGAMAL(sk->pubkey_algo) && sig->sig_class!=0x20)
-      return G10ERR_UNU_SECKEY;
 
     print_pubkey_algo_note(sk->pubkey_algo);
 
@@ -283,12 +306,41 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
     sig->digest_algo = digest_algo;
     sig->digest_start[0] = dp[0];
     sig->digest_start[1] = dp[1];
-    frame = encode_md_value( sk->pubkey_algo, md,
-			     digest_algo, mpi_get_nbits(sk->skey[0]), 0 );
-    if (!frame)
-        return G10ERR_GENERAL;
-    rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
-    mpi_free(frame);
+    if (sk->is_protected && sk->protect.s2k.mode == 1002) 
+      { 
+#ifdef ENABLE_CARD_SUPPORT
+        unsigned char *rbuf;
+        size_t rbuflen;
+        char *snbuf;
+        
+        snbuf = serialno_and_fpr_from_sk (sk->protect.iv,
+                                          sk->protect.ivlen, sk);
+        rc = agent_scd_pksign (snbuf, digest_algo,
+                               md_read (md, digest_algo),
+                               md_digest_length (digest_algo),
+                               &rbuf, &rbuflen);
+        xfree (snbuf);
+        if (!rc)
+          {
+            sig->data[0] = mpi_alloc ( (rbuflen+BYTES_PER_MPI_LIMB-1)
+                                       / BYTES_PER_MPI_LIMB );
+            mpi_set_buffer (sig->data[0], rbuf, rbuflen, 0);
+            xfree (rbuf);
+          }
+#else
+        return G10ERR_UNSUPPORTED;
+#endif /* ENABLE_CARD_SUPPORT */
+      }
+    else 
+      {
+        frame = encode_md_value( sk->pubkey_algo, md,
+                                 digest_algo, mpi_get_nbits(sk->skey[0]) );
+        if (!frame)
+          return G10ERR_GENERAL;
+        rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
+        mpi_free(frame);
+      }
+
     if (!rc && !opt.no_sig_create_check) {
         /* check that the signature verification worked and nothing is
          * fooling us e.g. by a bug in the signature create
@@ -300,13 +352,12 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
         else {
             frame = encode_md_value (pk->pubkey_algo, md,
                                      sig->digest_algo,
-                                     mpi_get_nbits(pk->pkey[0]), 0);
+                                     mpi_get_nbits(pk->pkey[0]) );
             if (!frame)
                 rc = G10ERR_GENERAL;
             else
                 rc = pubkey_verify (pk->pubkey_algo, frame,
-                                    sig->data, pk->pkey,
-                                    NULL, NULL );
+                                    sig->data, pk->pkey );
             mpi_free (frame);
         }
         if (rc)
@@ -318,9 +369,11 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 	log_error(_("signing failed: %s\n"), g10_errstr(rc) );
     else {
 	if( opt.verbose ) {
-	    char *ustr = get_user_id_string_printable (sig->keyid);
-	    log_info(_("%s signature from: \"%s\"\n"),
-		      pubkey_algo_to_string(sk->pubkey_algo), ustr );
+	    char *ustr = get_user_id_string_native (sig->keyid);
+	    log_info(_("%s/%s signature from: \"%s\"\n"),
+		     pubkey_algo_to_string(sk->pubkey_algo),
+		     digest_algo_to_string(sig->digest_algo),
+		     ustr );
 	    m_free(ustr);
 	}
     }
@@ -328,7 +381,6 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 }
 
 
-
 int
 complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md )
 {
@@ -342,14 +394,38 @@ complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md )
 static int
 hash_for(int pubkey_algo, int packet_version )
 {
-    if( opt.def_digest_algo )
-	return opt.def_digest_algo;
-    if( recipient_digest_algo )
-        return recipient_digest_algo;
-    if( pubkey_algo == PUBKEY_ALGO_DSA )
-	return DIGEST_ALGO_SHA1;
-    if( pubkey_algo == PUBKEY_ALGO_RSA && packet_version < 4 )
-	return DIGEST_ALGO_MD5;
+  if( opt.def_digest_algo )
+    return opt.def_digest_algo;
+  else if( recipient_digest_algo )
+    return recipient_digest_algo;
+  else if(PGP2 && pubkey_algo == PUBKEY_ALGO_RSA && packet_version < 4 )
+    {
+      /* Old-style PGP only understands MD5 */
+      return DIGEST_ALGO_MD5;
+    }
+  else if( pubkey_algo == PUBKEY_ALGO_DSA )
+    {
+      /* We need a 160-bit hash for DSA, so we can't just take the first
+	 in the pref list */
+
+      if(opt.personal_digest_prefs)
+	{
+	  prefitem_t *prefs;
+
+	  for(prefs=opt.personal_digest_prefs;prefs->type;prefs++)
+	    if(md_digest_length(prefs->value)==20)
+	      return prefs->value;
+	}
+
+      return DIGEST_ALGO_SHA1;
+    }
+  else if( opt.personal_digest_prefs )
+    {
+      /* It's not DSA, so we can use whatever the first hash algorithm
+	 is in the pref list */
+      return opt.personal_digest_prefs[0].value;
+    }
+  else
     return DEFAULT_DIGEST_ALGO;
 }
 
@@ -453,7 +529,8 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
     if (!opt.no_literal) {
         if (fname || opt.set_filename) {
             char *s = make_basename (opt.set_filename? opt.set_filename
-                                                     : fname);
+                                                     : fname,
+                                     iobuf_get_real_fname(inp));
             pt = m_alloc (sizeof *pt + strlen(s) - 1);
             pt->namelen = strlen (s);
             memcpy (pt->name, s, pt->namelen);
@@ -466,11 +543,12 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
     }
 
     /* try to calculate the length of the data */
-    if (fname && *fname && !(*fname=='-' && !fname[1])) {
+    if ( !iobuf_is_pipe_filename (fname) && *fname )
+      {
         off_t tmpsize;
        
         if( !(tmpsize = iobuf_get_filelength(inp)) )
-            log_info (_("WARNING: `%s' is an empty file\n"), fname);
+	  log_info (_("WARNING: `%s' is an empty file\n"), fname);
 
         /* We can't encode the length of very large files because
            OpenPGP uses only 32 bit for file sizes.  So if the size of
@@ -486,11 +564,10 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
          * without a double read of the file - to avoid that
          * we simple use partial length packets. */
         if ( ptmode == 't' )
-            filesize = 0;
-    }
-    else {
-        filesize = opt.set_filesize? opt.set_filesize : 0; /* stdin */
-    }
+	  filesize = 0;
+      }
+    else
+      filesize = opt.set_filesize? opt.set_filesize : 0; /* stdin */
 
     if (!opt.no_literal) {
         PACKET pkt;
@@ -551,7 +628,8 @@ write_signature_packets (SK_LIST sk_list, IOBUF out, MD_HANDLE hash,
 	sig = m_alloc_clear (sizeof *sig);
 	if(opt.force_v3_sigs || RFC1991)
 	  sig->version=3;
-	else if(duration || opt.sig_policy_url || opt.sig_notation_data)
+	else if(duration || opt.sig_policy_url
+		|| opt.sig_notation_data || opt.sig_keyserver_url)
 	  sig->version=4;
 	else
 	  sig->version=sk->version;
@@ -570,7 +648,7 @@ write_signature_packets (SK_LIST sk_list, IOBUF out, MD_HANDLE hash,
 
 	if (sig->version >= 4)
 	    build_sig_subpkt_from_sig (sig);
-	mk_notation_and_policy (sig, NULL, sk);
+	mk_notation_policy_etc (sig, NULL, sk);
 
         hash_sigversion_to_magic (md, sig);
 	md_final (md);
@@ -648,6 +726,10 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( fname && filenames->next && (!detached || encryptflag) )
 	log_bug("multiple files can only be detached signed");
 
+    if(encryptflag==2
+       && (rc=setup_symkey(&efx.symkey_s2k,&efx.symkey_dek)))
+      goto leave;
+
     if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !RFC1991)
       duration=ask_expire_interval(1);
 
@@ -668,8 +750,15 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( multifile )  /* have list of filenames */
 	inp = NULL; /* we do it later */
     else {
-        if( !(inp = iobuf_open(fname)) ) {
-	    log_error("can't open %s: %s\n", fname? fname: "[stdin]",
+      inp = iobuf_open(fname);
+      if (inp && is_secured_file (iobuf_get_fd (inp)))
+        {
+          iobuf_close (inp);
+          inp = NULL;
+          errno = EPERM;
+        }
+      if( !inp ) {
+	    log_error(_("can't open `%s': %s\n"), fname? fname: "[stdin]",
 		      strerror(errno) );
 	    rc = G10ERR_OPEN_FILE;
 	    goto leave;
@@ -679,11 +768,18 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     }
 
     if( outfile ) {
-	if( !(out = iobuf_create( outfile )) ) {
-	    log_error(_("can't create %s: %s\n"), outfile, strerror(errno) );
+        if (is_secured_filename ( outfile )) {
+            out = NULL;
+            errno = EPERM;
+        }
+        else
+            out = iobuf_create( outfile );
+	if( !out )
+	  {
+	    log_error(_("can't create `%s': %s\n"), outfile, strerror(errno) );
 	    rc = G10ERR_CREATE_FILE;
 	    goto leave;
-	}
+	  }
 	else if( opt.verbose )
 	    log_info(_("writing to `%s'\n"), outfile );
     }
@@ -709,8 +805,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	       select_algo_from_prefs(pk_list,PREFTYPE_HASH,
 				      opt.def_digest_algo,
 				      NULL)!=opt.def_digest_algo)
-	  log_info(_("forcing digest algorithm %s (%d) "
-		     "violates recipient preferences\n"),
+	  log_info(_("WARNING: forcing digest algorithm %s (%d)"
+		     " violates recipient preferences\n"),
 		   digest_algo_to_string(opt.def_digest_algo),
 		   opt.def_digest_algo);
 	  }
@@ -757,9 +853,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_push_filter( out, encrypt_filter, &efx );
     }
 
-    if( opt.compress && !outfile && ( !detached || opt.compress_sigs) )
+    if( opt.compress_algo && !outfile && ( !detached || opt.compress_sigs) )
       {
-        int compr_algo=opt.def_compress_algo;
+        int compr_algo=opt.compress_algo;
 
 	/* If not forced by user */
 	if(compr_algo==-1)
@@ -773,13 +869,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
  
 	    if((compr_algo=
 		select_algo_from_prefs(pk_list,PREFTYPE_ZIP,-1,NULL))==-1)
-	      compr_algo=DEFAULT_COMPRESS_ALGO;
+	      compr_algo=default_compress_algo();
 	  }
  	else if(!opt.expert && pk_list
  		&& select_algo_from_prefs(pk_list,PREFTYPE_ZIP,
 					  compr_algo,NULL)!=compr_algo)
- 	  log_info(_("forcing compression algorithm %s (%d) "
- 		     "violates recipient preferences\n"),
+ 	  log_info(_("WARNING: forcing compression algorithm %s (%d)"
+ 		     " violates recipient preferences\n"),
  		   compress_algo_to_string(compr_algo),compr_algo);
 
 	/* algo 0 means no compression */
@@ -805,12 +901,20 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    /* must walk reverse trough this list */
 	    for( sl = strlist_last(filenames); sl;
 			sl = strlist_prev( filenames, sl ) ) {
-		if( !(inp = iobuf_open(sl->d)) ) {
-		    log_error(_("can't open %s: %s\n"),
-					    sl->d, strerror(errno) );
+                inp = iobuf_open(sl->d);
+                if (inp && is_secured_file (iobuf_get_fd (inp)))
+                  {
+                    iobuf_close (inp);
+                    inp = NULL;
+                    errno = EPERM;
+                  }
+		if( !inp )
+		  {
+		    log_error(_("can't open `%s': %s\n"),
+			      sl->d,strerror(errno));
 		    rc = G10ERR_OPEN_FILE;
 		    goto leave;
-		}
+		  }
                 handle_progress (&pfx, inp, sl->d);
 		if( opt.verbose )
 		    fprintf(stderr, " `%s'", sl->d );
@@ -906,8 +1010,15 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
       }
 
     /* prepare iobufs */
-    if( !(inp = iobuf_open(fname)) ) {
-	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
+    inp = iobuf_open(fname);
+    if (inp && is_secured_file (iobuf_get_fd (inp)))
+      {
+        iobuf_close (inp);
+        inp = NULL;
+        errno = EPERM;
+      }
+    if( !inp ) {
+	log_error(_("can't open `%s': %s\n"), fname? fname: "[stdin]",
 					strerror(errno) );
 	rc = G10ERR_OPEN_FILE;
 	goto leave;
@@ -915,11 +1026,18 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     handle_progress (&pfx, inp, fname);
 
     if( outfile ) {
-	if( !(out = iobuf_create( outfile )) ) {
-	    log_error(_("can't create %s: %s\n"), outfile, strerror(errno) );
+        if (is_secured_filename (outfile) ) {
+            outfile = NULL;
+            errno = EPERM;
+        }
+        else 
+            out = iobuf_create( outfile );
+	if( !out )
+	  {
+	    log_error(_("can't create `%s': %s\n"), outfile, strerror(errno) );
 	    rc = G10ERR_CREATE_FILE;
 	    goto leave;
-	}
+	  }
 	else if( opt.verbose )
 	    log_info(_("writing to `%s'\n"), outfile );
     }
@@ -1038,9 +1156,15 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* prepare iobufs */
     inp = iobuf_open(fname);
+    if (inp && is_secured_file (iobuf_get_fd (inp)))
+      {
+        iobuf_close (inp);
+        inp = NULL;
+        errno = EPERM;
+      }
     if( !inp ) {
-	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
-					strerror(errno) );
+	log_error(_("can't open `%s': %s\n"), 
+                  fname? fname: "[stdin]", strerror(errno) );
 	rc = G10ERR_OPEN_FILE;
 	goto leave;
     }
@@ -1049,14 +1173,13 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     /* prepare key */
     s2k = m_alloc_clear( sizeof *s2k );
     s2k->mode = RFC1991? 0:opt.s2k_mode;
-    s2k->hash_algo = opt.def_digest_algo ? opt.def_digest_algo
-	                                 : opt.s2k_digest_algo;
+    s2k->hash_algo = S2K_DIGEST_ALGO;
 
-    algo = opt.def_cipher_algo ? opt.def_cipher_algo : opt.s2k_cipher_algo;
+    algo = default_cipher_algo();
     if (!opt.quiet || !opt.batch)
         log_info (_("%s encryption will be used\n"),
 		    cipher_algo_to_string(algo) );
-    cfx.dek = passphrase_to_dek( NULL, 0, algo, s2k, 2, NULL, NULL );
+    cfx.dek = passphrase_to_dek( NULL, 0, algo, s2k, 2, NULL, NULL);
 
     if (!cfx.dek || !cfx.dek->keylen) {
         rc = G10ERR_PASSPHRASE;
@@ -1109,9 +1232,9 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     /* Push the encryption filter */
     iobuf_push_filter( out, cipher_filter, &cfx );
 
-    /* Push the Zip filter */
-    if (opt.compress)
-      push_compress_filter(out,&zfx,opt.def_compress_algo);
+    /* Push the compress filter */
+    if (default_compress_algo())
+      push_compress_filter(out,&zfx,default_compress_algo());
 
     /* Write the one-pass signature packets */
     /*(current filters: zip - encrypt - armor)*/
@@ -1245,7 +1368,7 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
     sig->sig_class = sigclass;
     if( sig->version >= 4 )
 	build_sig_subpkt_from_sig( sig );
-    mk_notation_and_policy( sig, pk, sk );
+    mk_notation_policy_etc( sig, pk, sk );
 
     /* Crucial that the call to mksubpkt comes LAST before the calls
        to finalize the sig as that makes it possible for the mksubpkt
