@@ -1,5 +1,6 @@
 /* status.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,19 +27,19 @@
 #include <unistd.h>
 #include <signal.h>
 #ifdef USE_SHM_COPROCESSING
-  #ifdef USE_CAPABILITIES
-    #include <sys/capability.h>
-  #endif
-  #ifdef HAVE_SYS_IPC_H
-    #include <sys/types.h>
-    #include <sys/ipc.h>
-  #endif
-  #ifdef HAVE_SYS_SHM_H
-    #include <sys/shm.h>
-  #endif
-  #if defined(HAVE_MLOCK)
-    #include <sys/mman.h>
-  #endif
+#ifdef USE_CAPABILITIES
+#include <sys/capability.h>
+#endif
+#ifdef HAVE_SYS_IPC_H
+#include <sys/types.h>
+#include <sys/ipc.h>
+#endif
+#ifdef HAVE_SYS_SHM_H
+#include <sys/shm.h>
+#endif
+#if defined(HAVE_MLOCK)
+#include <sys/mman.h>
+#endif
 #endif
 #include "util.h"
 #include "status.h"
@@ -77,83 +78,129 @@ progress_cb ( void *ctx, int c )
 static const char *
 get_status_string ( int no ) 
 {
-    const char *s;
+  const char *s;
 
-    switch( no ) {
-      case STATUS_ENTER  : s = "ENTER"; break;
-      case STATUS_LEAVE  : s = "LEAVE"; break;
-      case STATUS_ABORT  : s = "ABORT"; break;
-      case STATUS_GOODSIG: s = "GOODSIG"; break;
-      case STATUS_KEYEXPIRED: s = "KEYEXPIRED"; break;
-      case STATUS_KEYREVOKED: s = "KEYREVOKED"; break;
-      case STATUS_BADSIG : s = "BADSIG"; break;
-      case STATUS_ERRSIG : s = "ERRSIG"; break;
-      case STATUS_BADARMOR : s = "BADARMOR"; break;
-      case STATUS_RSA_OR_IDEA : s= "RSA_OR_IDEA"; break;
-      case STATUS_TRUST_UNDEFINED: s = "TRUST_UNDEFINED"; break;
-      case STATUS_TRUST_NEVER	 : s = "TRUST_NEVER"; break;
-      case STATUS_TRUST_MARGINAL : s = "TRUST_MARGINAL"; break;
-      case STATUS_TRUST_FULLY	 : s = "TRUST_FULLY"; break;
-      case STATUS_TRUST_ULTIMATE : s = "TRUST_ULTIMATE"; break;
-      case STATUS_GET_BOOL	 : s = "GET_BOOL"; break;
-      case STATUS_GET_LINE	 : s = "GET_LINE"; break;
-      case STATUS_GET_HIDDEN	 : s = "GET_HIDDEN"; break;
-      case STATUS_GOT_IT	 : s = "GOT_IT"; break;
-      case STATUS_SHM_INFO	 : s = "SHM_INFO"; break;
-      case STATUS_SHM_GET	 : s = "SHM_GET"; break;
-      case STATUS_SHM_GET_BOOL	 : s = "SHM_GET_BOOL"; break;
-      case STATUS_SHM_GET_HIDDEN : s = "SHM_GET_HIDDEN"; break;
-      case STATUS_NEED_PASSPHRASE: s = "NEED_PASSPHRASE"; break;
-      case STATUS_VALIDSIG	 : s = "VALIDSIG"; break;
-      case STATUS_SIG_ID	 : s = "SIG_ID"; break;
-      case STATUS_ENC_TO	 : s = "ENC_TO"; break;
-      case STATUS_NODATA	 : s = "NODATA"; break;
-      case STATUS_BAD_PASSPHRASE : s = "BAD_PASSPHRASE"; break;
-      case STATUS_NO_PUBKEY	 : s = "NO_PUBKEY"; break;
-      case STATUS_NO_SECKEY	 : s = "NO_SECKEY"; break;
-      case STATUS_NEED_PASSPHRASE_SYM: s = "NEED_PASSPHRASE_SYM"; break;
-      case STATUS_DECRYPTION_FAILED: s = "DECRYPTION_FAILED"; break;
-      case STATUS_DECRYPTION_OKAY: s = "DECRYPTION_OKAY"; break;
-      case STATUS_MISSING_PASSPHRASE: s = "MISSING_PASSPHRASE"; break;
-      case STATUS_GOOD_PASSPHRASE : s = "GOOD_PASSPHRASE"; break;
-      case STATUS_GOODMDC	 : s = "GOODMDC"; break;
-      case STATUS_BADMDC	 : s = "BADMDC"; break;
-      case STATUS_ERRMDC	 : s = "ERRMDC"; break;
-      case STATUS_IMPORTED	 : s = "IMPORTED"; break;
-      case STATUS_IMPORT_OK	 : s = "IMPORT_OK"; break;
-      case STATUS_IMPORT_CHECK   : s = "IMPORT_CHECK"; break;
-      case STATUS_IMPORT_RES	 : s = "IMPORT_RES"; break;
-      case STATUS_FILE_START	 : s = "FILE_START"; break;
-      case STATUS_FILE_DONE	 : s = "FILE_DONE"; break;
-      case STATUS_FILE_ERROR	 : s = "FILE_ERROR"; break;
-      case STATUS_BEGIN_DECRYPTION:s = "BEGIN_DECRYPTION"; break;
-      case STATUS_END_DECRYPTION : s = "END_DECRYPTION"; break;
-      case STATUS_BEGIN_ENCRYPTION:s = "BEGIN_ENCRYPTION"; break;
-      case STATUS_END_ENCRYPTION : s = "END_ENCRYPTION"; break;
-      case STATUS_DELETE_PROBLEM : s = "DELETE_PROBLEM"; break;
-      case STATUS_PROGRESS	 : s = "PROGRESS"; break;
-      case STATUS_SIG_CREATED	 : s = "SIG_CREATED"; break;
-      case STATUS_SESSION_KEY	 : s = "SESSION_KEY"; break;
-      case STATUS_NOTATION_NAME  : s = "NOTATION_NAME" ; break;
-      case STATUS_NOTATION_DATA  : s = "NOTATION_DATA" ; break;
-      case STATUS_POLICY_URL     : s = "POLICY_URL" ; break;
-      case STATUS_BEGIN_STREAM   : s = "BEGIN_STREAM"; break;
-      case STATUS_END_STREAM     : s = "END_STREAM"; break;
-      case STATUS_KEY_CREATED    : s = "KEY_CREATED"; break;
-      case STATUS_USERID_HINT    : s = "USERID_HINT"; break;
-      case STATUS_UNEXPECTED     : s = "UNEXPECTED"; break;
-      case STATUS_INV_RECP       : s = "INV_RECP"; break;
-      case STATUS_NO_RECP        : s = "NO_RECP"; break;
-      case STATUS_ALREADY_SIGNED : s = "ALREADY_SIGNED"; break;
-      case STATUS_SIGEXPIRED     : s = "SIGEXPIRED deprecated-use-keyexpired-instead"; break;
-      case STATUS_EXPSIG         : s = "EXPSIG"; break;
-      case STATUS_EXPKEYSIG      : s = "EXPKEYSIG"; break;
-      case STATUS_ATTRIBUTE      : s = "ATTRIBUTE"; break;
-      default: s = "?"; break;
+  switch( no )
+    {
+    case STATUS_ENTER  : s = "ENTER"; break;
+    case STATUS_LEAVE  : s = "LEAVE"; break;
+    case STATUS_ABORT  : s = "ABORT"; break;
+    case STATUS_NEWSIG : s = "NEWSIG"; break;
+    case STATUS_GOODSIG: s = "GOODSIG"; break;
+    case STATUS_KEYEXPIRED: s = "KEYEXPIRED"; break;
+    case STATUS_KEYREVOKED: s = "KEYREVOKED"; break;
+    case STATUS_BADSIG : s = "BADSIG"; break;
+    case STATUS_ERRSIG : s = "ERRSIG"; break;
+    case STATUS_BADARMOR : s = "BADARMOR"; break;
+    case STATUS_RSA_OR_IDEA : s= "RSA_OR_IDEA"; break;
+    case STATUS_TRUST_UNDEFINED: s = "TRUST_UNDEFINED"; break;
+    case STATUS_TRUST_NEVER	 : s = "TRUST_NEVER"; break;
+    case STATUS_TRUST_MARGINAL : s = "TRUST_MARGINAL"; break;
+    case STATUS_TRUST_FULLY	 : s = "TRUST_FULLY"; break;
+    case STATUS_TRUST_ULTIMATE : s = "TRUST_ULTIMATE"; break;
+    case STATUS_GET_BOOL	 : s = "GET_BOOL"; break;
+    case STATUS_GET_LINE	 : s = "GET_LINE"; break;
+    case STATUS_GET_HIDDEN	 : s = "GET_HIDDEN"; break;
+    case STATUS_GOT_IT	 : s = "GOT_IT"; break;
+    case STATUS_SHM_INFO	 : s = "SHM_INFO"; break;
+    case STATUS_SHM_GET	 : s = "SHM_GET"; break;
+    case STATUS_SHM_GET_BOOL	 : s = "SHM_GET_BOOL"; break;
+    case STATUS_SHM_GET_HIDDEN : s = "SHM_GET_HIDDEN"; break;
+    case STATUS_NEED_PASSPHRASE: s = "NEED_PASSPHRASE"; break;
+    case STATUS_VALIDSIG	 : s = "VALIDSIG"; break;
+    case STATUS_SIG_ID	 : s = "SIG_ID"; break;
+    case STATUS_ENC_TO	 : s = "ENC_TO"; break;
+    case STATUS_NODATA	 : s = "NODATA"; break;
+    case STATUS_BAD_PASSPHRASE : s = "BAD_PASSPHRASE"; break;
+    case STATUS_BAD_PASSPHRASE_PIN: s = "BAD_PASSPHRASE_PIN"; break;
+    case STATUS_NO_PUBKEY	 : s = "NO_PUBKEY"; break;
+    case STATUS_NO_SECKEY	 : s = "NO_SECKEY"; break;
+    case STATUS_NEED_PASSPHRASE_SYM: s = "NEED_PASSPHRASE_SYM"; break;
+    case STATUS_NEED_PASSPHRASE_PIN: s = "NEED_PASSPHRASE_PIN"; break;
+    case STATUS_DECRYPTION_FAILED: s = "DECRYPTION_FAILED"; break;
+    case STATUS_DECRYPTION_OKAY: s = "DECRYPTION_OKAY"; break;
+    case STATUS_MISSING_PASSPHRASE: s = "MISSING_PASSPHRASE"; break;
+    case STATUS_GOOD_PASSPHRASE : s = "GOOD_PASSPHRASE"; break;
+    case STATUS_GOODMDC	 : s = "GOODMDC"; break;
+    case STATUS_BADMDC	 : s = "BADMDC"; break;
+    case STATUS_ERRMDC	 : s = "ERRMDC"; break;
+    case STATUS_IMPORTED	 : s = "IMPORTED"; break;
+    case STATUS_IMPORT_OK	 : s = "IMPORT_OK"; break;
+    case STATUS_IMPORT_CHECK   : s = "IMPORT_CHECK"; break;
+    case STATUS_IMPORT_RES	 : s = "IMPORT_RES"; break;
+    case STATUS_FILE_START	 : s = "FILE_START"; break;
+    case STATUS_FILE_DONE	 : s = "FILE_DONE"; break;
+    case STATUS_FILE_ERROR	 : s = "FILE_ERROR"; break;
+    case STATUS_BEGIN_DECRYPTION:s = "BEGIN_DECRYPTION"; break;
+    case STATUS_END_DECRYPTION : s = "END_DECRYPTION"; break;
+    case STATUS_BEGIN_ENCRYPTION:s = "BEGIN_ENCRYPTION"; break;
+    case STATUS_END_ENCRYPTION : s = "END_ENCRYPTION"; break;
+    case STATUS_DELETE_PROBLEM : s = "DELETE_PROBLEM"; break;
+    case STATUS_PROGRESS       : s = "PROGRESS"; break;
+    case STATUS_SIG_CREATED    : s = "SIG_CREATED"; break;
+    case STATUS_SESSION_KEY    : s = "SESSION_KEY"; break;
+    case STATUS_NOTATION_NAME  : s = "NOTATION_NAME" ; break;
+    case STATUS_NOTATION_DATA  : s = "NOTATION_DATA" ; break;
+    case STATUS_POLICY_URL     : s = "POLICY_URL" ; break;
+    case STATUS_BEGIN_STREAM   : s = "BEGIN_STREAM"; break;
+    case STATUS_END_STREAM     : s = "END_STREAM"; break;
+    case STATUS_KEY_CREATED    : s = "KEY_CREATED"; break;
+    case STATUS_KEY_NOT_CREATED: s = "KEY_NOT_CREATED"; break;
+    case STATUS_USERID_HINT    : s = "USERID_HINT"; break;
+    case STATUS_UNEXPECTED     : s = "UNEXPECTED"; break;
+    case STATUS_INV_RECP       : s = "INV_RECP"; break;
+    case STATUS_NO_RECP        : s = "NO_RECP"; break;
+    case STATUS_ALREADY_SIGNED : s = "ALREADY_SIGNED"; break;
+    case STATUS_SIGEXPIRED     : s = "SIGEXPIRED deprecated-use-keyexpired-instead"; break;
+    case STATUS_EXPSIG         : s = "EXPSIG"; break;
+    case STATUS_EXPKEYSIG      : s = "EXPKEYSIG"; break;
+    case STATUS_REVKEYSIG      : s = "REVKEYSIG"; break;
+    case STATUS_ATTRIBUTE      : s = "ATTRIBUTE"; break;
+    case STATUS_CARDCTRL       : s = "CARDCTRL"; break;
+    case STATUS_PLAINTEXT      : s = "PLAINTEXT"; break;
+    case STATUS_PLAINTEXT_LENGTH:s = "PLAINTEXT_LENGTH"; break;
+    case STATUS_SIG_SUBPACKET  : s = "SIG_SUBPACKET"; break;
+    case STATUS_SC_OP_SUCCESS  : s = "SC_OP_SUCCESS"; break;
+    case STATUS_SC_OP_FAILURE  : s = "SC_OP_FAILURE"; break;
+    case STATUS_BACKUP_KEY_CREATED:s="BACKUP_KEY_CREATED"; break;
+    default: s = "?"; break;
     }
-    return s;
+  return s;
 }
 
+
+/* Return true if the status message NO may currently be issued.  We
+   need this to avoid syncronisation problem while auto retrieving a
+   key.  There it may happen that a status NODATA is issued for a non
+   available key and the user may falsely interpret this has a missing
+   signature. */
+static int
+status_currently_allowed (int no)
+{
+  if (!glo_ctrl.in_auto_key_retrieve)
+    return 1; /* Yes. */
+
+  /* We allow some statis anyway, so that import statistics are
+     correct and to avoid problems if the retriebval subsystem will
+     prompt the user. */
+  switch (no)
+    {
+    case STATUS_GET_BOOL:	 
+    case STATUS_GET_LINE:	 
+    case STATUS_GET_HIDDEN:	 
+    case STATUS_GOT_IT:	 
+    case STATUS_IMPORTED:
+    case STATUS_IMPORT_OK:	
+    case STATUS_IMPORT_CHECK:  
+    case STATUS_IMPORT_RES:
+      return 1; /* Yes. */
+    default:
+      break;
+    }
+  return 0; /* No. */
+}
+
+
 void
 set_status_fd ( int fd )
 {
@@ -199,8 +246,8 @@ write_status ( int no )
 void
 write_status_text ( int no, const char *text)
 {
-    if( !statusfp )
-	return;  /* not enabled */
+    if( !statusfp || !status_currently_allowed (no) )
+	return;  /* Not enabled or allowed. */
 
     fputs ( "[GNUPG:] ", statusfp );
     fputs ( get_status_string (no), statusfp );
@@ -235,8 +282,8 @@ write_status_text_and_buffer ( int no, const char *string,
     int lower_limit = ' ';
     size_t n, count, dowrap;
 
-    if( !statusfp )
-	return;  /* not enabled */
+    if( !statusfp || !status_currently_allowed (no) )
+	return;  /* Not enabled or allowed. */
     
     if (wrap == -1) {
         lower_limit--;
@@ -290,7 +337,6 @@ write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
 }
 
 
-
 #ifdef USE_SHM_COPROCESSING
 
 #ifndef IPC_RMID_DEFERRED_RELEASE
@@ -310,9 +356,9 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
     char buf[100];
     struct shmid_ds shmds;
 
-  #ifndef IPC_RMID_DEFERRED_RELEASE
+#ifndef IPC_RMID_DEFERRED_RELEASE
     atexit( remove_shmid );
-  #endif
+#endif
     requested_shm_size = (requested_shm_size + 4095) & ~4095;
     if ( requested_shm_size > 2 * 4096 )
 	log_fatal("too much shared memory requested; only 8k are allowed\n");
@@ -323,7 +369,7 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
 	log_fatal("can't get %uk of shared memory: %s\n",
 				(unsigned)shm_size/1024, strerror(errno));
 
-  #if !defined(IPC_HAVE_SHM_LOCK) \
+#if !defined(IPC_HAVE_SHM_LOCK) \
       && defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
     /* part of the old code which uses mlock */
     shm_area = shmat( shm_id, 0, 0 );
@@ -333,25 +379,25 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
     log_debug("mapped %uk shared memory at %p, id=%d\n",
 			    (unsigned)shm_size/1024, shm_area, shm_id );
     if( lock_mem ) {
-      #ifdef USE_CAPABILITIES
+#ifdef USE_CAPABILITIES
 	cap_set_proc( cap_from_text("cap_ipc_lock+ep") );
-      #endif
+#endif
 	/* (need the cast for Solaris with Sun's workshop compilers) */
 	if ( mlock ( (char*)shm_area, shm_size) )
 	    log_info("locking shared memory %d failed: %s\n",
 				shm_id, strerror(errno));
 	else
 	    shm_is_locked = 1;
-      #ifdef USE_CAPABILITIES
+#ifdef USE_CAPABILITIES
 	cap_set_proc( cap_from_text("cap_ipc_lock+p") );
-      #endif
+#endif
     }
 
-  #ifdef IPC_RMID_DEFERRED_RELEASE
+#ifdef IPC_RMID_DEFERRED_RELEASE
     if( shmctl( shm_id, IPC_RMID, 0) )
 	log_fatal("shmctl IPC_RMDID of %d failed: %s\n",
 					    shm_id, strerror(errno));
-  #endif
+#endif
 
     if( shmctl( shm_id, IPC_STAT, &shmds ) )
 	log_fatal("shmctl IPC_STAT of %d failed: %s\n",
@@ -363,27 +409,27 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
 						shm_id, strerror(errno));
     }
 
-  #else /* this is the new code which handles the changes in the SHM semantics
-	 * introduced with Linux 2.4.  The changes is that we now change the
-	 * permissions and then attach to the memory.
-	 */
+#else /* this is the new code which handles the changes in the SHM
+       * semantics introduced with Linux 2.4.  The changes is that we
+       * now change the permissions and then attach to the memory.
+       */
 
     if( lock_mem ) {
-      #ifdef USE_CAPABILITIES
+#ifdef USE_CAPABILITIES
 	cap_set_proc( cap_from_text("cap_ipc_lock+ep") );
-      #endif
-      #ifdef IPC_HAVE_SHM_LOCK
+#endif
+#ifdef IPC_HAVE_SHM_LOCK
 	if ( shmctl (shm_id, SHM_LOCK, 0) )
 	    log_info("locking shared memory %d failed: %s\n",
 				shm_id, strerror(errno));
 	else
 	    shm_is_locked = 1;
-      #else
+#else
 	log_info("Locking shared memory %d failed: No way to do it\n", shm_id );
-      #endif
-      #ifdef USE_CAPABILITIES
+#endif
+#ifdef USE_CAPABILITIES
 	cap_set_proc( cap_from_text("cap_ipc_lock+p") );
-      #endif
+#endif
     }
 
     if( shmctl( shm_id, IPC_STAT, &shmds ) )
@@ -403,13 +449,13 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
     log_debug("mapped %uk shared memory at %p, id=%d\n",
 			    (unsigned)shm_size/1024, shm_area, shm_id );
 
-  #ifdef IPC_RMID_DEFERRED_RELEASE
+#ifdef IPC_RMID_DEFERRED_RELEASE
     if( shmctl( shm_id, IPC_RMID, 0) )
 	log_fatal("shmctl IPC_RMDID of %d failed: %s\n",
 					    shm_id, strerror(errno));
-  #endif
+#endif
 
-  #endif
+#endif
     /* write info; Protocol version, id, size, locked size */
     sprintf( buf, "pv=1 pid=%d shmid=%d sz=%u lz=%u", (int)getpid(),
 	    shm_id, (unsigned)shm_size, shm_is_locked? (unsigned)shm_size:0 );
@@ -479,11 +525,11 @@ myread(int fd, void *buf, size_t count)
             eof_emmited++;
         }
         else { /* Ctrl-D not caught - do something reasonable */
-          #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
             raise (SIGINT);  /* nothing to hangup under DOS */
-          #else
+#else
             raise (SIGHUP); /* no more input data */
-          #endif
+#endif
         }
     }    
     return rc;
@@ -501,6 +547,9 @@ do_get_from_fd( const char *keyword, int hidden, int bool )
     int i, len;
     char *string;
 
+    if(statusfp!=stdout)
+      fflush(stdout);
+
     write_status_text( bool? STATUS_GET_BOOL :
 		       hidden? STATUS_GET_HIDDEN : STATUS_GET_LINE, keyword );
 
@@ -541,10 +590,10 @@ cpr_enabled()
 {
     if( opt.command_fd != -1 )
 	return 1;
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     if( opt.shm_coprocess )
 	return 1;
-  #endif
+#endif
     return 0;
 }
 
@@ -555,10 +604,10 @@ cpr_get_no_help( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 0, 0 );
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     if( opt.shm_coprocess )
 	return do_shm_get( keyword, 0, 0 );
-  #endif
+#endif
     for(;;) {
 	p = tty_get( prompt );
         return p;
@@ -572,10 +621,10 @@ cpr_get( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 0, 0 );
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     if( opt.shm_coprocess )
 	return do_shm_get( keyword, 0, 0 );
-  #endif
+#endif
     for(;;) {
 	p = tty_get( prompt );
 	if( *p=='?' && !p[1] && !(keyword && !*keyword)) {
@@ -608,10 +657,10 @@ cpr_get_hidden( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 1, 0 );
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     if( opt.shm_coprocess )
 	return do_shm_get( keyword, 1, 0 );
-  #endif
+#endif
     for(;;) {
 	p = tty_get_hidden( prompt );
 	if( *p == '?' && !p[1] ) {
@@ -628,10 +677,10 @@ cpr_kill_prompt(void)
 {
     if( opt.command_fd != -1 )
 	return;
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     if( opt.shm_coprocess )
 	return;
-  #endif
+#endif
     tty_kill_prompt();
     return;
 }
@@ -692,3 +741,45 @@ cpr_get_answer_yes_no_quit( const char *keyword, const char *prompt )
     }
 }
 
+
+int
+cpr_get_answer_okay_cancel (const char *keyword,
+                            const char *prompt,
+                            int def_answer)
+{
+  int yes;
+  char *answer = NULL;
+  char *p;
+
+  if( opt.command_fd != -1 )
+    answer = do_get_from_fd ( keyword, 0, 0 );
+#ifdef USE_SHM_COPROCESSING
+  else if( opt.shm_coprocess )
+    answer = do_shm_get( keyword, 0, 0 );
+#endif
+
+  if (answer)
+    {
+      yes = answer_is_okay_cancel (answer, def_answer);
+      m_free (answer);
+      return yes;
+    }
+
+  for(;;)
+    {
+      p = tty_get( prompt );
+      trim_spaces(p); /* it is okay to do this here */
+      if (*p == '?' && !p[1])
+        {
+          m_free(p);
+          display_online_help (keyword);
+	}
+      else
+        {
+          tty_kill_prompt();
+          yes = answer_is_okay_cancel (p, def_answer);
+          m_free(p);
+          return yes;
+	}
+    }
+}
