@@ -1,5 +1,5 @@
 /* textfilter.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -61,14 +61,6 @@ len_without_trailing_chars( byte *line, unsigned len, const char *trimchars )
     return mark? (mark - line) : len;
 }
 
-unsigned
-len_without_trailing_ws( byte *line, unsigned len )
-{
-    return len_without_trailing_chars( line, len, " \t\r\n" );
-}
-
-
-
 
 static int
 standard( text_filter_context_t *tfx, IOBUF a,
@@ -101,7 +93,30 @@ standard( text_filter_context_t *tfx, IOBUF a,
 	    break;
 	}
 	lf_seen = tfx->buffer[tfx->buffer_len-1] == '\n';
-	tfx->buffer_len = trim_trailing_ws( tfx->buffer, tfx->buffer_len );
+
+	/* The story behind this is that 2440 says that textmode
+	   hashes should canonicalize line endings to CRLF and remove
+	   spaces and tabs.  2440bis-12 says to just canonicalize to
+	   CRLF.  1.4.0 was released using the bis-12 behavior, but it
+	   was discovered that many mail clients do not canonicalize
+	   PGP/MIME signature text appropriately (and were relying on
+	   GnuPG to handle trailing spaces).  So, we default to the
+	   2440 behavior, but use the 2440bis-12 behavior if the user
+	   specifies --no-rfc2440-text.  The default will be changed
+	   at some point in the future when the mail clients have been
+	   upgraded.  Aside from PGP/MIME and broken mail clients,
+	   this makes no difference to any signatures in the real
+	   world except for a textmode detached signature.  PGP always
+	   used the 2440bis-12 behavior (ignoring 2440 itself), so
+	   this actually makes us compatible with PGP textmode
+	   detached signatures for the first time. */
+	if(opt.rfc2440_text)
+	  tfx->buffer_len=trim_trailing_chars(tfx->buffer,tfx->buffer_len,
+					      " \t\r\n");
+	else
+	  tfx->buffer_len=trim_trailing_chars(tfx->buffer,tfx->buffer_len,
+					      "\r\n");
+
 	if( lf_seen ) {
 	    tfx->buffer[tfx->buffer_len++] = '\r';
 	    tfx->buffer[tfx->buffer_len++] = '\n';
@@ -112,8 +127,6 @@ standard( text_filter_context_t *tfx, IOBUF a,
 }
 
 
-
-
 /****************
  * The filter is used to make canonical text: Lines are terminated by
  * CR, LF, trailing white spaces are removed.
