@@ -1,6 +1,6 @@
 /* trustdb.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
- *                                             Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,6 +25,15 @@
 #include <string.h>
 #include <assert.h>
 
+#ifndef DISABLE_REGEX
+#include <sys/types.h>
+#ifdef USE_GNU_REGEX
+#include "_regex.h"
+#else
+#include <regex.h>
+#endif
+#endif /* !DISABLE_REGEX */
+
 #include "errors.h"
 #include "iobuf.h"
 #include "keydb.h"
@@ -44,7 +53,10 @@
  */
 struct key_item {
   struct key_item *next;
-  unsigned int ownertrust;
+  unsigned int ownertrust,min_ownertrust;
+  byte trust_depth;
+  byte trust_value;
+  char *trust_regexp;
   u32 kid[2];
 };
 
@@ -98,6 +110,7 @@ release_key_items (struct key_item *k)
   for (; k; k = k2)
     {
       k2 = k->next;
+      m_free (k->trust_regexp);
       m_free (k);
     }
 }
@@ -191,22 +204,30 @@ release_key_array ( struct key_array *keys )
  * before initializing the validation module.
  * FIXME: Should be replaced by a function to add those keys to the trustdb.
  */
+void
+register_trusted_keyid(u32 *keyid)
+{
+  struct key_item *k;
+
+  k = new_key_item ();
+  k->kid[0] = keyid[0];
+  k->kid[1] = keyid[1];
+  k->next = user_utk_list;
+  user_utk_list = k;
+}
+
 void
 register_trusted_key( const char *string )
 {
   KEYDB_SEARCH_DESC desc;
-  struct key_item *k;
 
-  if (classify_user_id (string, &desc) != KEYDB_SEARCH_MODE_LONG_KID ) {
-    log_error(_("`%s' is not a valid long keyID\n"), string );
-    return;
-  }
+  if (classify_user_id (string, &desc) != KEYDB_SEARCH_MODE_LONG_KID )
+    {
+      log_error(_("`%s' is not a valid long keyID\n"), string );
+      return;
+    }
 
-  k = new_key_item ();
-  k->kid[0] = desc.u.kid[0];
-  k->kid[1] = desc.u.kid[1];
-  k->next = user_utk_list;
-  user_utk_list = k;
+  register_trusted_keyid(desc.u.kid);
 }
 
 /*
@@ -233,7 +254,7 @@ add_utk (u32 *kid)
   k->next = utk_list;
   utk_list = k;
   if( opt.verbose > 1 )
-    log_info(_("key %08lX: accepted as trusted key\n"), (ulong)kid[1]);
+    log_info(_("key %s: accepted as trusted key\n"), keystr(kid));
   return 1;
 }
 
@@ -271,8 +292,8 @@ verify_own_keys(void)
             fprlen = (!fpr[16] && !fpr[17] && !fpr[18] && !fpr[19])? 16:20;
             keyid_from_fingerprint (fpr, fprlen, kid);
             if (!add_utk (kid))
-                log_info(_("key %08lX occurs more than once in the trustdb\n"),
-                            (ulong)kid[1]);
+	      log_info(_("key %s occurs more than once in the trustdb\n"),
+		       keystr(kid));
         }
     }
 
@@ -285,22 +306,21 @@ verify_own_keys(void)
 
           memset (&pk, 0, sizeof pk);
           rc = get_pubkey (&pk, k->kid);
-          if (rc) {
-            log_info(_("key %08lX: no public key for trusted key - skipped\n"),
-                     (ulong)k->kid[1] );
-          }
-          else {
-            update_ownertrust (&pk,
-                               ((get_ownertrust (&pk) & ~TRUST_MASK)
-                                | TRUST_ULTIMATE ));
-            release_public_key_parts (&pk);
-          }
-          log_info (_("key %08lX marked as ultimately trusted\n"),
-                    (ulong)k->kid[1]);
+          if (rc)
+	    log_info(_("key %s: no public key for trusted key - skipped\n"),
+		     keystr(k->kid));
+          else
+	    {
+	      update_ownertrust (&pk,
+				 ((get_ownertrust (&pk) & ~TRUST_MASK)
+				  | TRUST_ULTIMATE ));
+	      release_public_key_parts (&pk);
+	    }
+
+          log_info (_("key %s marked as ultimately trusted\n"),keystr(k->kid));
         }
     }
 
-
   /* release the helper table table */
   release_key_items (user_utk_list);
   user_utk_list = NULL;
@@ -362,6 +382,19 @@ do_sync(void)
       }
 }
 
+static const char *
+trust_model_string(void)
+{
+  switch(opt.trust_model)
+    {
+    case TM_CLASSIC:  return "classic";
+    case TM_PGP:      return "PGP";
+    case TM_EXTERNAL: return "external";
+    case TM_ALWAYS:   return "always";
+    case TM_DIRECT:   return "direct";
+    default:          return "unknown";
+    }
+}
 
 /****************
  * Perform some checks over the trustdb
@@ -382,7 +415,6 @@ setup_trustdb( int level, const char *dbname )
 void
 init_trustdb()
 {
-  int rc=0;
   int level = trustdb_args.level;
   const char* dbname = trustdb_args.dbname;
 
@@ -391,31 +423,48 @@ init_trustdb()
 
   trustdb_args.init = 1;
 
-  if ( !level || level==1)
+  if(level==0 || level==1)
     {
-      rc = tdbio_set_dbname( dbname, !!level );
-      if( !rc )
-        {
-          if( !level )
-            return;
-          
-          /* verify that our own keys are in the trustDB
-           * or move them to the trustdb. */
-          verify_own_keys();
-          
-          /* should we check whether there is no other ultimately trusted
-           * key in the database? */
-        }
+      int rc = tdbio_set_dbname( dbname, !!level );
+      if( rc )
+	log_fatal("can't init trustdb: %s\n", g10_errstr(rc) );
     }
   else
     BUG();
-  if( rc )
-    log_fatal("can't init trustdb: %s\n", g10_errstr(rc) );
+
+  if(opt.trust_model==TM_AUTO)
+    {
+      /* Try and set the trust model off of whatever the trustdb says
+	 it is. */
+      opt.trust_model=tdbio_read_model();
+
+      /* Sanity check this ;) */
+      if(opt.trust_model!=TM_CLASSIC
+	 && opt.trust_model!=TM_PGP
+	 && opt.trust_model!=TM_EXTERNAL)
+	{
+	  log_info(_("unable to use unknown trust model (%d) - "
+		     "assuming %s trust model\n"),opt.trust_model,"PGP");
+	  opt.trust_model=TM_PGP;
+	}
+
+      if(opt.verbose)
+	log_info(_("using %s trust model\n"),trust_model_string());
+    }
+
+  if(opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC)
+    {
+      /* Verify the list of ultimately trusted keys and move the
+	 --trusted-keys list there as well. */
+      if(level==1)
+	verify_own_keys();
+
+      if(!tdbio_db_matches_options())
+	pending_check_trustdb=1;
+    }
 }
 
 
-
-
 /***********************************************
  *************	Print helpers	****************
  ***********************************************/
@@ -436,10 +485,75 @@ trust_letter (unsigned int value)
     case TRUST_MARGINAL:  return 'm';
     case TRUST_FULLY:     return 'f';
     case TRUST_ULTIMATE:  return 'u';
-    default:              return 0;
+    default:              return '?';
     }
 }
 
+/* NOTE TO TRANSLATOR: these strings are similar to those in
+   trust_value_to_string(), but are a fixed length.  This is needed to
+   make attractive information listings where columns line up
+   properly.  The value "10" should be the length of the strings you
+   choose to translate to.  This is the length in printable columns.
+   It gets passed to atoi() so everything after the number is
+   essentially a comment and need not be translated.  Either key and
+   uid are both NULL, or neither are NULL. */
+const char *
+uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
+{
+  if(!key && !uid)
+    return _("10 translator see trustdb.c:uid_trust_string_fixed");
+  else if(uid->is_revoked || (key && key->is_revoked))
+    return                         _("[ revoked]");
+  else if(uid->is_expired)
+    return                         _("[ expired]");
+  else if(key)
+    switch(get_validity(key,uid)&TRUST_MASK)
+      {
+      case TRUST_UNKNOWN:   return _("[ unknown]");
+      case TRUST_EXPIRED:   return _("[ expired]");
+      case TRUST_UNDEFINED: return _("[  undef ]");
+      case TRUST_MARGINAL:  return _("[marginal]");
+      case TRUST_FULLY:     return _("[  full  ]");
+      case TRUST_ULTIMATE:  return _("[ultimate]");
+      }
+
+  return "err";
+}
+
+/* The strings here are similar to those in
+   pkclist.c:do_edit_ownertrust() */
+const char *
+trust_value_to_string (unsigned int value)
+{
+  switch( (value & TRUST_MASK) ) 
+    {
+    case TRUST_UNKNOWN:   return _("unknown");
+    case TRUST_EXPIRED:   return _("expired");
+    case TRUST_UNDEFINED: return _("undefined");
+    case TRUST_NEVER:     return _("never");
+    case TRUST_MARGINAL:  return _("marginal");
+    case TRUST_FULLY:     return _("full");
+    case TRUST_ULTIMATE:  return _("ultimate");
+    default:              return "err";
+    }
+}
+
+int
+string_to_trust_value (const char *str)
+{
+  if(ascii_strcasecmp(str,"undefined")==0)
+    return TRUST_UNDEFINED;
+  else if(ascii_strcasecmp(str,"never")==0)
+    return TRUST_NEVER;
+  else if(ascii_strcasecmp(str,"marginal")==0)
+    return TRUST_MARGINAL;
+  else if(ascii_strcasecmp(str,"full")==0)
+    return TRUST_FULLY;
+  else if(ascii_strcasecmp(str,"ultimate")==0)
+    return TRUST_ULTIMATE;
+  else
+    return -1;
+}
 
 /****************
  * Recreate the WoT but do not ask for new ownertrusts.  Special
@@ -450,26 +564,32 @@ void
 check_trustdb ()
 {
   init_trustdb();
-  if (opt.batch && !opt.answer_yes)
+  if(opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC)
     {
-      ulong scheduled;
+      if (opt.batch && !opt.answer_yes)
+	{
+	  ulong scheduled;
 
-      scheduled = tdbio_read_nextcheck ();
-      if (!scheduled)
-        {
-          log_info (_("no need for a trustdb check\n"));
-          return;
-        }
+	  scheduled = tdbio_read_nextcheck ();
+	  if (!scheduled)
+	    {
+	      log_info (_("no need for a trustdb check\n"));
+	      return;
+	    }
 
-      if (scheduled > make_timestamp ())
-        {
-          log_info (_("next trustdb check due at %s\n"),
-                    strtimestamp (scheduled));
-          return;
-        }
+	  if (scheduled > make_timestamp ())
+	    {
+	      log_info (_("next trustdb check due at %s\n"),
+			strtimestamp (scheduled));
+	      return;
+	    }
+	}
+
+      validate_keys (0);
     }
-
-  validate_keys (0);
+  else
+    log_info (_("no need for a trustdb check with `%s' trust model\n"),
+	      trust_model_string());
 }
 
 
@@ -480,7 +600,11 @@ void
 update_trustdb()
 {
   init_trustdb();
-  validate_keys (1);
+  if(opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC)
+    validate_keys (1);
+  else
+    log_info (_("no need for a trustdb update with `%s' trust model\n"),
+	      trust_model_string());
 }
 
 void
@@ -494,7 +618,50 @@ revalidation_mark (void)
   pending_check_trustdb = 1;
 }
 
-
+int
+trustdb_pending_check(void)
+{
+  return pending_check_trustdb;
+}
+
+/* If the trustdb is dirty, and we're interactive, update it.
+   Otherwise, check it unless no-auto-check-trustdb is set. */
+void
+trustdb_check_or_update(void)
+{
+  if(trustdb_pending_check())
+    {
+      if(opt.interactive)
+	update_trustdb();
+      else if(!opt.no_auto_check_trustdb)
+	check_trustdb();
+    }
+}
+
+void
+read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck,
+		   byte *marginals,byte *completes,byte *cert_depth)
+{
+  TRUSTREC opts;
+
+  init_trustdb();
+
+  read_record(0,&opts,RECTYPE_VER);
+
+  if(trust_model)
+    *trust_model=opts.r.ver.trust_model;
+  if(created)
+    *created=opts.r.ver.created;
+  if(nextcheck)
+    *nextcheck=opts.r.ver.nextcheck;
+  if(marginals)
+    *marginals=opts.r.ver.marginals;
+  if(completes)
+    *completes=opts.r.ver.completes;
+  if(cert_depth)
+    *cert_depth=opts.r.ver.cert_depth;
+}
+
 /***********************************************
  ***********  Ownertrust et al. ****************
  ***********************************************/
@@ -525,7 +692,6 @@ read_trust_record (PKT_public_key *pk, TRUSTREC *rec)
   return 0;
 }
 
-
 /****************
  * Return the assigned ownertrust value for the given public key.
  * The key should be the primary key.
@@ -544,24 +710,71 @@ get_ownertrust ( PKT_public_key *pk)
       tdbio_invalid ();
       return rc; /* actually never reached */
     }
-  
+
   return rec.r.trust.ownertrust;
 }
 
+unsigned int 
+get_min_ownertrust (PKT_public_key *pk)
+{
+  TRUSTREC rec;
+  int rc;
+  
+  rc = read_trust_record (pk, &rec);
+  if (rc == -1)
+    return TRUST_UNKNOWN; /* no record yet */
+  if (rc) 
+    {
+      tdbio_invalid ();
+      return rc; /* actually never reached */
+    }
+
+  return rec.r.trust.min_ownertrust;
+}
+
 /*
- * Same as get_ownertrust but return a trust letter instead of an value.
+ * Same as get_ownertrust but this takes the minimum ownertrust value
+ * into into account, and will bump up the value as needed.
+ */
+static int
+get_ownertrust_with_min (PKT_public_key *pk)
+{
+  unsigned int otrust,otrust_min;
+
+  otrust = (get_ownertrust (pk) & TRUST_MASK);
+  otrust_min = get_min_ownertrust (pk);
+  if(otrust<otrust_min)
+    {
+      /* If the trust that the user has set is less than the trust
+	 that was calculated from a trust signature chain, use the
+	 higher of the two.  We do this here and not in
+	 get_ownertrust since the underlying ownertrust should not
+	 really be set - just the appearance of the ownertrust. */
+
+      otrust=otrust_min;
+    }
+
+  return otrust;
+}
+
+/*
+ * Same as get_ownertrust but return a trust letter instead of an
+ * value.  This takes the minimum ownertrust value into account.
  */
 int
 get_ownertrust_info (PKT_public_key *pk)
 {
-    unsigned int otrust;
-    int c;
+  return trust_letter(get_ownertrust_with_min(pk));
+}
 
-    otrust = get_ownertrust (pk);
-    c = trust_letter( otrust );
-    if( !c )
-	c = '?';
-    return c;
+/*
+ * Same as get_ownertrust but return a trust string instead of an
+ * value.  This takes the minimum ownertrust value into account.
+ */
+const char *
+get_ownertrust_string (PKT_public_key *pk)
+{
+  return trust_value_to_string(get_ownertrust_with_min(pk));
 }
 
 /*
@@ -611,9 +824,64 @@ update_ownertrust (PKT_public_key *pk, unsigned int new_trust )
     }
 }
 
-/* Clear the ownertrust value.  Return true if a changed actually happend. */
+static void
+update_min_ownertrust (u32 *kid, unsigned int new_trust )
+{
+  PKT_public_key *pk;
+  TRUSTREC rec;
+  int rc;
+
+  pk = m_alloc_clear (sizeof *pk);
+  rc = get_pubkey (pk, kid);
+  if (rc)
+    {
+      log_error(_("public key %s not found: %s\n"),keystr(kid),g10_errstr(rc));
+      return;
+    }
+
+  rc = read_trust_record (pk, &rec);
+  if (!rc)
+    {
+      if (DBG_TRUST)
+        log_debug ("key %08lX%08lX: update min_ownertrust from %u to %u\n",
+                   (ulong)kid[0],(ulong)kid[1],
+		   (unsigned int)rec.r.trust.min_ownertrust,
+		   new_trust );
+      if (rec.r.trust.min_ownertrust != new_trust)
+        {
+          rec.r.trust.min_ownertrust = new_trust;
+          write_record( &rec );
+          revalidation_mark ();
+          do_sync ();
+        }
+    }
+  else if (rc == -1)
+    { /* no record yet - create a new one */
+      size_t dummy;
+
+      if (DBG_TRUST)
+        log_debug ("insert min_ownertrust %u\n", new_trust );
+
+      memset (&rec, 0, sizeof rec);
+      rec.recnum = tdbio_new_recnum ();
+      rec.rectype = RECTYPE_TRUST;
+      fingerprint_from_pk (pk, rec.r.trust.fingerprint, &dummy);
+      rec.r.trust.min_ownertrust = new_trust;
+      write_record (&rec);
+      revalidation_mark ();
+      do_sync ();
+      rc = 0;
+    }
+  else 
+    {
+      tdbio_invalid ();
+    }
+}
+
+/* Clear the ownertrust and min_ownertrust values.  Return true if a
+   change actually happened. */
 int
-clear_ownertrust (PKT_public_key *pk)
+clear_ownertrusts (PKT_public_key *pk)
 {
   TRUSTREC rec;
   int rc;
@@ -622,11 +890,16 @@ clear_ownertrust (PKT_public_key *pk)
   if (!rc)
     {
       if (DBG_TRUST)
-        log_debug ("clearing ownertrust (old value %u)\n",
-                   (unsigned int)rec.r.trust.ownertrust);
-      if (rec.r.trust.ownertrust)
+	{
+	  log_debug ("clearing ownertrust (old value %u)\n",
+		     (unsigned int)rec.r.trust.ownertrust);
+	  log_debug ("clearing min_ownertrust (old value %u)\n",
+		     (unsigned int)rec.r.trust.min_ownertrust);
+	}
+      if (rec.r.trust.ownertrust || rec.r.trust.min_ownertrust)
         {
           rec.r.trust.ownertrust = 0;
+          rec.r.trust.min_ownertrust = 0;
           write_record( &rec );
           revalidation_mark ();
           do_sync ();
@@ -650,12 +923,8 @@ update_validity (PKT_public_key *pk, PKT_user_id *uid,
   TRUSTREC trec, vrec;
   int rc;
   ulong recno;
-  byte namehash[20];
 
-  if(uid->attrib_data)
-    rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
-  else
-    rmd160_hash_buffer (namehash, uid->name, uid->len );
+  namehash_from_uid(uid);
 
   rc = read_trust_record (pk, &trec);
   if (rc && rc != -1)
@@ -680,7 +949,7 @@ update_validity (PKT_public_key *pk, PKT_user_id *uid,
   while (recno)
     {
       read_record (recno, &vrec, RECTYPE_VALID);
-      if ( !memcmp (vrec.r.valid.namehash, namehash, 20) )
+      if ( !memcmp (vrec.r.valid.namehash, uid->namehash, 20) )
         break;
       recno = vrec.r.valid.next;
     }
@@ -690,7 +959,7 @@ update_validity (PKT_public_key *pk, PKT_user_id *uid,
       memset (&vrec, 0, sizeof vrec);
       vrec.recnum = tdbio_new_recnum ();
       vrec.rectype = RECTYPE_VALID;
-      memcpy (vrec.r.valid.namehash, namehash, 20);
+      memcpy (vrec.r.valid.namehash, uid->namehash, 20);
       vrec.r.valid.next = trec.r.trust.validlist;
       trec.r.trust.validlist = vrec.recnum;
     }
@@ -703,65 +972,23 @@ update_validity (PKT_public_key *pk, PKT_user_id *uid,
 }
 
 
-/* reset validity for all user IDs.  Caller must sync. */
-static int
-clear_validity (PKT_public_key *pk)
-{
-  TRUSTREC trec, vrec;
-  int rc;
-  ulong recno;
-  int any = 0;
-  
-  rc = read_trust_record (pk, &trec);
-  if (rc && rc != -1)
-    {
-      tdbio_invalid ();
-      return 0;
-    }
-  if (rc == -1) /* no record yet - no need to clear it then ;-) */
-    return 0;
-
-  recno = trec.r.trust.validlist;
-  while (recno)
-    {
-      read_record (recno, &vrec, RECTYPE_VALID);
-      if ((vrec.r.valid.validity & TRUST_MASK)
-	  || vrec.r.valid.marginal_count || vrec.r.valid.full_count)
-        {
-          vrec.r.valid.validity &= ~TRUST_MASK;
-	  vrec.r.valid.marginal_count = vrec.r.valid.full_count = 0;
-          write_record (&vrec);
-          any = 1;
-        }
-      recno = vrec.r.valid.next;
-    }
-
-  return any;
-}
-
 /***********************************************
  *********  Query trustdb values  **************
  ***********************************************/
 
 /* Return true if key is disabled */
 int
-is_disabled(void *dummy,u32 *keyid)
+cache_disabled_value(PKT_public_key *pk)
 {
   int rc;
   TRUSTREC trec;
-  int disabled=0; /* default to not disabled */
-  PKT_public_key *pk=m_alloc_clear(sizeof(PKT_public_key));
+  int disabled=0;
 
-  init_trustdb ();
+  if(pk->is_disabled)
+    return (pk->is_disabled==2);
+
+  init_trustdb();
 
-  rc = get_pubkey(pk, keyid);
-  if(rc)
-    {
-      log_error("error checking disabled status of %08lX: %s\n",
- 		(ulong)keyid[1],g10_errstr(rc));
-      goto leave;
-    }
- 
   rc = read_trust_record (pk, &trec);
   if (rc && rc != -1)
     {
@@ -774,38 +1001,25 @@ is_disabled(void *dummy,u32 *keyid)
   if(trec.r.trust.ownertrust & TRUST_FLAG_DISABLED)
     disabled=1;
  
+  /* Cache it for later so we don't need to look at the trustdb every
+     time */
+  if(disabled)
+    pk->is_disabled=2;
+  else
+    pk->is_disabled=1;
+
  leave:
-  free_public_key(pk);
-  return disabled;
+   return disabled;
 }
 
-/*
- * Return the validity information for PK.  If the namehash is not
- * NULL, the validity of the corresponsing user ID is returned,
- * otherwise, a reasonable value for the entire key is returned. 
- */
-unsigned int
-get_validity (PKT_public_key *pk, PKT_user_id *uid)
+void
+check_trustdb_stale(void)
 {
-  static int did_nextcheck;
-  TRUSTREC trec, vrec;
-  int rc;
-  ulong recno;
-  unsigned int validity;
-  u32 kid[2];
-  PKT_public_key *main_pk;
-  byte namehash[20];
-
-  if(uid)
-    {
-      if(uid->attrib_data)
-	rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
-      else
-	rmd160_hash_buffer (namehash, uid->name, uid->len );
-    }
+  static int did_nextcheck=0;
 
   init_trustdb ();
-  if (!did_nextcheck)
+  if (!did_nextcheck
+      && (opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC))
     {
       ulong scheduled;
 
@@ -825,6 +1039,28 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
             }
         }
     }
+}
+
+/*
+ * Return the validity information for PK.  If the namehash is not
+ * NULL, the validity of the corresponsing user ID is returned,
+ * otherwise, a reasonable value for the entire key is returned. 
+ */
+unsigned int
+get_validity (PKT_public_key *pk, PKT_user_id *uid)
+{
+  TRUSTREC trec, vrec;
+  int rc;
+  ulong recno;
+  unsigned int validity;
+  u32 kid[2];
+  PKT_public_key *main_pk;
+
+  if(uid)
+    namehash_from_uid(uid);
+
+  init_trustdb ();
+  check_trustdb_stale();
 
   keyid_from_pk (pk, kid);
   if (pk->main_keyid[0] != kid[0] || pk->main_keyid[1] != kid[1])
@@ -833,8 +1069,10 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
       rc = get_pubkey (main_pk, pk->main_keyid);
       if (rc)
         {
-          log_error ("error getting main key %08lX of subkey %08lX: %s\n",
-                     (ulong)pk->main_keyid[1], (ulong)kid[1], g10_errstr(rc));
+	  char *tempkeystr=m_strdup(keystr(pk->main_keyid));
+          log_error ("error getting main key %s of subkey %s: %s\n",
+                     tempkeystr, keystr(kid), g10_errstr(rc));
+	  m_free(tempkeystr);
           validity = TRUST_UNKNOWN; 
           goto leave;
 	}
@@ -842,6 +1080,14 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
   else
     main_pk = pk;
 
+  if(opt.trust_model==TM_DIRECT)
+    {
+      /* Note that this happens BEFORE any user ID stuff is checked.
+	 The direct trust model applies to keys as a whole. */
+      validity=get_ownertrust(main_pk);
+      goto leave;
+    }
+
   rc = read_trust_record (main_pk, &trec);
   if (rc && rc != -1)
     {
@@ -867,7 +1113,7 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
 	     user ID ONLY.  If the namehash is not found, then there
 	     is no validity at all (i.e. the user ID wasn't
 	     signed). */
-	  if(memcmp(vrec.r.valid.namehash,namehash,20)==0)
+	  if(memcmp(vrec.r.valid.namehash,uid->namehash,20)==0)
 	    {
 	      validity=(vrec.r.valid.validity & TRUST_MASK);
 	      break;
@@ -885,7 +1131,12 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
     }
   
   if ( (trec.r.trust.ownertrust & TRUST_FLAG_DISABLED) )
-    validity |= TRUST_FLAG_DISABLED;
+    {
+      validity |= TRUST_FLAG_DISABLED;
+      pk->is_disabled=2;
+    }
+  else
+    pk->is_disabled=1;
 
  leave:
   /* set some flags direct from the key */
@@ -910,17 +1161,22 @@ int
 get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
     int trustlevel;
-    int c;
 
     trustlevel = get_validity (pk, uid);
-    if( trustlevel & TRUST_FLAG_DISABLED )
-	return 'd';
     if( trustlevel & TRUST_FLAG_REVOKED )
 	return 'r';
-    c = trust_letter ( trustlevel );
-    if( !c )
-	c = '?';
-    return c;
+    return trust_letter ( trustlevel );
+}
+
+const char *
+get_validity_string (PKT_public_key *pk, PKT_user_id *uid)
+{
+  int trustlevel;
+
+  trustlevel = get_validity (pk, uid);
+  if( trustlevel & TRUST_FLAG_REVOKED )
+    return _("revoked");
+  return trust_value_to_string(trustlevel);
 }
 
 static void
@@ -928,15 +1184,11 @@ get_validity_counts (PKT_public_key *pk, PKT_user_id *uid)
 {
   TRUSTREC trec, vrec;
   ulong recno;
-  byte namehash[20];
 
   if(pk==NULL || uid==NULL)
     BUG();
 
-  if(uid->attrib_data)
-    rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
-  else
-    rmd160_hash_buffer (namehash, uid->name, uid->len );
+  namehash_from_uid(uid);
 
   uid->help_marginal_count=uid->help_full_count=0;
 
@@ -951,7 +1203,7 @@ get_validity_counts (PKT_public_key *pk, PKT_user_id *uid)
     {
       read_record (recno, &vrec, RECTYPE_VALID);
 
-      if(memcmp(vrec.r.valid.namehash,namehash,20)==0)
+      if(memcmp(vrec.r.valid.namehash,uid->namehash,20)==0)
 	{
 	  uid->help_marginal_count=vrec.r.valid.marginal_count;
 	  uid->help_full_count=vrec.r.valid.full_count;
@@ -1011,7 +1263,7 @@ enum_cert_paths_print( void **context, FILE *fp,
  ****************************************/
 
 static int
-ask_ownertrust (u32 *kid)
+ask_ownertrust (u32 *kid,int minimum)
 {
   PKT_public_key *pk;
   int rc;
@@ -1021,19 +1273,31 @@ ask_ownertrust (u32 *kid)
   rc = get_pubkey (pk, kid);
   if (rc)
     {
-      log_error (_("public key %08lX not found: %s\n"),
-                 (ulong)kid[1], g10_errstr(rc) );
+      log_error (_("public key %s not found: %s\n"),
+                 keystr(kid), g10_errstr(rc) );
       return TRUST_UNKNOWN;
     }
  
-  ot=edit_ownertrust(pk,0);
-  if(ot>0)
-    ot = get_ownertrust (pk);
-  else if(ot==0)
-    ot = TRUST_UNDEFINED;
+  if(opt.force_ownertrust)
+    {
+      log_info("force trust for key %s to %s\n",
+	       keystr(kid),trust_value_to_string(opt.force_ownertrust));
+      update_ownertrust(pk,opt.force_ownertrust);
+      ot=opt.force_ownertrust;
+    }
   else
-    ot = -1; /* quit */
+    {
+      ot=edit_ownertrust(pk,0);
+      if(ot>0)
+	ot = get_ownertrust (pk);
+      else if(ot==0)
+	ot = minimum?minimum:TRUST_UNDEFINED;
+      else
+	ot = -1; /* quit */
+    }
+
   free_public_key( pk );
+
   return ot;
 }
 
@@ -1171,6 +1435,9 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
         continue; /* ignore self-signatures */
       if (!IS_UID_SIG(sig) && !IS_UID_REV(sig))
         continue; /* we only look at these signature classes */
+      if(sig->sig_class>=0x11 && sig->sig_class<=0x13 &&
+	 sig->sig_class-0x10<opt.min_cert_level)
+	continue;
       if (!is_in_klist (klist, sig))
         continue;  /* no need to check it then */
       if (check_key_signature (keyblock, node, NULL))
@@ -1290,6 +1557,35 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
     }
 }
 
+/* Used by validate_one_keyblock to confirm a regexp within a trust
+   signature.  Returns 1 for match, and 0 for no match or regex
+   error. */
+static int
+check_regexp(const char *expr,const char *string)
+{
+#ifdef DISABLE_REGEX
+  /* When DISABLE_REGEX is defined, assume all regexps do not
+     match. */
+  return 0;
+#elif defined(__riscos__)
+  return riscos_check_regexp(expr, string, DBG_TRUST);
+#else
+  int ret;
+  regex_t pat;
+
+  if(regcomp(&pat,expr,REG_ICASE|REG_NOSUB|REG_EXTENDED)!=0)
+    return 0;
+
+  ret=regexec(&pat,string,0,NULL,0);
+
+  regfree(&pat);
+
+  if(DBG_TRUST)
+    log_debug("regexp `%s' on `%s': %s\n",expr,string,ret==0?"YES":"NO");
+
+  return (ret==0);
+#endif
+}
 
 /*
  * Return true if the key is signed by one of the keys in the given
@@ -1324,10 +1620,14 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 	 signed (but not self-signed) uid does carry trust, of a sort,
 	 even if it is a statement being made by people other than the
 	 key owner "through" the uids on the key owner's key.  I'm
-	 going with the latter. -dshaw */
+	 going with the latter.  However, if the user ID was
+	 explicitly revoked, or passively allowed to expire, that
+	 should stop validity through the user ID until it is
+	 resigned.  -dshaw */
 
-      /* && node->pkt->pkt.user_id->created) */
-      if (node->pkt->pkttype == PKT_USER_ID)
+      if (node->pkt->pkttype == PKT_USER_ID
+	  && !node->pkt->pkt.user_id->is_revoked
+	  && !node->pkt->pkt.user_id->is_expired)
         {
           if (uidnode && issigned)
             {
@@ -1341,12 +1641,11 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
             }
           uidnode = node;
 	  uid=uidnode->pkt->pkt.user_id;
-#if 0
-	  /* If the selfsig is going to expire...  This is disabled as
-	     we do count un-self-signed uids in the web of trust. */
+
+	  /* If the selfsig is going to expire... */
 	  if(uid->expiredate && uid->expiredate<*next_expire)
 	    *next_expire = uid->expiredate;
-#endif
+
           issigned = 0;
 	  get_validity_counts(pk,uid);
           mark_usable_uid_certs (kb, uidnode, main_kid, klist, 
@@ -1355,11 +1654,64 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
       else if (node->pkt->pkttype == PKT_SIGNATURE
 	       && (node->flag & (1<<8)) && uid)
         {
+	  /* Note that we are only seeing unrevoked sigs here */
           PKT_signature *sig = node->pkt->pkt.signature;
           
           kr = is_in_klist (klist, sig);
-          if (kr)
+	  /* If the trust_regexp does not match, it's as if the sig
+             did not exist.  This is safe for non-trust sigs as well
+             since we don't accept a regexp on the sig unless it's a
+             trust sig. */
+          if (kr && (kr->trust_regexp==NULL || opt.trust_model!=TM_PGP ||
+		     (uidnode && check_regexp(kr->trust_regexp,
+					    uidnode->pkt->pkt.user_id->name))))
             {
+	      if(DBG_TRUST && opt.trust_model==TM_PGP && sig->trust_depth)
+		log_debug("trust sig on %s, sig depth is %d, kr depth is %d\n",
+			  uidnode->pkt->pkt.user_id->name,sig->trust_depth,
+			  kr->trust_depth);
+
+	      /* Are we part of a trust sig chain?  We always favor
+                 the latest trust sig, rather than the greater or
+                 lesser trust sig or value.  I could make a decent
+                 argument for any of these cases, but this seems to be
+                 what PGP does, and I'd like to be compatible. -dms */
+	      if(opt.trust_model==TM_PGP && sig->trust_depth
+		 && pk->trust_timestamp<=sig->timestamp
+		 && (sig->trust_depth<=kr->trust_depth
+		     || kr->ownertrust==TRUST_ULTIMATE))
+		{
+		  /* If we got here, we know that:
+
+		     this is a trust sig.
+
+		     it's a newer trust sig than any previous trust
+		     sig on this key (not uid).
+
+		     it is legal in that it was either generated by an
+		     ultimate key, or a key that was part of a trust
+		     chain, and the depth does not violate the
+		     original trust sig.
+
+		     if there is a regexp attached, it matched
+		     successfully.
+		  */
+
+		  if(DBG_TRUST)
+		    log_debug("replacing trust value %d with %d and "
+			      "depth %d with %d\n",
+			      pk->trust_value,sig->trust_value,
+			      pk->trust_depth,sig->trust_depth);
+
+		  pk->trust_value=sig->trust_value;
+		  pk->trust_depth=sig->trust_depth-1;
+
+		  /* If the trust sig contains a regexp, record it
+		     on the pk for the next round. */
+		  if(sig->trust_regexp)
+		    pk->trust_regexp=sig->trust_regexp;
+		}
+
               if (kr->ownertrust == TRUST_ULTIMATE)
                 uid->help_full_count = opt.completes_needed;
               else if (kr->ownertrust == TRUST_FULLY)
@@ -1367,7 +1719,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
               else if (kr->ownertrust == TRUST_MARGINAL)
                 uid->help_marginal_count++;
               issigned = 1;
-            }
+	    }
         }
     }
 
@@ -1387,7 +1739,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 
 
 static int
-search_skipfnc (void *opaque, u32 *kid)
+search_skipfnc (void *opaque, u32 *kid, PKT_user_id *dummy)
 {
   return test_key_hash_table ((KeyHashTable)opaque, kid);
 }
@@ -1514,56 +1866,40 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
 
 /* Caller must sync */
 static void
-reset_trust_records (KEYDB_HANDLE hd, KeyHashTable exclude)
+reset_trust_records(void)
 {
-  int rc;
-  KBNODE keyblock = NULL;
-  KEYDB_SEARCH_DESC desc;
+  TRUSTREC rec;
+  ulong recnum;
   int count = 0, nreset = 0;
-  
-  rc = keydb_search_reset (hd);
-  if (rc)
+
+  for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ ) 
     {
-      log_error ("keydb_search_reset failed: %s\n", g10_errstr(rc));
-      return;
+      if(rec.rectype==RECTYPE_TRUST)
+	{
+	  count++;
+	  if(rec.r.trust.min_ownertrust)
+	    {
+	      rec.r.trust.min_ownertrust=0;
+	      write_record(&rec);
+	    }
+
+	}
+      else if(rec.rectype==RECTYPE_VALID
+	      && ((rec.r.valid.validity&TRUST_MASK)
+		  || rec.r.valid.marginal_count
+		  || rec.r.valid.full_count))
+	{
+	  rec.r.valid.validity &= ~TRUST_MASK;
+	  rec.r.valid.marginal_count=rec.r.valid.full_count=0;
+	  nreset++;
+	  write_record(&rec);
+	}
+
     }
 
-  memset (&desc, 0, sizeof desc);
-  desc.mode = KEYDB_SEARCH_MODE_FIRST;
-  if(exclude)
-    {
-      desc.skipfnc = search_skipfnc;
-      desc.skipfncvalue = exclude;
-    }
-  rc = keydb_search (hd, &desc, 1);
-  if (rc && rc != -1 )
-    log_error ("keydb_search_first failed: %s\n", g10_errstr(rc));
-  else if (!rc)
-    {
-      desc.mode = KEYDB_SEARCH_MODE_NEXT; /* change mode */
-      do
-        {
-          rc = keydb_get_keyblock (hd, &keyblock);
-          if (rc) 
-            {
-              log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
-              break;
-            }
-          count++;
-
-          if (keyblock->pkt->pkttype == PKT_PUBLIC_KEY) /* paranoid assertion*/
-            {
-              nreset += clear_validity (keyblock->pkt->pkt.public_key);
-              release_kbnode (keyblock);
-            } 
-        }
-      while ( !(rc = keydb_search (hd, &desc, 1)) );
-      if (rc && rc != -1) 
-        log_error ("keydb_search_next failed: %s\n", g10_errstr(rc));
-    }
   if (opt.verbose)
     log_info (_("%d keys processed (%d validity counts cleared)\n"),
-              count, nreset);
+	      count, nreset);
 }
 
 /*
@@ -1602,16 +1938,26 @@ validate_keys (int interactive)
   KEYDB_HANDLE kdb = NULL;
   KBNODE node;
   int depth;
-  int key_count;
   int ot_unknown, ot_undefined, ot_never, ot_marginal, ot_full, ot_ultimate;
   KeyHashTable stored,used,full_trust;
   u32 start_time, next_expire;
 
+  /* Make sure we have all sigs cached.  TODO: This is going to
+     require some architectual re-thinking, as it is agonizingly slow.
+     Perhaps combine this with reset_trust_records(), or only check
+     the caches on keys that are actually involved in the web of
+     trust. */
+  keydb_rebuild_caches(0);
+
   start_time = make_timestamp ();
   next_expire = 0xffffffff; /* set next expire to the year 2106 */
   stored = new_key_hash_table ();
   used = new_key_hash_table ();
   full_trust = new_key_hash_table ();
+
+  kdb = keydb_new (0);
+  reset_trust_records();
+
   /* Fixme: Instead of always building a UTK list, we could just build it
    * here when needed */
   if (!utk_list)
@@ -1620,10 +1966,6 @@ validate_keys (int interactive)
       goto leave;
     }
 
-  kdb = keydb_new (0);
-
-  reset_trust_records (kdb,NULL);
-
   /* mark all UTKs as used and fully_trusted and set validity to
      ultimate */
   for (k=utk_list; k; k = k->next)
@@ -1635,7 +1977,7 @@ validate_keys (int interactive)
       if (!keyblock)
         {
           log_error (_("public key of ultimately"
-                       " trusted key %08lX not found\n"), (ulong)k->kid[1]);
+                       " trusted key %s not found\n"), keystr(k->kid));
           continue;
         }
       mark_keyblock_seen (used, keyblock);
@@ -1657,22 +1999,57 @@ validate_keys (int interactive)
 
   klist = utk_list;
 
+  log_info(_("%d marginal(s) needed, %d complete(s) needed, %s trust model\n"),
+	   opt.marginals_needed,opt.completes_needed,trust_model_string());
+
   for (depth=0; depth < opt.max_cert_depth; depth++)
     {
+      int valids=0,key_count;
       /* See whether we should assign ownertrust values to the keys in
-         utk_list.  */
+         klist.  */
       ot_unknown = ot_undefined = ot_never = 0;
       ot_marginal = ot_full = ot_ultimate = 0;
       for (k=klist; k; k = k->next)
         {
+	  int min=0;
+
+	  /* 120 and 60 are as per RFC2440 */
+	  if(k->trust_value>=120)
+	    min=TRUST_FULLY;
+	  else if(k->trust_value>=60)
+	    min=TRUST_MARGINAL;
+
+	  if(min!=k->min_ownertrust)
+	    update_min_ownertrust(k->kid,min);
+
           if (interactive && k->ownertrust == TRUST_UNKNOWN)
-              k->ownertrust = ask_ownertrust (k->kid);
-	  if (k->ownertrust == -1)
 	    {
-	      quit=1;
-	      goto leave;
+	      k->ownertrust = ask_ownertrust (k->kid,min);
+
+	      if (k->ownertrust == -1)
+		{
+		  quit=1;
+		  goto leave;
+		}
 	    }
-	  else if (k->ownertrust == TRUST_UNKNOWN)
+
+	  /* This can happen during transition from an old trustdb
+	     before trust sigs.  It can also happen if a user uses two
+	     different versions of GnuPG or changes the --trust-model
+	     setting. */
+	  if(k->ownertrust<min)
+	    {
+	      if(DBG_TRUST)
+		log_debug("key %08lX%08lX:"
+			  " overriding ownertrust `%s' with `%s'\n",
+			  (ulong)k->kid[0],(ulong)k->kid[1],
+			  trust_value_to_string(k->ownertrust),
+			  trust_value_to_string(min));
+
+	      k->ownertrust=min;
+	    }
+
+	  if (k->ownertrust == TRUST_UNKNOWN)
             ot_unknown++;
           else if (k->ownertrust == TRUST_UNDEFINED)
             ot_undefined++;
@@ -1684,6 +2061,8 @@ validate_keys (int interactive)
             ot_full++;
           else if (k->ownertrust == TRUST_ULTIMATE)
             ot_ultimate++;
+
+	  valids++;
         }
 
       /* Find all keys which are signed by a key in kdlist */
@@ -1706,12 +2085,9 @@ validate_keys (int interactive)
       for (kar=keys; kar->keyblock; kar++)
           store_validation_status (depth, kar->keyblock, stored);
 
-      /* This should be valid=%d now, but I'm not changing it so I
-	 don't break the translated strings in the stable branch.
-	 Change it in devel. -dms */
-      log_info (_("checking at depth %d signed=%d"
-                  " ot(-/q/n/m/f/u)=%d/%d/%d/%d/%d/%d\n"), 
-                depth, key_count, ot_unknown, ot_undefined,
+      log_info (_("depth: %d  valid: %3d  signed: %3d"
+                  "  trust: %d-, %dq, %dn, %dm, %df, %du\n"), 
+                depth, valids, key_count, ot_unknown, ot_undefined,
                 ot_never, ot_marginal, ot_full, ot_ultimate ); 
 
       /* Build a new kdlist from all fully valid keys in KEYS */
@@ -1738,9 +2114,19 @@ validate_keys (int interactive)
 		      k = new_key_item ();
 		      k->kid[0]=kid[0];
 		      k->kid[1]=kid[1];
-		      k->ownertrust = (get_ownertrust (kar->keyblock
-						       ->pkt->pkt.public_key)
-				       & TRUST_MASK);
+		      k->ownertrust =
+			(get_ownertrust (kar->keyblock->pkt->pkt.public_key)
+			 & TRUST_MASK);
+		      k->min_ownertrust =
+			get_min_ownertrust(kar->keyblock->pkt->pkt.public_key);
+		      k->trust_depth=
+			kar->keyblock->pkt->pkt.public_key->trust_depth;
+		      k->trust_value=
+			kar->keyblock->pkt->pkt.public_key->trust_value;
+		      if(kar->keyblock->pkt->pkt.public_key->trust_regexp)
+			k->trust_regexp=
+			  m_strdup(kar->keyblock->pkt->
+				   pkt.public_key->trust_regexp);
 		      k->next = klist;
 		      klist = k;
 		      break;
@@ -1771,6 +2157,14 @@ validate_keys (int interactive)
           log_info (_("next trustdb check due at %s\n"),
                     strtimestamp (next_expire));
         }
+
+      if(tdbio_update_version_record()!=0)
+	{
+	  log_error(_("unable to update trustdb version record: "
+		      "write failed: %s\n"), g10_errstr(rc));
+	  tdbio_invalid();
+	}
+
       do_sync ();
       pending_check_trustdb = 0;
     }
