@@ -74,10 +74,16 @@
 #define G10ERR_NOT_PROCESSED  52
 #define G10ERR_UNU_PUBKEY     53
 #define G10ERR_UNU_SECKEY     54
-
+#define G10ERR_KEYSERVER      55
+#define G10ERR_CANCELED       56
+#define G10ERR_NO_CARD        57
 
 #ifndef HAVE_STRERROR
-char *strerror( int n );
+char *strerror (int n);
+#endif
+
+#ifdef _WIN32
+const char * w32_strerror (int w32_errno);
 #endif
 
 #endif /*G10_ERRORS_H*/
