@@ -1,5 +1,5 @@
 /* http.h  -  HTTP protocol handler
- *	Copyright (C) 1999 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2000, 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -17,8 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifndef GPG_HTTP_H
-#define GPG_HTTP_H 1
+
+#ifndef G10_HTTP_H
+#define G10_HTTP_H 1
 
 #include "iobuf.h"
 
@@ -34,6 +35,7 @@ typedef struct uri_tuple *URI_TUPLE;
 struct parsed_uri {
     /* all these pointers point into buffer; most stuff is not escaped */
     char *scheme;	/* pointer to the scheme string (lowercase) */
+    char *auth;         /* username/password for basic auth */
     char *host; 	/* host (converted to lowercase) */
     ushort port;	/* port (always set if the host is set) */
     char *path; 	/* the path */
@@ -49,9 +51,12 @@ typedef enum {
     HTTP_REQ_POST = 3
 } HTTP_REQ_TYPE;
 
-enum {	/* put flag values into an enum, so that gdb can display them */
-    HTTP_FLAG_TRY_PROXY = 1
-};
+/* put flag values into an enum, so that gdb can display them */
+enum
+  { 
+    HTTP_FLAG_NO_SHUTDOWN = 1,
+    HTTP_FLAG_TRY_SRV = 2
+  };
 
 struct http_context {
     int initialized;
@@ -64,17 +69,17 @@ struct http_context {
     PARSED_URI uri;
     HTTP_REQ_TYPE req_type;
     byte *buffer;	   /* line buffer */
-    unsigned int buffer_size;
+    unsigned buffer_size;
     unsigned int flags;
 };
 typedef struct http_context *HTTP_HD;
 
 int http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
-						  unsigned int flags );
+	       unsigned int flags, const char *proxy );
 void http_start_data( HTTP_HD hd );
 int  http_wait_response( HTTP_HD hd, unsigned int *ret_status );
 void http_close( HTTP_HD hd );
+int http_open_document( HTTP_HD hd, const char *document,
+			unsigned int flags, const char *proxy );
 
-int http_open_document( HTTP_HD hd, const char *document, unsigned int flags );
-
-#endif /*GPG_HTTP_H*/
+#endif /*G10_HTTP_H*/
