@@ -1,5 +1,5 @@
 /* iobuf.h - I/O buffer
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -26,7 +26,6 @@
 
 #define DBG_IOBUF   iobuf_debug_mode
 
-
 #define IOBUFCTRL_INIT	    1
 #define IOBUFCTRL_FREE	    2
 #define IOBUFCTRL_UNDERFLOW 3
@@ -72,15 +71,16 @@ struct iobuf_struct {
 };
 
 #ifndef EXTERN_UNLESS_MAIN_MODULE
- #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
-  #define EXTERN_UNLESS_MAIN_MODULE extern
- #else
-  #define EXTERN_UNLESS_MAIN_MODULE 
- #endif
+#if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
+#define EXTERN_UNLESS_MAIN_MODULE extern
+#else
+#define EXTERN_UNLESS_MAIN_MODULE 
+#endif
 #endif
 EXTERN_UNLESS_MAIN_MODULE int iobuf_debug_mode;
 
 void  iobuf_enable_special_filenames ( int yes );
+int   iobuf_is_pipe_filename (const char *fname);
 IOBUF iobuf_alloc(int use, size_t bufsize);
 IOBUF iobuf_temp(void);
 IOBUF iobuf_temp_with_content( const char *buffer, size_t length );
@@ -124,18 +124,16 @@ int  iobuf_write_temp( IOBUF a, IOBUF temp );
 size_t iobuf_temp_to_buffer( IOBUF a, byte *buffer, size_t buflen );
 void iobuf_unget_and_close_temp( IOBUF a, IOBUF temp );
 
+int  iobuf_get_fd (IOBUF a);
 off_t iobuf_get_filelength( IOBUF a );
 #define IOBUF_FILELENGTH_LIMIT 0xffffffff
 const char *iobuf_get_real_fname( IOBUF a );
 const char *iobuf_get_fname( IOBUF a );
 
-void iobuf_set_block_mode( IOBUF a, size_t n );
 void iobuf_set_partial_block_mode( IOBUF a, size_t len );
-int  iobuf_in_block_mode( IOBUF a );
 
 int iobuf_translate_file_handle ( int fd, int for_write );
 
-
 /* get a byte form the iobuf; must check for eof prior to this function
  * this function returns values in the range 0 .. 255 or -1 to indicate EOF
  * iobuf_get_noeof() does not return -1 to indicate EOF, but masks the
@@ -158,4 +156,6 @@ int iobuf_translate_file_handle ( int fd, int for_write );
 #define iobuf_get_temp_length(a) ( (a)->d.len )
 #define iobuf_is_temp(a)	 ( (a)->use == 3 )
 
+void iobuf_skip_rest (IOBUF a, unsigned long n, int partial);
+
 #endif /*G10_IOBUF_H*/
