@@ -20,15 +20,19 @@
 #ifndef G10_TTYIO_H
 #define G10_TTYIO_H
 
+const char *tty_get_ttyname (void);
 int tty_batchmode( int onoff );
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
  void tty_printf (const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
+ void tty_fprintf (FILE *fp, const char *fmt, ... )
+                                __attribute__ ((format (printf,2,3)));
 #else
  void tty_printf (const char *fmt, ... );
+ void tty_fprintf (FILE *fp, const char *fmt, ... );
 #endif
-void tty_print_string( byte *p, size_t n );
-void tty_print_utf8_string( byte *p, size_t n );
-void tty_print_utf8_string2( byte *p, size_t n, size_t max_n );
+void tty_print_string( const byte *p, size_t n );
+void tty_print_utf8_string( const byte *p, size_t n );
+void tty_print_utf8_string2( const byte *p, size_t n, size_t max_n );
 char *tty_get( const char *prompt );
 char *tty_get_hidden( const char *prompt );
 void tty_kill_prompt(void);
