@@ -1,5 +1,6 @@
 /* util.h
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -70,6 +71,7 @@ const char *log_get_name(void);
 void log_set_pid( int pid );
 int  log_get_errorcount( int clear );
 void log_inc_errorcount(void);
+int log_set_strict(int val);
 void g10_log_hexdump( const char *text, const char *buf, size_t len );
 
 #if defined (__riscos__) \
@@ -81,15 +83,8 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
 			    __attribute__ ((noreturn, format (printf,1,2)));
   void g10_log_error( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
   void g10_log_info( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
+  void g10_log_warning( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
   void g10_log_debug( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
-  void g10_log_fatal_f( const char *fname, const char *fmt, ... )
-			    __attribute__ ((noreturn, format (printf,2,3)));
-  void g10_log_error_f( const char *fname, const char *fmt, ... )
-			    __attribute__ ((format (printf,2,3)));
-  void g10_log_info_f( const char *fname, const char *fmt, ... )
-			    __attribute__ ((format (printf,2,3)));
-  void g10_log_debug_f( const char *fname,  const char *fmt, ... )
-			    __attribute__ ((format (printf,2,3)));
 #ifndef __riscos__
 #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
 #else
@@ -101,11 +96,8 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
   void g10_log_fatal( const char *fmt, ... );
   void g10_log_error( const char *fmt, ... );
   void g10_log_info( const char *fmt, ... );
+  void g10_log_warning( const char *fmt, ... );
   void g10_log_debug( const char *fmt, ... );
-  void g10_log_fatal_f( const char *fname, const char *fmt, ... );
-  void g10_log_error_f( const char *fname, const char *fmt, ... );
-  void g10_log_info_f( const char *fname, const char *fmt, ... );
-  void g10_log_debug_f( const char *fname, const char *fmt, ... );
 #define BUG() g10_log_bug0( __FILE__ , __LINE__ )
 #endif
 
@@ -115,11 +107,8 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
 #define log_fatal   g10_log_fatal
 #define log_error   g10_log_error
 #define log_info    g10_log_info
+#define log_warning g10_log_warning
 #define log_debug   g10_log_debug
-#define log_fatal_f g10_log_fatal_f
-#define log_error_f g10_log_error_f
-#define log_info_f  g10_log_info_f
-#define log_debug_f g10_log_debug_f
 
 
 /*-- errors.c --*/
@@ -143,19 +132,19 @@ typedef struct dotlock_handle *DOTLOCK;
 
 void disable_dotlock(void);
 DOTLOCK create_dotlock( const char *file_to_lock );
+void destroy_dotlock ( DOTLOCK h );
 int make_dotlock( DOTLOCK h, long timeout );
 int release_dotlock( DOTLOCK h );
 void remove_lockfiles (void);
 
 /*-- fileutil.c --*/
-char * make_basename(const char *filepath);
+char * make_basename(const char *filepath, const char *inputpath);
 char * make_dirname(const char *filepath);
 char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
 const char *print_fname_stdin( const char *s );
 const char *print_fname_stdout( const char *s );
-int  is_file_compressed(const char *s, int *r_status);
-
+int is_file_compressed(const char *s, int *r_status);
 
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
@@ -163,15 +152,19 @@ u32 scan_isodatestr( const char *string );
 u32 add_days_to_timestamp( u32 stamp, u16 days );
 const char *strtimevalue( u32 stamp );
 const char *strtimestamp( u32 stamp ); /* GMT */
+const char *isotimestamp( u32 stamp ); /* GMT with hh:mm:ss */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, const byte *p, size_t n, int delim );
+void print_string2( FILE *fp, const byte *p, size_t n, int delim, int delim2 );
 void  print_utf8_string( FILE *fp, const byte *p, size_t n );
 void  print_utf8_string2( FILE *fp, const byte *p, size_t n, int delim);
 char *make_printable_string( const byte *p, size_t n, int delim );
 int answer_is_yes_no_default( const char *s, int def_answer );
 int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
+int answer_is_okay_cancel (const char *s, int def_answer);
 int match_multistr(const char *multistr,const char *match);
+int hextobyte( const char *s );
 
 /*-- strgutil.c --*/
 void free_strlist( STRLIST sl );
@@ -238,7 +231,8 @@ int write_w32_registry_string(const char *root, const char *dir,
                               const char *name, const char *value);
 
 /*-- strgutil.c --*/
-int vasprintf ( char **result, const char *format, va_list args);
+int vasprintf (char **result, const char *format, va_list args);
+int asprintf (char **buf, const char *fmt, ...);
 #endif
 
 /**** other missing stuff ****/
@@ -250,6 +244,10 @@ int vasprintf ( char **result, const char *format, va_list args);
 #define raise(a) kill(getpid(), (a))
 #endif
 
+/*-- Replacement functions from funcname.c --*/
+
+
+
 /******** some macros ************/
 #ifndef STR
 #define STR(v) #v
@@ -267,33 +265,39 @@ int vasprintf ( char **result, const char *format, va_list args);
 #define hexdigitp(a) (digitp (a)                     \
                       || (*(a) >= 'A' && *(a) <= 'F')  \
                       || (*(a) >= 'a' && *(a) <= 'f'))
+/* the atoi macros assume that the buffer has only valid digits */
+#define atoi_1(p)   (*(p) - '0' )
+#define atoi_2(p)   ((atoi_1(p) * 10) + atoi_1((p)+1))
+#define atoi_4(p)   ((atoi_2(p) * 100) + atoi_2((p)+2))
+#define xtoi_1(p)   (*(p) <= '9'? (*(p)- '0'): \
+                     *(p) <= 'F'? (*(p)-'A'+10):(*(p)-'a'+10))
+#define xtoi_2(p)   ((xtoi_1(p) * 16) + xtoi_1((p)+1))
+
 /* Note this isn't identical to a C locale isspace() without \f and
    \v, but works for the purposes used here. */
 #define ascii_isspace(a) ((a)==' ' || (a)=='\n' || (a)=='\r' || (a)=='\t')
 
 /******* RISC OS stuff ***********/
 #ifdef __riscos__
-/* needed for strcasecmp() */
-#include <strings.h>
-/* needed for filename munging */
-#include <unixlib/local.h>
-/* needed for image file system feature */
-#include <unixlib/features.h>
-void riscos_global_defaults(void);
-#define RISCOS_GLOBAL_STATICS(a) const char *__dynamic_da_name = (a);
-void riscos_set_filetype(const char *filename, const char *mimetype);
+int riscos_load_module(const char *name, const char * const path[], int fatal);
+int riscos_get_filetype_from_string(const char *string, int len);
+int riscos_get_filetype(const char *filename);
+void riscos_set_filetype_by_number(const char *filename, int type);
+void riscos_set_filetype_by_mimetype(const char *filename, const char *mimetype);
 pid_t riscos_getpid(void);
 int riscos_kill(pid_t pid, int sig);
 int riscos_access(const char *path, int amode);
 int riscos_getchar(void);
-int fdopenfile(const char *filename, const int allow_write);
-void close_fds(void);
-int renamefile(const char *old, const char *new);
-char *gstrans(const char *old);
-void not_implemented(const char *feature);
+char *riscos_make_basename(const char *filepath, const char *inputpath);
+int riscos_check_regexp(const char *exp, const char *string, int debug);
+int riscos_fdopenfile(const char *filename, const int allow_write);
+void riscos_close_fds(void);
+int riscos_renamefile(const char *old, const char *new);
+char *riscos_gstrans(const char *old);
+void riscos_not_implemented(const char *feature);
 #ifdef DEBUG
-void dump_fdlist(void);
-void list_openfiles(void);
+void riscos_dump_fdlist(void);
+void riscos_list_openfiles(void);
 #endif
 #ifndef __RISCOS__C__
 #define getpid riscos_getpid
