@@ -1,5 +1,5 @@
 /* mpiutil.ac  -  Utility functions for MPI
- *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,10 +24,17 @@
 #include <string.h>
 #include <assert.h>
 
-#include "g10lib.h"
 #include "mpi.h"
 #include "mpi-internal.h"
 #include "memory.h"
+#include "util.h"
+
+
+#ifdef M_DEBUG
+#undef mpi_alloc
+#undef mpi_alloc_secure
+#undef mpi_free
+#endif
 
 /****************
  * Note:  It was a bad idea to use the number of limbs to allocate
@@ -37,59 +44,148 @@
  *	  But mpi_alloc is used in a lot of places :-)
  */
 MPI
+#ifdef M_DEBUG
+mpi_debug_alloc( unsigned nlimbs, const char *info )
+#else
 mpi_alloc( unsigned nlimbs )
+#endif
 {
     MPI a;
 
-    a = g10_xmalloc( sizeof *a );
+    if( DBG_MEMORY )
+	log_debug("mpi_alloc(%u)\n", nlimbs*BITS_PER_MPI_LIMB );
+#ifdef M_DEBUG
+    a = m_debug_alloc( sizeof *a, info );
+    a->d = nlimbs? mpi_debug_alloc_limb_space( nlimbs, 0, info ) : NULL;
+#else
+    a = m_alloc( sizeof *a );
     a->d = nlimbs? mpi_alloc_limb_space( nlimbs, 0 ) : NULL;
+#endif
     a->alloced = nlimbs;
     a->nlimbs = 0;
     a->sign = 0;
     a->flags = 0;
+    a->nbits = 0;
     return a;
 }
 
 void
 mpi_m_check( MPI a )
 {
-    g10_check_heap(a);
-    g10_check_heap(a->d);
+    m_check(a);
+    m_check(a->d);
 }
 
 MPI
+#ifdef M_DEBUG
+mpi_debug_alloc_secure( unsigned nlimbs, const char *info )
+#else
 mpi_alloc_secure( unsigned nlimbs )
+#endif
 {
     MPI a;
 
-    a = g10_xmalloc( sizeof *a );
+    if( DBG_MEMORY )
+	log_debug("mpi_alloc_secure(%u)\n", nlimbs*BITS_PER_MPI_LIMB );
+#ifdef M_DEBUG
+    a = m_debug_alloc( sizeof *a, info );
+    a->d = nlimbs? mpi_debug_alloc_limb_space( nlimbs, 1, info ) : NULL;
+#else
+    a = m_alloc( sizeof *a );
     a->d = nlimbs? mpi_alloc_limb_space( nlimbs, 1 ) : NULL;
+#endif
     a->alloced = nlimbs;
     a->flags = 1;
     a->nlimbs = 0;
     a->sign = 0;
+    a->nbits = 0;
     return a;
 }
 
 
+#if 0
+static void *unused_limbs_5;
+static void *unused_limbs_32;
+static void *unused_limbs_64;
+#endif
 
 mpi_ptr_t
+#ifdef M_DEBUG
+mpi_debug_alloc_limb_space( unsigned nlimbs, int secure, const char *info )
+#else
 mpi_alloc_limb_space( unsigned nlimbs, int secure )
+#endif
 {
     size_t len = nlimbs * sizeof(mpi_limb_t);
     mpi_ptr_t p;
 
-    p = secure? g10_xmalloc_secure( len ) : g10_xmalloc( len );
+    if( DBG_MEMORY )
+	log_debug("mpi_alloc_limb_space(%u)\n", (unsigned)len*8 );
+#if 0
+    if( !secure ) {
+	if( nlimbs == 5 && unused_limbs_5 ) {  /* DSA 160 bits */
+	    p = unused_limbs_5;
+	    unused_limbs_5 = *p;
+	    return p;
+	}
+	else if( nlimbs == 32 && unused_limbs_32 ) {  /* DSA 1024 bits */
+	    p = unused_limbs_32;
+	    unused_limbs_32 = *p;
+	    return p;
+	}
+	else if( nlimbs == 64 && unused_limbs_64 ) {  /* DSA 2*1024 bits */
+	    p = unused_limbs_64;
+	    unused_limbs_64 = *p;
+	    return p;
+	}
+    }
+#endif
+
+#ifdef M_DEBUG
+    p = secure? m_debug_alloc_secure(len, info):m_debug_alloc( len, info );
+#else
+    p = secure? m_alloc_secure( len ):m_alloc( len );
+#endif
 
     return p;
 }
 
 void
+#ifdef M_DEBUG
+mpi_debug_free_limb_space( mpi_ptr_t a, const char *info )
+#else
 mpi_free_limb_space( mpi_ptr_t a )
+#endif
 {
     if( !a )
 	return;
-    g10_free(a);
+    if( DBG_MEMORY )
+	log_debug("mpi_free_limb_space of size %lu\n", (ulong)m_size(a)*8 );
+
+#if 0
+    if( !m_is_secure(a) ) {
+	size_t nlimbs = m_size(a) / 4 ;
+	void *p = a;
+
+	if( nlimbs == 5 ) {  /* DSA 160 bits */
+	    *a = unused_limbs_5;
+	    unused_limbs_5 = a;
+	    return;
+	}
+	else if( nlimbs == 32 ) {  /* DSA 1024 bits */
+	    *a = unused_limbs_32;
+	    unused_limbs_32 = a;
+	    return;
+	}
+	else if( nlimbs == 64 ) {  /* DSA 2*1024 bits */
+	    *a = unused_limbs_64;
+	    unused_limbs_64 = a;
+	    return;
+	}
+    }
+#endif
+
+    m_free(a);
 }
 
 
@@ -105,22 +201,33 @@ mpi_assign_limb_space( MPI a, mpi_ptr_t ap, unsigned nlimbs )
 
 /****************
  * Resize the array of A to NLIMBS. the additional space is cleared
- * (set to 0) [done by g10_realloc()]
+ * (set to 0) [done by m_realloc()]
  */
 void
+#ifdef M_DEBUG
+mpi_debug_resize( MPI a, unsigned nlimbs, const char *info )
+#else
 mpi_resize( MPI a, unsigned nlimbs )
+#endif
 {
     if( nlimbs <= a->alloced )
 	return; /* no need to do it */
     /* Note: a->secure is not used - instead the realloc functions
      * take care of it. Maybe we should drop a->secure completely
      * and rely on a mpi_is_secure function, which would be
-     * a wrapper around g10_is_secure
+     * a wrapper around m_is_secure
      */
+#ifdef M_DEBUG
     if( a->d )
-	a->d = g10_xrealloc(a->d, nlimbs * sizeof(mpi_limb_t) );
-    else  /* FIXME: It may not be allocted in secure memory */
-	a->d = g10_xcalloc( nlimbs , sizeof(mpi_limb_t) );
+	a->d = m_debug_realloc(a->d, nlimbs * sizeof(mpi_limb_t), info );
+    else
+	a->d = m_debug_alloc_clear( nlimbs * sizeof(mpi_limb_t), info );
+#else
+    if( a->d )
+	a->d = m_realloc(a->d, nlimbs * sizeof(mpi_limb_t) );
+    else
+	a->d = m_alloc_clear( nlimbs * sizeof(mpi_limb_t) );
+#endif
     a->alloced = nlimbs;
 }
 
@@ -128,26 +235,38 @@ void
 mpi_clear( MPI a )
 {
     a->nlimbs = 0;
+    a->nbits = 0;
     a->flags = 0;
 }
 
 
 void
+#ifdef M_DEBUG
+mpi_debug_free( MPI a, const char *info )
+#else
 mpi_free( MPI a )
+#endif
 {
     if( !a )
 	return;
+    if( DBG_MEMORY )
+	log_debug("mpi_free\n" );
     if( a->flags & 4 )
-	g10_free( a->d );
+	m_free( a->d );
     else {
+#ifdef M_DEBUG
+	mpi_debug_free_limb_space(a->d, info);
+#else
 	mpi_free_limb_space(a->d);
+#endif
     }
     if( a->flags & ~7 )
 	log_bug("invalid flag value in mpi\n");
-    g10_free(a);
+    m_free(a);
 }
 
-static void
+
+void
 mpi_set_secure( MPI a )
 {
     mpi_ptr_t ap, bp;
@@ -160,42 +279,58 @@ mpi_set_secure( MPI a )
 	assert(!ap);
 	return;
     }
+#ifdef M_DEBUG
+    bp = mpi_debug_alloc_limb_space( a->nlimbs, 1, "set_secure" );
+#else
     bp = mpi_alloc_limb_space( a->nlimbs, 1 );
+#endif
     MPN_COPY( bp, ap, a->nlimbs );
     a->d = bp;
+#ifdef M_DEBUG
+    mpi_debug_free_limb_space(ap, "set_secure");
+#else
     mpi_free_limb_space(ap);
+#endif
 }
 
 
 MPI
-gcry_mpi_set_opaque( MPI a, void *p, unsigned int nbits )
+mpi_set_opaque( MPI a, void *p, unsigned int len )
 {
     if( !a ) {
+#ifdef M_DEBUG
+	a = mpi_debug_alloc(0,"alloc_opaque");
+#else
 	a = mpi_alloc(0);
+#endif
     }
 
     if( a->flags & 4 )
-	g10_free( a->d );
+	m_free( a->d );
     else {
+#ifdef M_DEBUG
+	mpi_debug_free_limb_space(a->d, "alloc_opaque");
+#else
 	mpi_free_limb_space(a->d);
+#endif
     }
 
     a->d = p;
     a->alloced = 0;
     a->nlimbs = 0;
-    a->sign  = nbits;
+    a->nbits = len;
     a->flags = 4;
     return a;
 }
 
 
 void *
-gcry_mpi_get_opaque( MPI a, unsigned int *nbits )
+mpi_get_opaque( MPI a, unsigned int *len )
 {
     if( !(a->flags & 4) )
 	log_bug("mpi_get_opaque on normal mpi\n");
-    if( nbits )
-	*nbits = a->sign;
+    if( len )
+	*len = a->nbits;
     return a->d;
 }
 
@@ -205,23 +340,33 @@ gcry_mpi_get_opaque( MPI a, unsigned int *nbits )
  *	 but copy it transparently.
  */
 MPI
+#ifdef M_DEBUG
+mpi_debug_copy( MPI a, const char *info )
+#else
 mpi_copy( MPI a )
+#endif
 {
     int i;
     MPI b;
 
     if( a && (a->flags & 4) ) {
-	void *p = g10_is_secure(a->d)? g10_xmalloc_secure( (a->sign+7)/8 )
-				     : g10_xmalloc( (a->sign+7)/8 );
-	memcpy( p, a->d, (a->sign+7)/8 );
-	b = gcry_mpi_set_opaque( NULL, p, a->sign );
+	void *p = m_is_secure(a->d)? m_alloc_secure( a->nbits )
+				   : m_alloc( a->nbits );
+	memcpy( p, a->d, a->nbits );
+	b = mpi_set_opaque( NULL, p, a->nbits );
     }
     else if( a ) {
+#ifdef M_DEBUG
+	b = mpi_is_secure(a)? mpi_debug_alloc_secure( a->nlimbs, info )
+			    : mpi_debug_alloc( a->nlimbs, info );
+#else
 	b = mpi_is_secure(a)? mpi_alloc_secure( a->nlimbs )
 			    : mpi_alloc( a->nlimbs );
+#endif
 	b->nlimbs = a->nlimbs;
 	b->sign = a->sign;
 	b->flags  = a->flags;
+	b->nbits = a->nbits;
 	for(i=0; i < b->nlimbs; i++ )
 	    b->d[i] = a->d[i];
     }
@@ -237,23 +382,32 @@ mpi_copy( MPI a )
  * with the same flags as A.
  */
 MPI
+#ifdef M_DEBUG
+mpi_debug_alloc_like( MPI a, const char *info )
+#else
 mpi_alloc_like( MPI a )
+#endif
 {
     MPI b;
 
     if( a && (a->flags & 4) ) {
-	int n = (a->sign+7)/8;
-	void *p = g10_is_secure(a->d)? g10_malloc_secure( n )
-				     : g10_malloc( n );
-	memcpy( p, a->d, n );
-	b = gcry_mpi_set_opaque( NULL, p, a->sign );
+	void *p = m_is_secure(a->d)? m_alloc_secure( a->nbits )
+				   : m_alloc( a->nbits );
+	memcpy( p, a->d, a->nbits );
+	b = mpi_set_opaque( NULL, p, a->nbits );
     }
     else if( a ) {
+#ifdef M_DEBUG
+	b = mpi_is_secure(a)? mpi_debug_alloc_secure( a->nlimbs, info )
+			    : mpi_debug_alloc( a->nlimbs, info );
+#else
 	b = mpi_is_secure(a)? mpi_alloc_secure( a->nlimbs )
 			    : mpi_alloc( a->nlimbs );
+#endif
 	b->nlimbs = 0;
 	b->sign = 0;
 	b->flags = a->flags;
+	b->nbits = 0;
     }
     else
 	b = NULL;
@@ -273,6 +427,7 @@ mpi_set( MPI w, MPI u)
     up = u->d;
     MPN_COPY( wp, up, usize );
     w->nlimbs = usize;
+    w->nbits = u->nbits;
     w->flags = u->flags;
     w->sign = usign;
 }
@@ -285,6 +440,7 @@ mpi_set_ui( MPI w, unsigned long u)
     w->d[0] = u;
     w->nlimbs = u? 1:0;
     w->sign = 0;
+    w->nbits = 0;
     w->flags = 0;
 }
 
@@ -292,7 +448,11 @@ mpi_set_ui( MPI w, unsigned long u)
 MPI
 mpi_alloc_set_ui( unsigned long u)
 {
+#ifdef M_DEBUG
+    MPI w = mpi_debug_alloc(1,"alloc_set_ui");
+#else
     MPI w = mpi_alloc(1);
+#endif
     w->d[0] = u;
     w->nlimbs = u? 1:0;
     w->sign = 0;
@@ -307,104 +467,3 @@ mpi_swap( MPI a, MPI b)
 
     tmp = *a; *a = *b; *b = tmp;
 }
-
-
-GCRY_MPI
-gcry_mpi_new( unsigned int nbits )
-{
-    return mpi_alloc( (nbits+BITS_PER_MPI_LIMB-1) / BITS_PER_MPI_LIMB );
-}
-
-
-GCRY_MPI
-gcry_mpi_snew( unsigned int nbits )
-{
-    return mpi_alloc_secure( (nbits+BITS_PER_MPI_LIMB-1) / BITS_PER_MPI_LIMB );
-}
-
-void
-gcry_mpi_release( GCRY_MPI a )
-{
-    mpi_free( a );
-}
-
-GCRY_MPI
-gcry_mpi_copy( const GCRY_MPI a )
-{
-    return mpi_copy( (GCRY_MPI)a );
-}
-
-GCRY_MPI
-gcry_mpi_set( GCRY_MPI w, const GCRY_MPI u )
-{
-    if( !w )
-	w = mpi_alloc( mpi_get_nlimbs(u) );
-    mpi_set( w, (GCRY_MPI)u );
-    return w;
-}
-
-GCRY_MPI
-gcry_mpi_set_ui( GCRY_MPI w, unsigned long u )
-{
-    if( !w )
-	w = mpi_alloc(1);
-    mpi_set_ui( w, u );
-    return w;
-}
-
-
-int
-gcry_mpi_cmp( const GCRY_MPI u, const GCRY_MPI v )
-{
-    return mpi_cmp( (GCRY_MPI)u, (GCRY_MPI)v );
-}
-
-int
-gcry_mpi_cmp_ui( const GCRY_MPI u, unsigned long v )
-{
-    return mpi_cmp_ui( (GCRY_MPI)u, v );
-}
-
-
-void
-gcry_mpi_randomize( GCRY_MPI w,
-		    unsigned int nbits, enum gcry_random_level level )
-{
-    char *p = mpi_is_secure(w) ? gcry_random_bytes( (nbits+7)/8, level )
-			       : gcry_random_bytes_secure( (nbits+7)/8, level );
-    mpi_set_buffer( w, p, (nbits+7)/8, 0 );
-    g10_free(p);
-}
-
-
-void
-gcry_mpi_set_flag( GCRY_MPI a, enum gcry_mpi_flag flag )
-{
-    switch( flag ) {
-      case GCRYMPI_FLAG_SECURE:  mpi_set_secure(a); break;
-      case GCRYMPI_FLAG_OPAQUE:
-      default: log_bug("invalid flag value\n");
-    }
-}
-
-void
-gcry_mpi_clear_flag( GCRY_MPI a, enum gcry_mpi_flag flag )
-{
-    switch( flag ) {
-      case GCRYMPI_FLAG_SECURE:
-      case GCRYMPI_FLAG_OPAQUE:
-      default: log_bug("invalid flag value\n");
-    }
-}
-
-int
-gcry_mpi_get_flag( GCRY_MPI a, enum gcry_mpi_flag flag )
-{
-    switch( flag ) {
-      case GCRYMPI_FLAG_SECURE: return (a->flags & 1);
-      case GCRYMPI_FLAG_OPAQUE: return (a->flags & 4);
-      default: log_bug("invalid flag value\n");
-    }
-}
-
-
