@@ -1,5 +1,5 @@
 /* http.c  -  HTTP protocol handler
- *	Copyright (C) 1999, 2001, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -42,16 +42,8 @@
 #include "util.h"
 #include "iobuf.h"
 #include "i18n.h"
-
 #include "http.h"
-
-#ifdef __riscos__
-#define HTTP_PROXY_ENV           "GnuPG$HttpProxy"
-#define HTTP_PROXY_ENV_PRINTABLE "<GnuPG$HttpProxy>"
-#else
-#define HTTP_PROXY_ENV           "http_proxy"
-#define HTTP_PROXY_ENV_PRINTABLE "$http_proxy"
-#endif
+#include "srv.h"
 
 #ifdef _WIN32
 #define sock_close(a)  closesocket(a)
@@ -76,11 +68,12 @@ static int remove_escapes( byte *string );
 static int insert_escapes( byte *buffer, const byte *string,
 					 const byte *special );
 static URI_TUPLE parse_tuple( byte *string );
-static int send_request( HTTP_HD hd );
+static int send_request( HTTP_HD hd, const char *proxy );
 static byte *build_rel_path( PARSED_URI uri );
 static int parse_response( HTTP_HD hd );
 
-static int connect_server( const char *server, ushort port );
+static int connect_server( const char *server, ushort port, unsigned int flags,
+			   const char *srvtag );
 static int write_server( int sock, const char *data, size_t length );
 
 #ifdef _WIN32
@@ -115,10 +108,44 @@ init_sockets (void)
 }
 #endif /*_WIN32*/
 
+static byte bintoasc[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+			 "abcdefghijklmnopqrstuvwxyz"
+			 "0123456789+/";
+
+/****************
+ * create a radix64 encoded string.
+ */
+
+/* TODO: This is a duplicate of code in g10/armor.c.  Better to use a
+   single copy in strgutil.c */
+static char *
+make_radix64_string( const byte *data, size_t len )
+{
+    char *buffer, *p;
+
+    buffer = p = m_alloc( (len+2)/3*4 + 1 );
+    for( ; len >= 3 ; len -= 3, data += 3 ) {
+	*p++ = bintoasc[(data[0] >> 2) & 077];
+	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
+	*p++ = bintoasc[(((data[1]<<2)&074)|((data[2]>>6)&03))&077];
+	*p++ = bintoasc[data[2]&077];
+    }
+    if( len == 2 ) {
+	*p++ = bintoasc[(data[0] >> 2) & 077];
+	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
+	*p++ = bintoasc[((data[1]<<2)&074)];
+    }
+    else if( len == 1 ) {
+	*p++ = bintoasc[(data[0] >> 2) & 077];
+	*p++ = bintoasc[(data[0] <<4)&060];
+    }
+    *p = 0;
+    return buffer;
+}
 
 int
 http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
-					      unsigned int flags )
+	   unsigned int flags, const char *proxy )
 {
     int rc;
 
@@ -134,7 +161,7 @@ http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
 
     rc = parse_uri( &hd->uri, url );
     if( !rc ) {
-	rc = send_request( hd );
+	rc = send_request( hd, proxy );
 	if( !rc ) {
 	    hd->fp_write = iobuf_sockopen( hd->sock , "w" );
 	    if( hd->fp_write )
@@ -197,11 +224,12 @@ http_wait_response( HTTP_HD hd, unsigned int *ret_status )
 
 
 int
-http_open_document( HTTP_HD hd, const char *document, unsigned int flags )
+http_open_document( HTTP_HD hd, const char *document,
+		    unsigned int flags, const char *proxy )
 {
     int rc;
 
-    rc = http_open( hd, HTTP_REQ_GET, document, flags );
+    rc = http_open( hd, HTTP_REQ_GET, document, flags, proxy );
     if( rc )
 	return rc;
 
@@ -213,8 +241,6 @@ http_open_document( HTTP_HD hd, const char *document, unsigned int flags )
 }
 
 
-
-
 void
 http_close( HTTP_HD hd )
 {
@@ -284,13 +310,12 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	*p2++ = 0;
 	strlwr( p );
 	uri->scheme = p;
-        uri->port = 80;
-	if( !strcmp( uri->scheme, "http" ) )
-	    ;
-	else if( !strcmp( uri->scheme, "x-hkp" ) ) /* same as HTTP */
-	    uri->port = 11371;
+	if(strcmp(uri->scheme,"http")==0)
+	  uri->port = 80;
+	else if(strcmp(uri->scheme,"hkp")==0)
+	  uri->port = 11371;
 	else
-	    return G10ERR_INVALID_URI; /* Unsupported scheme */
+	  return G10ERR_INVALID_URI; /* Unsupported scheme */
 
 	p = p2;
 
@@ -303,6 +328,15 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	    p++;
 	    if( (p2 = strchr(p, '/')) )
 		*p2++ = 0;
+
+	    /* Check for username/password encoding */
+	    if((p3=strchr(p,'@')))
+	      {
+		uri->auth=p;
+		*p3++='\0';
+		p=p3;
+	      }
+
 	    strlwr( p );
 	    uri->host = p;
 	    if( (p3=strchr( p, ':' )) ) {
@@ -420,7 +454,7 @@ insert_escapes( byte *buffer, const byte *string, const byte *special )
 	}
 	else {
 	    if( buffer ) {
-		sprintf( buffer, "%02X", *string );
+		sprintf( buffer, "%%%02X", *string );
 		buffer += 3;
 	    }
 	    n += 3;
@@ -430,9 +464,6 @@ insert_escapes( byte *buffer, const byte *string, const byte *special )
 }
 
 
-
-
-
 static URI_TUPLE
 parse_tuple( byte *string )
 {
@@ -471,58 +502,84 @@ parse_tuple( byte *string )
  * Returns 0 if the request was successful
  */
 static int
-send_request( HTTP_HD hd )
+send_request( HTTP_HD hd, const char *proxy )
 {
     const byte *server;
     byte *request, *p;
     ushort port;
     int rc;
-    const char *http_proxy = NULL;
+    char *auth=NULL;
 
     server = *hd->uri->host? hd->uri->host : "localhost";
     port   = hd->uri->port?  hd->uri->port : 80;
 
-    if( (hd->flags & HTTP_FLAG_TRY_PROXY)
-	&& (http_proxy = getenv( HTTP_PROXY_ENV )) ) {
+    if(proxy)
+      {
 	PARSED_URI uri;
 
-	rc = parse_uri( &uri, http_proxy );
-	if (rc) {
-	    log_error("invalid " HTTP_PROXY_ENV_PRINTABLE ": %s\n",
-                      g10_errstr(rc));
+	rc = parse_uri( &uri, proxy );
+	if (rc)
+	  {
+	    log_error("invalid HTTP proxy (%s): %s\n",proxy,g10_errstr(rc));
 	    release_parsed_uri( uri );
 	    return G10ERR_NETWORK;
-	}
+	  }
 	hd->sock = connect_server( *uri->host? uri->host : "localhost",
-				    uri->port? uri->port : 80 );
+				   uri->port? uri->port : 80, 0, NULL );
+	if(uri->auth)
+	  {
+	    char *x=make_radix64_string(uri->auth,strlen(uri->auth));
+	    auth=m_alloc(50+strlen(x));
+	    sprintf(auth,"Proxy-Authorization: Basic %s\r\n",x);
+	    m_free(x);
+	  }
+
 	release_parsed_uri( uri );
-    }
+      }
     else
-	hd->sock = connect_server( server, port );
+      {
+	hd->sock = connect_server( server, port, hd->flags, hd->uri->scheme );
+	if(hd->uri->auth)
+	  {
+	    char *x=make_radix64_string(hd->uri->auth,strlen(hd->uri->auth));
+	    auth=m_alloc(50+strlen(x));
+	    sprintf(auth,"Authorization: Basic %s\r\n",x);
+	    m_free(x);
+	  }
+      }
 
     if( hd->sock == -1 )
 	return G10ERR_NETWORK;
 
     p = build_rel_path( hd->uri );
-    request = m_alloc( strlen(server)*2 + strlen(p) + 50 );
-    if( http_proxy ) {
-	sprintf( request, "%s http://%s:%hu%s%s HTTP/1.0\r\n",
-			  hd->req_type == HTTP_REQ_GET ? "GET" :
-			  hd->req_type == HTTP_REQ_HEAD? "HEAD":
-			  hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
-			  server, port,  *p == '/'? "":"/", p );
-    }
-    else {
-	sprintf( request, "%s %s%s HTTP/1.0\r\nHost: %s\r\n",
-			  hd->req_type == HTTP_REQ_GET ? "GET" :
-			  hd->req_type == HTTP_REQ_HEAD? "HEAD":
-			  hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
-						 *p == '/'? "":"/", p, server);
-    }
+
+    request=m_alloc(strlen(server)*2 + strlen(p) + (auth?strlen(auth):0) + 65);
+    if( proxy )
+      sprintf( request, "%s http://%s:%hu%s%s HTTP/1.0\r\n%s",
+	       hd->req_type == HTTP_REQ_GET ? "GET" :
+	       hd->req_type == HTTP_REQ_HEAD? "HEAD":
+	       hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
+	       server, port,  *p == '/'? "":"/", p, auth?auth:"" );
+    else
+      {
+	char portstr[15];
+
+	if(port!=80)
+	  sprintf(portstr,":%u",port);
+
+	sprintf( request, "%s %s%s HTTP/1.0\r\nHost: %s%s\r\n%s",
+		 hd->req_type == HTTP_REQ_GET ? "GET" :
+		 hd->req_type == HTTP_REQ_HEAD? "HEAD":
+		 hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
+		 *p == '/'? "":"/", p, server, (port!=80)?portstr:"",
+		 auth?auth:"");
+      }
+
     m_free(p);
 
     rc = write_server( hd->sock, request, strlen(request) );
     m_free( request );
+    m_free(auth);
 
     return rc;
 }
@@ -632,7 +689,7 @@ parse_response( HTTP_HD hd )
     return 0;
 }
 
-#if 0
+#ifdef TEST
 static int
 start_server()
 {
@@ -705,74 +762,193 @@ start_server()
 #endif
 
 
-
 static int
-connect_server( const char *server, ushort port )
+connect_server( const char *server, ushort port, unsigned int flags,
+		const char *srvtag )
 {
-    int sd;
+  int sock=-1,srv,srvcount=0,connected=0,hostfound=0;
+  struct srventry *srvlist=NULL;
+
 #ifdef _WIN32
-    struct hostent *hp;
-    struct sockaddr_in ad;
-    unsigned long l;
-    
-    init_sockets ();
+  unsigned long inaddr;
 
-    memset (&ad, 0, sizeof(ad));
-    ad.sin_family = AF_INET;
-    ad.sin_port = htons(port);
+  init_sockets();
+  /* Win32 gethostbyname doesn't handle IP addresses internally, so we
+     try inet_addr first on that platform only. */
+  if((inaddr=inet_addr(server))!=INADDR_NONE)
+    {
+      struct sockaddr_in addr;
 
-    if( (l = inet_addr (server)) != SOCKET_ERROR ) {
-        memcpy (&ad.sin_addr, &l, sizeof(l));
-    }
-    else if( (hp = gethostbyname (server)) ) {
-        if( hp->h_addrtype != AF_INET ) {
-            log_error ("%s: unknown address family\n", server);
-            return -1;
-        }
-        if ( hp->h_length != 4 ) {
-            log_error ("%s: illegal address length\n", server);
-            return -1;
-        }
-        memcpy (&ad.sin_addr, hp->h_addr, hp->h_length);
-    }
-    else {
-        log_error ("%s: host not found: ec=%d\n",
-                   server, (int)WSAGetLastError ());
-        return -1;
-    }
+      memset(&addr,0,sizeof(addr));
 
-    if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
-        log_error ("error creating socket: ex=%d\n", 
-                   (int)WSAGetLastError ());
-        return -1;
-    }
+      if((sock=socket(AF_INET,SOCK_STREAM,0))==INVALID_SOCKET)
+	{
+	  log_error("error creating socket: ec=%d\n",(int)WSAGetLastError());
+	  return -1;
+	}
 
-    if( connect (sd, (struct sockaddr *)&ad, sizeof (ad) ) ) {
-        sock_close (sd);
-        return -1;
-    }
-#else
-    struct sockaddr_in addr;
-    struct hostent *host;
+      addr.sin_family=AF_INET; 
+      addr.sin_port=htons(port);
+      memcpy(&addr.sin_addr,&inaddr,sizeof(inaddr));      
 
-    addr.sin_family = AF_INET;
-    addr.sin_port = htons(port);
-    host = gethostbyname((char*)server);
-    if( !host )
-	 return -1;
-
-    addr.sin_addr = *(struct in_addr*)host->h_addr;
-
-    sd = socket(AF_INET, SOCK_STREAM, 0);
-    if( sd == -1 )
-	return -1;
-
-    if( connect( sd, (struct sockaddr *)&addr, sizeof addr) == -1 ) {
-	sock_close(sd);
-	return -1;
+      if(connect(sock,(struct sockaddr *)&addr,sizeof(addr))==0)
+	return sock;
+      else
+	{
+	  sock_close(sock);
+	  return -1;
+	}
     }
 #endif
-    return sd;
+
+#ifdef USE_DNS_SRV
+  /* Do the SRV thing */
+  if(flags&HTTP_FLAG_TRY_SRV && srvtag)
+    {
+      /* We're using SRV, so append the tags */
+      if(1+strlen(srvtag)+6+strlen(server)+1<=MAXDNAME)
+	{
+	  char srvname[MAXDNAME];
+
+	  strcpy(srvname,"_");
+	  strcat(srvname,srvtag);
+	  strcat(srvname,"._tcp.");
+	  strcat(srvname,server);
+	  srvcount=getsrv(srvname,&srvlist);
+	}
+    }
+#endif
+
+  if(srvlist==NULL)
+    {
+      /* Either we're not using SRV, or the SRV lookup failed.  Make
+	 up a fake SRV record. */
+      srvlist=m_alloc_clear(sizeof(struct srventry));
+      srvlist->port=port;
+      strncpy(srvlist->target,server,MAXDNAME);
+      srvlist->target[MAXDNAME-1]='\0';
+      srvcount=1;
+    }
+
+#ifdef HAVE_GETADDRINFO
+
+  for(srv=0;srv<srvcount;srv++)
+    {
+      struct addrinfo hints,*res,*ai;
+      char portstr[6];
+
+      sprintf(portstr,"%u",srvlist[srv].port);
+      memset(&hints,0,sizeof(hints));
+      hints.ai_socktype=SOCK_STREAM;
+      if(getaddrinfo(srvlist[srv].target,portstr,&hints,&res)==0)
+	hostfound=1;
+      else
+	continue;
+
+      for(ai=res;ai;ai=ai->ai_next)
+	{
+	  if((sock=socket(ai->ai_family,ai->ai_socktype,ai->ai_protocol))==-1)
+	    {
+	      log_error("error creating socket: %s\n",strerror(errno));
+	      freeaddrinfo(res);
+	      return -1;
+	    }
+
+	  if(connect(sock,ai->ai_addr,ai->ai_addrlen)==0)
+	    {
+	      connected=1;
+	      break;
+	    }
+
+	  sock_close(sock);
+	}
+
+      freeaddrinfo(res);
+
+      if(ai)
+	break;
+    }
+
+#else /* !HAVE_GETADDRINFO */
+
+  for(srv=0;srv<srvcount;srv++)
+    {
+      int i=0;
+      struct hostent *host=NULL;
+      struct sockaddr_in addr;
+
+      memset(&addr,0,sizeof(addr));
+
+      if((host=gethostbyname(srvlist[srv].target))==NULL)
+	continue;
+
+      hostfound=1;
+
+      if((sock=socket(host->h_addrtype,SOCK_STREAM,0))==-1)
+	{
+	  log_error("error creating socket: %s\n",strerror(errno));
+	  return -1;
+	}
+
+      addr.sin_family=host->h_addrtype;
+      if(addr.sin_family!=AF_INET)
+	{
+	  log_error("%s: unknown address family\n",srvlist[srv].target);
+	  return -1;
+	}
+
+      addr.sin_port=htons(srvlist[srv].port);
+
+      /* Try all A records until one responds. */
+      while(host->h_addr_list[i])
+	{
+	  if(host->h_length!=4)
+	    {
+	      log_error("%s: illegal address length\n",srvlist[srv].target);
+	      return -1;
+	    }
+
+	  memcpy(&addr.sin_addr,host->h_addr_list[i],host->h_length);
+
+	  if(connect(sock,(struct sockaddr *)&addr,sizeof(addr))==0)
+	    {
+	      connected=1;
+	      break;
+	    }
+
+	  i++;
+	}
+
+      if(host->h_addr_list[i])
+	break;
+
+      sock_close(sock);
+    }
+#endif /* !HAVE_GETADDRINFO */
+
+  m_free(srvlist);
+
+  if(!connected)
+    {
+      int err=errno;
+#ifdef _WIN32
+      if(hostfound)
+	log_error("%s: Unable to connect: ec=%d\n",server,(int)WSAGetLastError());
+      else
+	log_error("%s: Host not found: ec=%d\n",server,(int)WSAGetLastError());
+#else
+      if(hostfound)
+	log_error("%s: %s\n",server,strerror(err));
+      else
+	log_error("%s: Host not found\n",server);
+#endif
+      if(sock!=-1)
+	sock_close(sock);
+      errno=err;
+      return -1;
+    }
+
+  return sock;
 }
 
 
@@ -864,7 +1040,7 @@ main(int argc, char **argv)
     }
     release_parsed_uri( uri ); uri = NULL;
 
-    rc = http_open_document( &hd, *argv, 0 );
+    rc = http_open_document( &hd, *argv, 0, NULL );
     if( rc ) {
 	log_error("can't get `%s': %s\n", *argv, g10_errstr(rc));
 	return 1;
