@@ -1,5 +1,6 @@
 /* miscutil.c -  miscellaneous utilities
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2003,
+ *               2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,7 +26,7 @@
 #include <time.h>
 #include <ctype.h>
 #ifdef HAVE_LANGINFO_H
-  #include <langinfo.h>
+#include <langinfo.h>
 #endif
 #include "types.h"
 #include "util.h"
@@ -56,11 +57,11 @@ scan_isodatestr( const char *string )
     if( strlen(string) != 10 || string[4] != '-' || string[7] != '-' )
 	return 0;
     for( i=0; i < 4; i++ )
-	if( !isdigit(string[i]) )
+	if( !digitp(string+i) )
 	    return 0;
-    if( !isdigit(string[5]) || !isdigit(string[6]) )
+    if( !digitp(string+5) || !digitp(string+6) )
 	return 0;
-    if( !isdigit(string[8]) || !isdigit(string[9]) )
+    if( !digitp(string+8) || !digitp(string+9) )
 	return 0;
     year = atoi(string);
     month = atoi(string+5);
@@ -136,6 +137,29 @@ strtimestamp( u32 stamp )
     return buffer;
 }
 
+
+/****************
+ * Note: this function returns GMT
+ */
+const char *
+isotimestamp (u32 stamp)
+{
+    static char buffer[25+5];
+    struct tm *tp;
+    time_t atime = stamp;
+    
+    if (atime < 0) {
+        strcpy (buffer, "????" "-??" "-??" " " "??" ":" "??" ":" "??");
+    }
+    else {
+        tp = gmtime( &atime );
+        sprintf(buffer,"%04d-%02d-%02d %02d:%02d:%02d",
+                1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday,
+                tp->tm_hour, tp->tm_min, tp->tm_sec);
+    }
+    return buffer;
+}
+
 /****************
  * Note: this function returns local time
  */
@@ -143,9 +167,9 @@ const char *
 asctimestamp( u32 stamp )
 {
     static char buffer[50];
-    #if defined (HAVE_STRFTIME) && defined (HAVE_NL_LANGINFO)
+#if defined (HAVE_STRFTIME) && defined (HAVE_NL_LANGINFO)
       static char fmt[50];
-    #endif
+#endif
     struct tm *tp;
     time_t atime = stamp;
 
@@ -155,35 +179,44 @@ asctimestamp( u32 stamp )
     }
 
     tp = localtime( &atime );
-  #ifdef HAVE_STRFTIME
-    #if defined(HAVE_NL_LANGINFO)
+#ifdef HAVE_STRFTIME
+#if defined(HAVE_NL_LANGINFO)
       mem2str( fmt, nl_langinfo(D_T_FMT), DIM(fmt)-3 );
       if( strstr( fmt, "%Z" ) == NULL )
 	strcat( fmt, " %Z");
       strftime( buffer, DIM(buffer)-1, fmt, tp );
-    #else
+#else
       /* fixme: we should check whether the locale appends a " %Z"
        * These locales from glibc don't put the " %Z":
        * fi_FI hr_HR ja_JP lt_LT lv_LV POSIX ru_RU ru_SU sv_FI sv_SE zh_CN
        */
-      strftime( buffer, DIM(buffer)-1, "%c %Z", tp );
-    #endif
+      strftime( buffer, DIM(buffer)-1, 
+#ifdef HAVE_W32_SYSTEM                
+                "%c"
+#else
+                "%c %Z"
+#endif
+                , tp );
+#endif
     buffer[DIM(buffer)-1] = 0;
-  #else
+#else
     mem2str( buffer, asctime(tp), DIM(buffer) );
-  #endif
+#endif
     return buffer;
 }
 
+
 /****************
  * Print a string to FP, but filter all control characters out.
  */
 void
-print_string( FILE *fp, const byte *p, size_t n, int delim )
+print_string2( FILE *fp, const byte *p, size_t n, int delim, int delim2 )
 {
     for( ; n; n--, p++ )
-	if( *p < 0x20 || (*p >= 0x7f && *p < 0xa0) || *p == delim ||
-	    (delim && *p=='\\')) {
+	if( *p < 0x20 || (*p >= 0x7f && *p < 0xa0)
+	    || *p == delim || *p == delim2
+	    || ((delim || delim2) && *p=='\\'))
+	  {
 	    putc('\\', fp);
 	    if( *p == '\n' )
 		putc('n', fp);
@@ -199,9 +232,15 @@ print_string( FILE *fp, const byte *p, size_t n, int delim )
 		putc('0', fp);
 	    else
 		fprintf(fp, "x%02x", *p );
-	}
+	  }
 	else
-	    putc(*p, fp);
+	  putc(*p, fp);
+}
+
+void
+print_string( FILE *fp, const byte *p, size_t n, int delim )
+{
+  print_string2(fp,p,n,delim,0);
 }
 
 /****************
@@ -293,18 +332,20 @@ make_printable_string( const byte *p, size_t n, int delim )
 int
 answer_is_yes_no_default( const char *s, int def_answer )
 {
+    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
     const char *short_yes = _("yY");
+    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
     const char *short_no = _("nN");
 
     /* Note: we have to use the local dependent strcasecmp here */
-    if( !strcasecmp(s, long_yes ) )
+    if( match_multistr(long_yes,s) )
 	return 1;
     if( *s && strchr( short_yes, *s ) && !s[1] )
 	return 1;
     /* test for no strings to catch ambiguities for the next test */
-    if( !strcasecmp(s, long_no ) )
+    if( match_multistr(long_no,s) )
 	return 0;
     if( *s && strchr( short_no, *s ) && !s[1] )
 	return 0;
@@ -328,19 +369,21 @@ answer_is_yes( const char *s )
 int
 answer_is_yes_no_quit( const char *s )
 {
+    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
+    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
+    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
     const char *long_quit = _("quit");
     const char *short_yes = _("yY");
     const char *short_no = _("nN");
     const char *short_quit = _("qQ");
 
-    /* Note: We have to use the locale dependent strcasecmp */
-    if( !strcasecmp(s, long_no ) )
+    if( match_multistr(long_no,s) )
 	return 0;
-    if( !strcasecmp(s, long_yes ) )
+    if( match_multistr(long_yes,s) )
 	return 1;
-    if( !strcasecmp(s, long_quit ) )
+    if( match_multistr(long_quit,s) )
 	return -1;
     if( *s && strchr( short_no, *s ) && !s[1] )
 	return 0;
@@ -359,3 +402,85 @@ answer_is_yes_no_quit( const char *s )
 	return -1;
     return 0;
 }
+
+/*
+   Return 1 for okay, 0 for for cancel or DEF_ANSWER for default. 
+ */
+int
+answer_is_okay_cancel (const char *s, int def_answer)
+{
+  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  const char *long_okay = _("okay|okay");
+  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  const char *long_cancel = _("cancel|cancel");
+  const char *short_okay = _("oO");
+  const char *short_cancel = _("cC");
+  
+  /* Note: We have to use the locale dependent strcasecmp */
+  if ( match_multistr(long_okay,s) )
+    return 1;
+  if ( match_multistr(long_cancel,s) )
+    return 0;
+  if ( *s && strchr( short_okay, *s ) && !s[1] )
+    return 1;
+  if ( *s && strchr( short_cancel, *s ) && !s[1] )
+    return 0;
+  /* Always test for the English values (not locale here) */
+  if ( !ascii_strcasecmp(s, "okay" ) )
+    return 1;
+  if ( !ascii_strcasecmp(s, "ok" ) )
+    return 1;
+  if ( !ascii_strcasecmp(s, "cancel" ) )
+    return 0;
+  if ( *s && strchr( "oO", *s ) && !s[1] )
+    return 1;
+  if ( *s && strchr( "cC", *s ) && !s[1] )
+    return 0;
+  return def_answer;
+}
+
+/* Try match against each substring of multistr, delimited by | */
+int
+match_multistr(const char *multistr,const char *match)
+{
+  do
+    {
+      size_t seglen=strcspn(multistr,"|");
+      if(!seglen)
+	break;
+      /* Using the localized strncasecmp */
+      if(strncasecmp(multistr,match,seglen)==0)
+	return 1;
+      multistr+=seglen;
+      if(*multistr=='|')
+	multistr++;
+    }
+  while(*multistr);
+
+  return 0;
+}
+
+int
+hextobyte( const char *s )
+{
+    int c;
+
+    if( *s >= '0' && *s <= '9' )
+	c = 16 * (*s - '0');
+    else if( *s >= 'A' && *s <= 'F' )
+	c = 16 * (10 + *s - 'A');
+    else if( *s >= 'a' && *s <= 'f' )
+	c = 16 * (10 + *s - 'a');
+    else
+	return -1;
+    s++;
+    if( *s >= '0' && *s <= '9' )
+	c += *s - '0';
+    else if( *s >= 'A' && *s <= 'F' )
+	c += 10 + *s - 'A';
+    else if( *s >= 'a' && *s <= 'f' )
+	c += 10 + *s - 'a';
+    else
+	return -1;
+    return c;
+}
