@@ -1,5 +1,5 @@
 /* secmem.c  -	memory allocation from a secure heap
- *	Copyright (C) 1998,1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,12 +26,15 @@
 #include <stdarg.h>
 #include <unistd.h>
 #if defined(HAVE_MLOCK) || defined(HAVE_MMAP)
-  #include <sys/mman.h>
-  #include <sys/types.h>
-  #include <fcntl.h>
-  #ifdef USE_CAPABILITIES
-    #include <sys/capability.h>
-  #endif
+#include <sys/mman.h>
+#include <sys/types.h>
+#include <fcntl.h>
+#ifdef USE_CAPABILITIES
+#include <sys/capability.h>
+#endif
+#ifdef HAVE_PLOCK
+#include <sys/lock.h>
+#endif
 #endif
 
 #include "types.h"
@@ -40,8 +43,13 @@
 #include "i18n.h"
 
 #if defined(MAP_ANON) && !defined(MAP_ANONYMOUS)
-  #define MAP_ANONYMOUS MAP_ANON
+#define MAP_ANONYMOUS MAP_ANON
 #endif
+/* It seems that Slackware 7.1 does not know about EPERM */
+#if !defined(EPERM) && defined(ENOMEM)
+#define EPERM  ENOMEM
+#endif
+
 
 #define DEFAULT_POOLSIZE 16384
 
@@ -58,7 +66,9 @@ struct memblock_struct {
 
 static void  *pool;
 static volatile int pool_okay; /* may be checked in an atexit function */
-static int   pool_is_mmapped;
+#ifdef HAVE_MMAP
+static volatile int pool_is_mmapped;
+#endif
 static size_t poolsize; /* allocated length */
 static size_t poollen;	/* used length */
 static MEMBLOCK *unused_blocks;
@@ -75,15 +85,19 @@ static int suspend_warning;
 static void
 print_warn(void)
 {
-    if( !no_warning )
-	log_info(_("Warning: using insecure memory!\n"));
+  if (!no_warning)
+    {
+      log_info(_("WARNING: using insecure memory!\n"));
+      log_info(_("please see http://www.gnupg.org/faq.html"
+		 " for more information\n"));
+    }
 }
 
 
 static void
 lock_pool( void *p, size_t n )
 {
-  #if defined(USE_CAPABILITIES) && defined(HAVE_MLOCK)
+#if defined(USE_CAPABILITIES) && defined(HAVE_MLOCK)
     int err;
 
     cap_set_proc( cap_from_text("cap_ipc_lock+ep") );
@@ -94,24 +108,46 @@ lock_pool( void *p, size_t n )
 
     if( err ) {
 	if( errno != EPERM
-	  #ifdef EAGAIN  /* OpenBSD returns this */
+#ifdef EAGAIN  /* OpenBSD returns this */
 	    && errno != EAGAIN
-	  #endif
-	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
+#endif
+#ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
 	    && errno != ENOSYS
-	  #endif
+#endif
+#ifdef ENOMEM  /* Linux can return this */
+            && errno != ENOMEM
+#endif
 	  )
 	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
     }
 
-  #elif defined(HAVE_MLOCK)
+#elif defined(HAVE_MLOCK)
     uid_t uid;
     int err;
 
     uid = getuid();
 
-  #ifdef HAVE_BROKEN_MLOCK
+#ifdef HAVE_BROKEN_MLOCK
+    /* ick. but at least we get secured memory. about to lock
+       entire data segment. */
+#ifdef HAVE_PLOCK
+# ifdef _AIX
+    /* The configure for AIX returns broken mlock but the plock has
+       the strange requirement to somehow set the stack limit first.
+       The problem might turn out in indeterministic program behaviour
+       and hanging processes which can somehow be solved when enough
+       processes are clogging up the memory.  To get this problem out
+       of the way we simply don't try to lock the memory at all.
+       */    
+    errno = EPERM;
+    err = errno;
+# else /* !_AIX */
+    err = plock( DATLOCK );
+    if( err && errno )
+        err = errno;
+# endif /*_AIX*/
+#else /*!HAVE_PLOCK*/
     if( uid ) {
 	errno = EPERM;
 	err = errno;
@@ -121,33 +157,54 @@ lock_pool( void *p, size_t n )
 	if( err && errno )
 	    err = errno;
     }
-  #else
+#endif /*!HAVE_PLOCK*/
+#else
     err = mlock( p, n );
     if( err && errno )
 	err = errno;
-  #endif
+#endif
 
     if( uid && !geteuid() ) {
-	if( setuid( uid ) || getuid() != geteuid()  )
+	/* check that we really dropped the privs.
+	 * Note: setuid(0) should always fail */
+	if( setuid( uid ) || getuid() != geteuid() || !setuid(0) )
 	    log_fatal("failed to reset uid: %s\n", strerror(errno));
     }
 
     if( err ) {
 	if( errno != EPERM
-	  #ifdef EAGAIN  /* OpenBSD returns this */
+#ifdef EAGAIN  /* OpenBSD returns this */
 	    && errno != EAGAIN
-	  #endif
-	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
+#endif
+#ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
 	    && errno != ENOSYS
-	  #endif
+#endif
+#ifdef ENOMEM  /* Linux can return this */
+            && errno != ENOMEM
+#endif
 	  )
 	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
     }
 
-  #else
+#elif defined ( __QNX__ )
+    /* QNX does not page at all, so the whole secure memory stuff does
+     * not make much sense.  However it is still of use because it
+     * wipes out the memory on a free().
+     * Therefore it is sufficient to suppress the warning
+     */
+#elif defined (HAVE_DOSISH_SYSTEM) || defined (__CYGWIN__)
+    /* It does not make sense to print such a warning, given the fact that 
+     * this whole Windows !@#$% and their user base are inherently insecure
+     */
+#elif defined (__riscos__)
+    /* no virtual memory on RISC OS, so no pages are swapped to disc,
+     * besides we don't have mmap, so we don't use it! ;-)
+     * But don't complain, as explained above.
+     */
+#else
     log_info("Please note that you don't have secure memory on this system\n");
-  #endif
+#endif
 }
 
 
@@ -161,18 +218,18 @@ init_pool( size_t n)
     if( disable_secmem )
 	log_bug("secure memory is disabled");
 
-  #ifdef HAVE_GETPAGESIZE
+#ifdef HAVE_GETPAGESIZE
     pgsize = getpagesize();
-  #else
+#else
     pgsize = 4096;
-  #endif
+#endif
 
-  #if HAVE_MMAP
+#ifdef HAVE_MMAP
     poolsize = (poolsize + pgsize -1 ) & ~(pgsize-1);
-    #ifdef MAP_ANONYMOUS
+#ifdef MAP_ANONYMOUS
        pool = mmap( 0, poolsize, PROT_READ|PROT_WRITE,
 				 MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
-    #else /* map /dev/zero instead */
+#else /* map /dev/zero instead */
     {	int fd;
 
 	fd = open("/dev/zero", O_RDWR);
@@ -185,7 +242,7 @@ init_pool( size_t n)
 				      MAP_PRIVATE, fd, 0);
 	}
     }
-    #endif
+#endif
     if( pool == (void*)-1 )
 	log_info("can't mmap pool of %u bytes: %s - using malloc\n",
 			    (unsigned)poolsize, strerror(errno));
@@ -194,7 +251,7 @@ init_pool( size_t n)
 	pool_okay = 1;
     }
 
-  #endif
+#endif
     if( !pool_okay ) {
 	pool = malloc( poolsize );
 	if( !pool )
@@ -240,24 +297,27 @@ secmem_get_flags(void)
     return flags;
 }
 
-void
+/* Returns 1 if memory was locked, 0 if not. */
+int
 secmem_init( size_t n )
 {
     if( !n ) {
-      #ifdef USE_CAPABILITIES
+#ifndef __riscos__
+#ifdef USE_CAPABILITIES
 	/* drop all capabilities */
 	cap_set_proc( cap_from_text("all-eip") );
 
-      #elif !defined(HAVE_DOSISH_SYSTEM)
+#elif !defined(HAVE_DOSISH_SYSTEM)
 	uid_t uid;
 
 	disable_secmem=1;
 	uid = getuid();
 	if( uid != geteuid() ) {
-	    if( setuid( uid ) || getuid() != geteuid() )
+	    if( setuid( uid ) || getuid() != geteuid() || !setuid(0) )
 		log_fatal("failed to drop setuid\n" );
 	}
-      #endif
+#endif
+#endif /* !__riscos__ */
     }
     else {
 	if( n < DEFAULT_POOLSIZE )
@@ -267,6 +327,8 @@ secmem_init( size_t n )
 	else
 	    log_error("Oops, secure memory pool already initialized\n");
     }
+
+    return !show_warning;
 }
 
 
@@ -287,7 +349,10 @@ secmem_malloc( size_t size )
 	print_warn();
     }
 
-    /* blocks are always a multiple of 32 */
+    /* Blocks are always a multiple of 32.  Note that we allocate an
+       extra of the size of an entire MEMBLOCK.  This is required
+       becuase we do not only need the SIZE info but also extra space
+       to chain up unused memory blocks.  */
     size += sizeof(MEMBLOCK);
     size = ((size + 31) / 32) * 32;
 
@@ -336,12 +401,18 @@ secmem_realloc( void *p, size_t newsize )
 
     mb = (MEMBLOCK*)((char*)p - ((size_t) &((MEMBLOCK*)0)->u.aligned.c));
     size = mb->size;
-    if( newsize < size )
-	return p; /* it is easier not to shrink the memory */
+    if (size < sizeof(MEMBLOCK))
+      log_bug ("secure memory corrupted at block %p\n", mb);
+    size -= ((size_t) &((MEMBLOCK*)0)->u.aligned.c);
+
+    if( newsize <= size )
+	return p; /* It is easier not to shrink the memory.  */
     a = secmem_malloc( newsize );
-    memcpy(a, p, size);
-    memset((char*)a+size, 0, newsize-size);
-    secmem_free(p);
+    if ( a ) {
+        memcpy(a, p, size);
+        memset((char*)a+size, 0, newsize-size);
+        secmem_free(p);
+    }
     return a;
 }
 
@@ -359,10 +430,10 @@ secmem_free( void *a )
     size = mb->size;
     /* This does not make much sense: probably this memory is held in the
      * cache. We do it anyway: */
-    memset(mb, 0xff, size );
-    memset(mb, 0xaa, size );
-    memset(mb, 0x55, size );
-    memset(mb, 0x00, size );
+    wipememory2(mb, 0xff, size );
+    wipememory2(mb, 0xaa, size );
+    wipememory2(mb, 0x55, size );
+    wipememory2(mb, 0x00, size );
     mb->size = size;
     mb->u.next = unused_blocks;
     unused_blocks = mb;
@@ -380,7 +451,7 @@ m_is_secure( const void *p )
 
 /****************
  * Warning:  This code might be called by an interrupt handler
- *	     and frankly, thre should really be such a handler,
+ *	     and frankly, there should really be such a handler,
  *	     to make sure that the memory is wiped out.
  *	     We hope that the OS wipes out mlocked memory after
  *	     receiving a SIGKILL - it really should do so, otherwise
@@ -392,14 +463,14 @@ secmem_term()
     if( !pool_okay )
 	return;
 
-    memset( pool, 0xff, poolsize);
-    memset( pool, 0xaa, poolsize);
-    memset( pool, 0x55, poolsize);
-    memset( pool, 0x00, poolsize);
-  #if HAVE_MMAP
+    wipememory2( pool, 0xff, poolsize);
+    wipememory2( pool, 0xaa, poolsize);
+    wipememory2( pool, 0x55, poolsize);
+    wipememory2( pool, 0x00, poolsize);
+#ifdef HAVE_MMAP
     if( pool_is_mmapped )
 	munmap( pool, poolsize );
-  #endif
+#endif
     pool = NULL;
     pool_okay = 0;
     poolsize=0;
@@ -418,4 +489,3 @@ secmem_dump_stats()
 		cur_alloced, max_alloced, cur_blocks, max_blocks,
 		(ulong)poollen, (ulong)poolsize );
 }
-
